@@ -23,11 +23,14 @@
 #include <QString>
 #include <QStringList>
 #include <QDebug>
+#include <QRegularExpression>
 
 #ifdef HAVE_KF5
 #include <KSharedConfig>
 #include <KConfigGroup>
 #include <KLocalizedString>
+#else // HAVE_KF5
+#define i18n(text) QObject::tr(text)
 #endif // HAVE_KF5
 
 #include "file.h"
@@ -40,17 +43,17 @@ uint qHash(const QSharedPointer<ValueItem> &valueItem)
     return qHash(valueItem->id());
 }
 
-const QRegExp ValueItem::ignoredInSorting = QRegExp("[{}\\\\]+");
+const QRegularExpression ValueItem::ignoredInSorting(QStringLiteral("[{}\\\\]+"));
 
 ValueItem::ValueItem()
         : internalId(++internalIdCounter)
 {
-    // nothing
+    /// nothing
 }
 
 ValueItem::~ValueItem()
 {
-    // nothing
+    /// nothing
 }
 
 quint64 ValueItem::id() const
@@ -58,16 +61,21 @@ quint64 ValueItem::id() const
     return internalId;
 }
 
+bool ValueItem::operator!=(const ValueItem &other) const
+{
+    return !operator ==(other);
+}
+
 Keyword::Keyword(const Keyword &other)
         : m_text(other.m_text)
 {
-    // nothing
+    /// nothing
 }
 
 Keyword::Keyword(const QString &text)
         : m_text(text)
 {
-    // nothing
+    /// nothing
 }
 
 void Keyword::setText(const QString &text)
@@ -111,13 +119,13 @@ bool Keyword::isKeyword(const ValueItem &other) {
 Person::Person(const QString &firstName, const QString &lastName, const QString &suffix)
         : m_firstName(firstName), m_lastName(lastName), m_suffix(suffix)
 {
-    // nothing
+    /// nothing
 }
 
 Person::Person(const Person &other)
         : m_firstName(other.firstName()), m_lastName(other.lastName()), m_suffix(other.suffix())
 {
-    // nothing
+    /// nothing
 }
 
 QString Person::firstName() const
@@ -210,18 +218,16 @@ QDebug operator<<(QDebug dbg, const Person &person) {
 }
 
 
-const QRegExp MacroKey::validMacroKey = QRegExp("^[a-z][-.:/+_a-z0-9]*$|^[0-9]+$", Qt::CaseInsensitive);
-
 MacroKey::MacroKey(const MacroKey &other)
         : m_text(other.m_text)
 {
-    // nothing
+    /// nothing
 }
 
 MacroKey::MacroKey(const QString &text)
         : m_text(text)
 {
-    // nothing
+    /// nothing
 }
 
 void MacroKey::setText(const QString &text)
@@ -237,8 +243,9 @@ QString MacroKey::text() const
 bool MacroKey::isValid()
 {
     const QString t = text();
-    int idx = validMacroKey.indexIn(t);
-    return idx > -1 && validMacroKey.cap(0) == t;
+    static const QRegularExpression validMacroKey(QStringLiteral("^[a-z][-.:/+_a-z0-9]*$|^[0-9]+$"), QRegularExpression::CaseInsensitiveOption);
+    const QRegularExpressionMatch match = validMacroKey.match(t);
+    return match.hasMatch() && match.captured(0) == t;
 }
 
 void MacroKey::replace(const QString &before, const QString &after, ValueItem::ReplaceMode replaceMode)
@@ -277,13 +284,13 @@ QDebug operator<<(QDebug dbg, const MacroKey &macrokey) {
 PlainText::PlainText(const PlainText &other)
         : m_text(other.text())
 {
-    // nothing
+    /// nothing
 }
 
 PlainText::PlainText(const QString &text)
         : m_text(text)
 {
-    // nothing
+    /// nothing
 }
 
 void PlainText::setText(const QString &text)
@@ -337,13 +344,13 @@ QList<VerbatimText::ColorLabelPair> VerbatimText::colorLabelPairs = QList<Verbat
 VerbatimText::VerbatimText(const VerbatimText &other)
         : m_text(other.text())
 {
-    // nothing
+    /// nothing
 }
 
 VerbatimText::VerbatimText(const QString &text)
         : m_text(text)
 {
-    // nothing
+    /// nothing
 }
 
 void VerbatimText::setText(const QString &text)
@@ -543,9 +550,73 @@ Value &Value::operator=(Value &&rhs)
     return static_cast<Value &>(QVector<QSharedPointer<ValueItem> >::operator =((rhs)));
 }
 
-Value &Value::operator<<(const QSharedPointer<ValueItem> &vi)
+Value &Value::operator<<(const QSharedPointer<ValueItem> &value)
 {
-    return static_cast<Value &>(QVector<QSharedPointer<ValueItem> >::operator<<(vi));
+    return static_cast<Value &>(QVector<QSharedPointer<ValueItem> >::operator<<((value)));
+}
+
+bool Value::operator==(const Value &rhs) const
+{
+    const Value &lhs = *this; ///< just for readability to have a 'lhs' matching 'rhs'
+
+    /// Obviously, both Values must be of same size
+    if (lhs.count() != rhs.count()) return false;
+
+    /// Synchronously iterate over both Values' ValueItems
+    for (Value::ConstIterator lhsIt = lhs.constBegin(), rhsIt = rhs.constBegin(); lhsIt != lhs.constEnd() && rhsIt != rhs.constEnd(); ++lhsIt, ++rhsIt) {
+        /// Are both ValueItems PlainTexts and are both PlainTexts equal?
+        const QSharedPointer<PlainText> lhsPlainText = lhsIt->dynamicCast<PlainText>();
+        const QSharedPointer<PlainText> rhsPlainText = rhsIt->dynamicCast<PlainText>();
+        if ((lhsPlainText.isNull() && !rhsPlainText.isNull()) || (!lhsPlainText.isNull() && rhsPlainText.isNull())) return false;
+        if (!lhsPlainText.isNull() && !rhsPlainText.isNull()) {
+            if (*lhsPlainText.data() != *rhsPlainText.data())
+                return false;
+        } else {
+            /// Remainder of comparisons is like for PlainText above, just for other descendants of ValueItem
+            const QSharedPointer<MacroKey> lhsMacroKey = lhsIt->dynamicCast<MacroKey>();
+            const QSharedPointer<MacroKey> rhsMacroKey = rhsIt->dynamicCast<MacroKey>();
+            if ((lhsMacroKey.isNull() && !rhsMacroKey.isNull()) || (!lhsMacroKey.isNull() && rhsMacroKey.isNull())) return false;
+            if (!lhsMacroKey.isNull() && !rhsMacroKey.isNull()) {
+                if (*lhsMacroKey.data() != *rhsMacroKey.data())
+                    return false;
+            } else {
+                const QSharedPointer<Person> lhsPerson = lhsIt->dynamicCast<Person>();
+                const QSharedPointer<Person> rhsPerson = rhsIt->dynamicCast<Person>();
+                if ((lhsPerson.isNull() && !rhsPerson.isNull()) || (!lhsPerson.isNull() && rhsPerson.isNull())) return false;
+                if (!lhsPerson.isNull() && !rhsPerson.isNull()) {
+                    if (*lhsPerson.data() != *rhsPerson.data())
+                        return false;
+                } else {
+                    const QSharedPointer<VerbatimText> lhsVerbatimText = lhsIt->dynamicCast<VerbatimText>();
+                    const QSharedPointer<VerbatimText> rhsVerbatimText = rhsIt->dynamicCast<VerbatimText>();
+                    if ((lhsVerbatimText.isNull() && !rhsVerbatimText.isNull()) || (!lhsVerbatimText.isNull() && rhsVerbatimText.isNull())) return false;
+                    if (!lhsVerbatimText.isNull() && !rhsVerbatimText.isNull()) {
+                        if (*lhsVerbatimText.data() != *rhsVerbatimText.data())
+                            return false;
+                    } else {
+                        const QSharedPointer<Keyword> lhsKeyword = lhsIt->dynamicCast<Keyword>();
+                        const QSharedPointer<Keyword> rhsKeyword = rhsIt->dynamicCast<Keyword>();
+                        if ((lhsKeyword.isNull() && !rhsKeyword.isNull()) || (!lhsKeyword.isNull() && rhsKeyword.isNull())) return false;
+                        if (!lhsKeyword.isNull() && !rhsKeyword.isNull()) {
+                            if (*lhsKeyword.data() != *rhsKeyword.data())
+                                return false;
+                        } else {
+                            /// If there are other descendants of ValueItem, add tests here ...
+                            return false;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    /// No check failed, so equalness is proven
+    return true;
+}
+
+bool Value::operator!=(const Value &rhs) const
+{
+    return !operator ==(rhs);
 }
 
 QDebug operator<<(QDebug dbg, const Value &value) {
@@ -602,8 +673,11 @@ QString PlainTextValue::text(const ValueItem &valueItem, ValueItemType &vit)
     vit = VITOther;
 
 #ifdef HAVE_KF5
-    if (notificationListener == nullptr)
-        notificationListener = new PlainTextValue();
+    /// Required to have static instance of PlainTextValue here
+    /// to initialize @see personNameFormatting from settings
+    /// as well as update @see personNameFormatting upon notification
+    /// from NotificationHub
+    static PlainTextValue ptv;
 #endif // HAVE_KF5
 
     bool isVerbatim = false;
@@ -689,7 +763,6 @@ void PlainTextValue::readConfiguration()
     personNameFormatting = configGroup.readEntry(Preferences::keyPersonNameFormatting, Preferences::defaultPersonNameFormatting);
 }
 
-PlainTextValue *PlainTextValue::notificationListener = nullptr;
 QString PlainTextValue::personNameFormatting;
 #else // HAVE_KF5
 const QString PlainTextValue::personNameFormatting = QStringLiteral("<%l><, %s><, %f>");
