@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "elementeditor.h"
@@ -28,13 +28,12 @@
 #include <QFileInfo>
 #include <QMenu>
 #include <QScrollArea>
+#include <QPushButton>
 
-#include <KPushButton>
 #include <KMessageBox>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KSharedConfig>
 #include <KConfigGroup>
-#include <kio/netaccess.h>
 
 #include "menulineedit.h"
 #include "entry.h"
@@ -60,12 +59,12 @@ private:
     ElementEditor *p;
     ElementWidget *previousWidget;
     ReferenceWidget *referenceWidget;
-    ElementWidget *sourceWidget;
-    KPushButton *buttonCheckWithBibTeX;
+    SourceWidget *sourceWidget;
+    QPushButton *buttonCheckWithBibTeX;
 
     /// Settings management through a push button with menu
     KSharedConfigPtr config;
-    KPushButton *buttonOptions;
+    QPushButton *buttonOptions;
     QAction *actionForceShowAllWidgets, *actionLimitKeyboardTabStops;
 
 public:
@@ -74,7 +73,7 @@ public:
     bool elementChanged, elementUnapplied;
 
     ElementEditorPrivate(bool scrollable, ElementEditor *parent)
-            : file(NULL), p(parent), previousWidget(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), elementChanged(false), elementUnapplied(false) {
+            : file(nullptr), p(parent), previousWidget(nullptr), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), elementChanged(false), elementUnapplied(false) {
         internalEntry = QSharedPointer<Entry>();
         internalMacro = QSharedPointer<Macro>();
         internalComment = QSharedPointer<Comment>();
@@ -83,7 +82,7 @@ public:
         createGUI(scrollable);
     }
 
-    ~ElementEditorPrivate() {
+    ~ElementEditorPrivate() override {
         clearWidgets();
     }
 
@@ -95,7 +94,7 @@ public:
         widgets.clear();
     }
 
-    void setElement(QSharedPointer<Element> element, File *file) {
+    void setElement(QSharedPointer<Element> element, const File *file) {
         this->element = element;
         this->file = file;
         referenceWidget->setOriginalElement(element);
@@ -103,32 +102,30 @@ public:
     }
 
     void addTabWidgets() {
-        EntryLayout *el = EntryLayout::self();
-        for (EntryLayout::ConstIterator elit = el->constBegin(); elit != el->constEnd(); ++elit) {
-            QSharedPointer<EntryTabLayout> etl = *elit;
+        for (const auto &etl : EntryLayout::instance()) {
             ElementWidget *widget = new EntryConfiguredWidget(etl, tab);
-            connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
+            connect(widget, &ElementWidget::modified, p, &ElementEditor::childModified);
             widgets << widget;
-            if (previousWidget == NULL)
+            if (previousWidget == nullptr)
                 previousWidget = widget; ///< memorize the first tab
             int index = tab->addTab(widget, widget->icon(), widget->label());
             tab->hideTab(index);
         }
 
         ElementWidget *widget = new PreambleWidget(tab);
-        connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
+        connect(widget, &ElementWidget::modified, p, &ElementEditor::childModified);
         widgets << widget;
         int index = tab->addTab(widget, widget->icon(), widget->label());
         tab->hideTab(index);
 
         widget = new MacroWidget(tab);
-        connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
+        connect(widget, &ElementWidget::modified, p, &ElementEditor::childModified);
         widgets << widget;
         index = tab->addTab(widget, widget->icon(), widget->label());
         tab->hideTab(index);
 
         FilesWidget *filesWidget = new FilesWidget(tab);
-        connect(filesWidget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
+        connect(filesWidget, &FilesWidget::modified, p, &ElementEditor::childModified);
         widgets << filesWidget;
         index = tab->addTab(filesWidget, filesWidget->icon(), filesWidget->label());
         tab->hideTab(index);
@@ -136,23 +133,23 @@ public:
         QStringList blacklistedFields;
 
         /// blacklist fields covered by EntryConfiguredWidget
-        for (EntryLayout::ConstIterator elit = el->constBegin(); elit != el->constEnd(); ++elit)
-            for (QList<SingleFieldLayout>::ConstIterator sflit = (*elit)->singleFieldLayouts.constBegin(); sflit != (*elit)->singleFieldLayouts.constEnd(); ++sflit)
-                blacklistedFields << (*sflit).bibtexLabel;
+        for (const auto &etl : EntryLayout::instance())
+            for (const auto &sfl : const_cast<const QList<SingleFieldLayout> &>(etl->singleFieldLayouts))
+                blacklistedFields << sfl.bibtexLabel;
 
         /// blacklist fields covered by FilesWidget
-        blacklistedFields << QString(Entry::ftUrl) << QString(Entry::ftLocalFile) << QString(Entry::ftDOI) << QLatin1String("ee") << QLatin1String("biburl") << QLatin1String("postscript");
+        blacklistedFields << QString(Entry::ftUrl) << QString(Entry::ftLocalFile) << QString(Entry::ftFile) << QString(Entry::ftDOI) << QStringLiteral("ee") << QStringLiteral("biburl") << QStringLiteral("postscript");
         for (int i = 2; i < 256; ++i) // FIXME replace number by constant
-            blacklistedFields << QString(Entry::ftUrl) + QString::number(i) << QString(Entry::ftLocalFile) + QString::number(i) <<  QString(Entry::ftDOI) + QString::number(i) << QLatin1String("ee") + QString::number(i) << QLatin1String("postscript") + QString::number(i);
+            blacklistedFields << QString(Entry::ftUrl) + QString::number(i) << QString(Entry::ftLocalFile) + QString::number(i) << QString(Entry::ftFile) + QString::number(i) <<  QString(Entry::ftDOI) + QString::number(i) << QStringLiteral("ee") + QString::number(i) << QStringLiteral("postscript") + QString::number(i);
 
         widget = new OtherFieldsWidget(blacklistedFields, tab);
-        connect(widget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
+        connect(widget, &ElementWidget::modified, p, &ElementEditor::childModified);
         widgets << widget;
         index = tab->addTab(widget, widget->icon(), widget->label());
         tab->hideTab(index);
 
         sourceWidget = new SourceWidget(tab);
-        connect(sourceWidget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
+        connect(sourceWidget, &ElementWidget::modified, p, &ElementEditor::childModified);
         widgets << sourceWidget;
         index = tab->addTab(sourceWidget, sourceWidget->icon(), sourceWidget->label());
         tab->hideTab(index);
@@ -160,8 +157,8 @@ public:
 
     void createGUI(bool scrollable) {
         /// load configuration for options push button
-        static const QString configGroupName = QLatin1String("User Interface");
-        static const QString keyEnableAllWidgets = QLatin1String("EnableAllWidgets");
+        static const QString configGroupName = QStringLiteral("User Interface");
+        static const QString keyEnableAllWidgets = QStringLiteral("EnableAllWidgets");
         KConfigGroup configGroup(config, configGroupName);
         const bool showAll = configGroup.readEntry(keyEnableAllWidgets, true);
         const bool limitKeyboardTabStops = configGroup.readEntry(MenuLineEdit::keyLimitKeyboardTabStops, false);
@@ -170,8 +167,8 @@ public:
 
         referenceWidget = new ReferenceWidget(p);
         referenceWidget->setApplyElementInterface(this);
-        connect(referenceWidget, SIGNAL(modified(bool)), p, SLOT(childModified(bool)));
-        connect(referenceWidget, SIGNAL(entryTypeChanged()), p, SLOT(updateReqOptWidgets()));
+        connect(referenceWidget, &ElementWidget::modified, p, &ElementEditor::childModified);
+        connect(referenceWidget, &ReferenceWidget::entryTypeChanged, p, &ElementEditor::updateReqOptWidgets);
         vLayout->addWidget(referenceWidget, 0);
         widgets << referenceWidget;
 
@@ -193,7 +190,7 @@ public:
         vLayout->addLayout(hLayout, 0);
 
         /// Push button with menu to toggle various options
-        buttonOptions = new KPushButton(KIcon("configure"), i18n("Options"), p);
+        buttonOptions = new QPushButton(QIcon::fromTheme(QStringLiteral("configure")), i18n("Options"), p);
         hLayout->addWidget(buttonOptions, 0);
         QMenu *menuOptions = new QMenu(buttonOptions);
         buttonOptions->setMenu(menuOptions);
@@ -210,23 +207,22 @@ public:
 
         hLayout->addStretch(10);
 
-        buttonCheckWithBibTeX = new KPushButton(KIcon("tools-check-spelling"), i18n("Check with BibTeX"), p);
+        buttonCheckWithBibTeX = new QPushButton(QIcon::fromTheme(QStringLiteral("tools-check-spelling")), i18n("Check with BibTeX"), p);
         hLayout->addWidget(buttonCheckWithBibTeX, 0);
-        connect(buttonCheckWithBibTeX, SIGNAL(clicked()), p, SLOT(checkBibTeX()));
+        connect(buttonCheckWithBibTeX, &QPushButton::clicked, p, &ElementEditor::checkBibTeX);
 
         addTabWidgets();
     }
 
     void updateTabVisibility() {
-        disconnect(tab, SIGNAL(currentChanged(int)), p, SLOT(tabChanged()));
+        disconnect(tab, &HidingTabWidget::currentChanged, p, &ElementEditor::tabChanged);
         if (element.isNull()) {
             p->setEnabled(false);
         } else {
             p->setEnabled(true);
             int firstEnabledTab = 1024;
 
-            for (WidgetList::ConstIterator it = widgets.constBegin(); it != widgets.constEnd(); ++it) {
-                ElementWidget *widget = *it;
+            for (ElementWidget *widget : const_cast<const WidgetList &>(widgets)) {
                 const int index = tab->indexOf(widget);
                 const bool canEdit = widget->canEdit(element.data());
 
@@ -244,11 +240,37 @@ public:
             if (firstEnabledTab < 1024)
                 tab->setCurrentIndex(firstEnabledTab);
         }
-        connect(tab, SIGNAL(currentChanged(int)), p, SLOT(tabChanged()));
+        connect(tab, &HidingTabWidget::currentChanged, p, &ElementEditor::tabChanged);
     }
 
-    bool hasDuplicateId() const {
-        return referenceWidget != NULL && referenceWidget->isDuplicateId();
+    /**
+     * If this element editor makes use of a reference widget
+     * (e.g. where entry type and entry id/macro key can be edited),
+     * then return the current value of the entry id/macro key
+     * editing widget.
+     * Otherwise, return an empty string.
+     *
+     * @return Current value of entry id/macro key if any, otherwise empty string
+     */
+    QString currentId() const {
+        if (referenceWidget != nullptr)
+            return referenceWidget->currentId();
+        return QString();
+    }
+
+    void setCurrentId(const QString &newId) {
+        if (referenceWidget != nullptr)
+            return referenceWidget->setCurrentId(newId);
+    }
+
+    /**
+     * Return the current File object set for this element editor.
+     * May be NULL if nothing has been set or if it has been cleared.
+     *
+     * @return Current File object, may be nullptr
+     */
+    const File *currentFile() const {
+        return file;
     }
 
     void apply() {
@@ -257,27 +279,36 @@ public:
         apply(element);
     }
 
-    void apply(QSharedPointer<Element> element) {
+    void apply(QSharedPointer<Element> element) override {
+        QSharedPointer<Entry> e = element.dynamicCast<Entry>();
+        QSharedPointer<Macro> m = e.isNull() ? element.dynamicCast<Macro>() : QSharedPointer<Macro>();
+        QSharedPointer<Comment> c = e.isNull() && m.isNull() ? element.dynamicCast<Comment>() : QSharedPointer<Comment>();
+        QSharedPointer<Preamble> p = e.isNull() && m.isNull() && c.isNull() ? element.dynamicCast<Preamble>() : QSharedPointer<Preamble>();
+
         if (tab->currentWidget() == sourceWidget) {
             /// Very simple if source view is active: BibTeX code contains
             /// all necessary data
+            if (!e.isNull())
+                sourceWidget->setElementClass(SourceWidget::elementEntry);
+            else if (!m.isNull())
+                sourceWidget->setElementClass(SourceWidget::elementMacro);
+            else if (!p.isNull())
+                sourceWidget->setElementClass(SourceWidget::elementPreamble);
+            else
+                sourceWidget->setElementClass(SourceWidget::elementInvalid);
             sourceWidget->apply(element);
         } else {
             /// Start by assigning the current internal element's
             /// data to the output element
-            QSharedPointer<Entry> e = element.dynamicCast<Entry>();
             if (!e.isNull())
                 *e = *internalEntry;
             else {
-                QSharedPointer<Macro> m = element.dynamicCast<Macro>();
                 if (!m.isNull())
                     *m = *internalMacro;
                 else {
-                    QSharedPointer<Comment> c = element.dynamicCast<Comment>();
                     if (!c.isNull())
                         *c = *internalComment;
                     else {
-                        QSharedPointer<Preamble> p = element.dynamicCast<Preamble>();
                         if (!p.isNull())
                             *p = *internalPreamble;
                         else
@@ -288,16 +319,32 @@ public:
 
             /// The internal element may be outdated (only updated on tab switch),
             /// so apply the reference widget's data on the output element
-            if (referenceWidget != NULL)
+            if (referenceWidget != nullptr)
                 referenceWidget->apply(element);
             /// The internal element may be outdated (only updated on tab switch),
             /// so apply the current widget's data on the output element
             ElementWidget *currentElementWidget = qobject_cast<ElementWidget *>(tab->currentWidget());
-            if (currentElementWidget != NULL)
+            if (currentElementWidget != nullptr)
                 currentElementWidget->apply(element);
         }
     }
 
+    bool validate(QWidget **widgetWithIssue, QString &message) const override {
+        if (tab->currentWidget() == sourceWidget) {
+            /// Source widget must check its textual content for being valid BibTeX code
+            return sourceWidget->validate(widgetWithIssue, message);
+        } else {
+            /// All widgets except for the source widget must validate their values
+            for (WidgetList::ConstIterator it = widgets.begin(); it != widgets.end(); ++it) {
+                if ((*it) == sourceWidget) continue;
+                const bool v = (*it)->validate(widgetWithIssue, message);
+                /// A single widget failing to validate lets the whole validation fail
+                if (!v) return false;
+            }
+            return true;
+        }
+    }
+
     void reset() {
         elementChanged = false;
         elementUnapplied = false;
@@ -318,21 +365,25 @@ public:
         }
 
         QSharedPointer<const Entry> e = element.dynamicCast<const Entry>();
-        if (!e.isNull())
+        if (!e.isNull()) {
             internalEntry = QSharedPointer<Entry>(new Entry(*e.data()));
-        else {
+            sourceWidget->setElementClass(SourceWidget::elementEntry);
+        } else {
             QSharedPointer<const Macro> m = element.dynamicCast<const Macro>();
-            if (!m.isNull())
+            if (!m.isNull()) {
                 internalMacro = QSharedPointer<Macro>(new Macro(*m.data()));
-            else {
+                sourceWidget->setElementClass(SourceWidget::elementMacro);
+            } else {
                 QSharedPointer<const Comment> c = element.dynamicCast<const Comment>();
-                if (!c.isNull())
+                if (!c.isNull()) {
                     internalComment = QSharedPointer<Comment>(new Comment(*c.data()));
-                else {
+                    sourceWidget->setElementClass(SourceWidget::elementComment);
+                } else {
                     QSharedPointer<const Preamble> p = element.dynamicCast<const Preamble>();
-                    if (!p.isNull())
+                    if (!p.isNull()) {
                         internalPreamble = QSharedPointer<Preamble>(new Preamble(*p.data()));
-                    else
+                        sourceWidget->setElementClass(SourceWidget::elementPreamble);
+                    } else
                         Q_ASSERT_X(element.isNull(), "ElementEditor::ElementEditorPrivate::reset(QSharedPointer<const Element> element)", "element is not NULL but could not be cast on a valid Element sub-class");
                 }
             }
@@ -356,13 +407,13 @@ public:
 
         /// update the enabled/disabled state of required and optional widgets/fields
         bool forceVisible = actionForceShowAllWidgets->isChecked();
-        foreach (ElementWidget *elementWidget, widgets) {
+        for (ElementWidget *elementWidget : const_cast<const WidgetList &>(widgets)) {
             elementWidget->showReqOptWidgets(forceVisible, tempEntry->type());
         }
 
         /// save configuration
-        static const QString configGroupName = QLatin1String("User Interface");
-        static const QString keyEnableAllWidgets = QLatin1String("EnableAllWidgets");
+        static const QString configGroupName = QStringLiteral("User Interface");
+        static const QString keyEnableAllWidgets = QStringLiteral("EnableAllWidgets");
         KConfigGroup configGroup(config, configGroupName);
         configGroup.writeEntry(keyEnableAllWidgets, actionForceShowAllWidgets->isChecked());
         config->sync();
@@ -370,7 +421,7 @@ public:
 
     void limitKeyboardTabStops() {
         /// save configuration
-        static const QString configGroupName = QLatin1String("User Interface");
+        static const QString configGroupName = QStringLiteral("User Interface");
         KConfigGroup configGroup(config, configGroupName);
         configGroup.writeEntry(MenuLineEdit::keyLimitKeyboardTabStops, actionLimitKeyboardTabStops->isChecked());
         config->sync();
@@ -387,7 +438,7 @@ public:
         /// Interprete future widget as an ElementWidget
         ElementWidget *futureWidget = qobject_cast<ElementWidget *>(futureTab);
         /// Past and future ElementWidget values are valid?
-        if (previousWidget != NULL && futureWidget != NULL) {
+        if (previousWidget != nullptr && futureWidget != nullptr) {
             /// Assign to temp wihch internal variable holds current state
             QSharedPointer<Element> temp;
             if (!internalEntry.isNull())
@@ -403,11 +454,11 @@ public:
             /// Past widget writes its state to the internal state
             previousWidget->apply(temp);
             /// Before switching to source widget, store internally reference widget's state
-            if (isToSourceWidget && referenceWidget != NULL) referenceWidget->apply(temp);
+            if (isToSourceWidget && referenceWidget != nullptr) referenceWidget->apply(temp);
             /// Tell future widget to initialize itself based on internal state
             futureWidget->reset(temp);
             /// When switchin from source widget to another widget, initialize reference widget
-            if (isFromSourceWidget && referenceWidget != NULL)
+            if (isFromSourceWidget && referenceWidget != nullptr)
                 referenceWidget->reset(temp);
         }
         previousWidget = futureWidget;
@@ -443,44 +494,57 @@ public:
 ElementEditor::ElementEditor(bool scrollable, QWidget *parent)
         : QWidget(parent), d(new ElementEditorPrivate(scrollable, this))
 {
-    connect(d->tab, SIGNAL(currentChanged(int)), this, SLOT(tabChanged()));
+    connect(d->tab, &HidingTabWidget::currentChanged, this, &ElementEditor::tabChanged);
 }
 
 ElementEditor::~ElementEditor()
 {
-    disconnect(d->tab, SIGNAL(currentChanged(int)), this, SLOT(tabChanged()));
+    disconnect(d->tab, &HidingTabWidget::currentChanged, this, &ElementEditor::tabChanged);
     delete d;
 }
 
 void ElementEditor::apply()
 {
+    /// The prime problem to tackle in this function is to cope with
+    /// invalid/problematic entry ids or macro keys, respectively:
+    ///  - empty ids/keys
+    ///  - ids/keys that are duplicates of already used ids/keys
+
     QSharedPointer<Entry> entry = d->element.dynamicCast<Entry>();
     QSharedPointer<Macro> macro = d->element.dynamicCast<Macro>();
-    const bool doReplaceId = d->hasDuplicateId() && (
-                                 /// the id of the entry is already in use by another entry
-                                 (!entry.isNull() && KMessageBox::warningContinueCancel(this, i18n("The entered id '%1' is already in use for another entry.", entry->id()), i18n("Id already in use"), KGuiItem(i18n("Keep duplicate ids")), KGuiItem(i18n("Restore original id"))) == KMessageBox::Cancel)
-                                 /// the key of a macro is already in use by another macro
-                                 || (!macro.isNull() && KMessageBox::warningContinueCancel(this, i18n("The entered key '%1' is already in use for another macro.", macro->key()), i18n("Key already in use"), KGuiItem(i18n("Keep duplicate keys")), KGuiItem(i18n("Restore original key"))) == KMessageBox::Cancel)
-                             );
+    /// Determine id/key as it was set before the current editing started
+    const QString originalId = !entry.isNull() ? entry->id() : (!macro.isNull() ? macro->key() : QString());
+    /// Get the id/key as it is in the editing widget right now
+    const QString newId = d->currentId();
+    /// Keep track whether the 'original' id/key or the 'new' id/key will eventually be used
+    enum IdToUse {UseOriginalId, UseNewId};
+    IdToUse idToUse = UseNewId;
 
-    QString replacementId;
-    if (doReplaceId) {
-        if (!entry.isNull())
-            replacementId = entry->id();
-        else if (!macro.isNull())
-            replacementId = macro->key();
+    if (newId.isEmpty() && !originalId.isEmpty()) {
+        /// New id/key is empty (invalid by definition), so just notify use and revert back to original id/key
+        /// (assuming that original id/key is valid)
+        KMessageBox::sorry(this, i18n("No id was entered, so the previous id '%1' will be restored.", originalId), i18n("No id given"));
+        idToUse = UseOriginalId;
+    } else if (!newId.isEmpty()) {
+        /// If new id/key is not empty, then check if it is identical to another entry/macro in the current file
+        const QSharedPointer<Element> knownElementWithSameId = d->currentFile() != nullptr ? d->currentFile()->containsKey(newId) : QSharedPointer<Element>();
+        if (!knownElementWithSameId.isNull() && d->element != knownElementWithSameId) {
+            /// Some other, different element (entry or macro) uses same id/key, so ask user how to proceed
+            const int msgBoxResult = KMessageBox::warningContinueCancel(this, i18n("The entered id '%1' is already in use for another element.\n\nKeep original id '%2' instead?", newId, originalId), i18n("Id already in use"), KGuiItem(i18n("Keep duplicate ids")), KGuiItem(i18n("Restore original id")));
+            idToUse = msgBoxResult == KMessageBox::Continue ? UseNewId : UseOriginalId;
+        }
     }
 
+    if (idToUse == UseOriginalId) {
+        /// As 'apply()' above set the 'new' id/key but the 'original' id/key is to be used,
+        /// now UI must be updated accordingly. Changes will propagate to the entry id or
+        /// macro key, respectively, when invoking apply() further down
+        d->setCurrentId(originalId);
+    }
+
+    /// Apply will always set the 'new' id/key to the entry or macro, respectively
     d->apply();
 
-    if (doReplaceId) {
-        if (!entry.isNull())
-            entry->setId(replacementId);
-        else if (!macro.isNull())
-            macro->setKey(replacementId);
-        d->reset();
-    }
-
     d->setModified(false);
     emit modified(false);
 }
@@ -491,14 +555,39 @@ void ElementEditor::reset()
     emit modified(false);
 }
 
-void ElementEditor::setElement(QSharedPointer<Element> element, File *file)
+bool ElementEditor::validate() {
+    QWidget *widgetWithIssue = nullptr;
+    QString message;
+    if (!validate(&widgetWithIssue, message)) {
+        const QString msgBoxMessage = message.isEmpty() ? i18n("Validation for the current element failed.") : i18n("Validation for the current element failed:\n%1", message);
+        KMessageBox::error(this, msgBoxMessage, i18n("Element validation failed"));
+        if (widgetWithIssue != nullptr) {
+            /// Probe if widget with issue is inside a QTabWiget; if yes, make parenting tab the current tab
+            QWidget *cur = widgetWithIssue;
+            do {
+                QTabWidget *tabWidget = cur->parent() != nullptr && cur->parent()->parent() != nullptr ? qobject_cast<QTabWidget *>(cur->parent()->parent()) : nullptr;
+                if (tabWidget != nullptr) {
+                    tabWidget->setCurrentWidget(cur);
+                    break;
+                }
+                cur = qobject_cast<QWidget *>(cur->parent());
+            } while (cur != nullptr);
+            /// Set focus to widget with issue
+            widgetWithIssue->setFocus();
+        }
+        return false;
+    }
+    return true;
+}
+
+void ElementEditor::setElement(QSharedPointer<Element> element, const File *file)
 {
     d->setElement(element, file);
     d->reset();
     emit modified(false);
 }
 
-void ElementEditor::setElement(QSharedPointer<const Element> element, File *file)
+void ElementEditor::setElement(QSharedPointer<const Element> element, const File *file)
 {
     QSharedPointer<Element> clone;
     QSharedPointer<const Entry> entry = element.dynamicCast<const Entry>();
@@ -517,7 +606,7 @@ void ElementEditor::setElement(QSharedPointer<const Element> element, File *file
                 if (!comment.isNull())
                     clone = QSharedPointer<Comment>(new Comment(*comment.data()));
                 else
-                    Q_ASSERT_X(element == NULL, "ElementEditor::ElementEditor(const Element *element, QWidget *parent)", "element is not NULL but could not be cast on a valid Element sub-class");
+                    Q_ASSERT_X(element == nullptr, "ElementEditor::ElementEditor(const Element *element, QWidget *parent)", "element is not NULL but could not be cast on a valid Element sub-class");
             }
         }
     }
@@ -541,6 +630,11 @@ bool ElementEditor::elementUnapplied()
     return d->elementUnapplied;
 }
 
+bool ElementEditor::validate(QWidget **widgetWithIssue, QString &message)
+{
+    return d->validate(widgetWithIssue, message);
+}
+
 QWidget *ElementEditor::currentPage() const
 {
     return d->tab->currentWidget();
