@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "findpdfui.h"
@@ -30,21 +30,24 @@
 #include <QRadioButton>
 #include <QLabel>
 #include <QPointer>
+#include <QPushButton>
+#include <QFileDialog>
+#include <QDialog>
+#include <QDialogButtonBox>
+#include <QMimeDatabase>
+#include <QMimeType>
+#include <QTemporaryFile>
 
-#include <KDialog>
-#include <KLocale>
-#include <KDebug>
+#include <KLocalizedString>
+#include <KIconLoader>
 #include <KSqueezedTextLabel>
-#include <KMenu>
-#include <KPushButton>
-#include <KMimeType>
 #include <KRun>
-#include <KTemporaryFile>
-#include <KFileDialog>
-#include <KIO/NetAccess>
+#include <kio_version.h>
 
+#include "preferences.h"
 #include "fileinfo.h"
 #include "fieldlistedit.h"
+#include "logging_gui.h"
 
 class PDFListModel;
 
@@ -55,22 +58,17 @@ const int posRadioNoDownload = 3;
 const int posRadioDownload = 4;
 const int posRadioURLonly = 5;
 
-const int URLRole = Qt::UserRole + 1234;
-const int TextualPreviewRole = Qt::UserRole + 1237;
-const int TempFileNameRole = Qt::UserRole + 1236;
-const int DownloadModeRole = Qt::UserRole + 1235;
-
 /// inspired by KNewStuff3's ItemsViewDelegate
 PDFItemDelegate::PDFItemDelegate(QListView *itemView, QObject *parent)
         : KWidgetItemDelegate(itemView, parent), m_parent(itemView)
 {
-    // nothing
+    /// nothing
 }
 
 void PDFItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
     QStyle *style = QApplication::style();
-    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, 0);
+    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, nullptr);
 
     painter->save();
 
@@ -90,8 +88,9 @@ void PDFItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &optio
     painter->restore();
 }
 
-QList<QWidget *> PDFItemDelegate::createItemWidgets() const
+QList<QWidget *> PDFItemDelegate::createItemWidgets(const QModelIndex &index) const
 {
+    Q_UNUSED(index) // FIXME really of no use?
     QList<QWidget *> list;
 
     /// first, the label with shows the found PDF file's origin (URL)
@@ -109,9 +108,9 @@ QList<QWidget *> PDFItemDelegate::createItemWidgets() const
     Q_ASSERT_X(list.count() == posLabelPreview + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posLabelPreview + 1");
 
     /// add a push button to view the PDF file
-    KPushButton *pushButton = new KPushButton(KIcon("application-pdf"), i18n("View"));
+    QPushButton *pushButton = new QPushButton(QIcon::fromTheme(QStringLiteral("application-pdf")), i18n("View"));
     list << pushButton;
-    connect(pushButton, SIGNAL(clicked()), this, SLOT(slotViewPDF()));
+    connect(pushButton, &QPushButton::clicked, this, &PDFItemDelegate::slotViewPDF);
     Q_ASSERT_X(list.count() == posViewButton + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posViewButton + 1");
 
     /// a button group to choose what to do with this particular PDF file
@@ -121,34 +120,36 @@ QList<QWidget *> PDFItemDelegate::createItemWidgets() const
     QRadioButton *radioButton = new QRadioButton(i18n("Ignore"));
     bg->addButton(radioButton);
     list << radioButton;
-    connect(radioButton, SIGNAL(toggled(bool)), this, SLOT(slotRadioNoDownloadToggled(bool)));
+    connect(radioButton, &QRadioButton::toggled, this, &PDFItemDelegate::slotRadioNoDownloadToggled);
     Q_ASSERT_X(list.count() == posRadioNoDownload + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posRadioNoDownload + 1");
 
     /// download this file and store it locally, user will be asked for "Save As"
     radioButton = new QRadioButton(i18n("Download"));
     bg->addButton(radioButton);
     list << radioButton;
-    connect(radioButton, SIGNAL(toggled(bool)), this, SLOT(slotRadioDownloadToggled(bool)));
+    connect(radioButton, &QRadioButton::toggled, this, &PDFItemDelegate::slotRadioDownloadToggled);
     Q_ASSERT_X(list.count() == posRadioDownload + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posRadioDownload + 1");
 
     /// paste URL into BibTeX entry, no local copy is stored
     radioButton = new QRadioButton(i18n("Use URL only"));
     bg->addButton(radioButton);
     list << radioButton;
-    connect(radioButton, SIGNAL(toggled(bool)), this, SLOT(slotRadioURLonlyToggled(bool)));
+    connect(radioButton, &QRadioButton::toggled, this, &PDFItemDelegate::slotRadioURLonlyToggled);
     Q_ASSERT_X(list.count() == posRadioURLonly + 1, "QList<QWidget *> PDFItemDelegate::createItemWidgets() const", "list.count() != posRadioURLonly + 1");
 
     return list;
 }
 
-/// update the widgets
+/// Update the widgets
+/// Clazy warns: "Missing reference on non-trivial type" for argument 'widgets',
+/// but KWidgetItemDelegate defines this function this way and cannot be changed.
 void PDFItemDelegate::updateItemWidgets(const QList<QWidget *> widgets, const QStyleOptionViewItem &option, const QPersistentModelIndex &index) const
 {
     if (!index.isValid()) return;
 
     const PDFListModel *model = qobject_cast<const PDFListModel *>(index.model());
-    if (model == NULL) {
-        kDebug() << "WARNING - INVALID MODEL!";
+    if (model == nullptr) {
+        qCDebug(LOG_KBIBTEX_GUI) << "WARNING - INVALID MODEL!";
         return;
     }
 
@@ -165,8 +166,8 @@ void PDFItemDelegate::updateItemWidgets(const QList<QWidget *> widgets, const QS
 
     /// setup label which will show the PDF file's URL
     KSqueezedTextLabel *label = qobject_cast<KSqueezedTextLabel *>(widgets[posLabelUrl]);
-    if (label != NULL) {
-        const QString text = index.data(URLRole).toUrl().toString();
+    if (label != nullptr) {
+        const QString text = index.data(PDFListModel::URLRole).toUrl().toDisplayString();
         label->setText(text);
         label->setToolTip(text);
         label->move(margin * 2 + KIconLoader::SizeMedium, margin);
@@ -175,15 +176,15 @@ void PDFItemDelegate::updateItemWidgets(const QList<QWidget *> widgets, const QS
 
     /// setup label which will show the PDF's title or textual beginning
     QLabel *previewLabel = qobject_cast<QLabel *>(widgets[posLabelPreview]);
-    if (previewLabel != NULL) {
-        previewLabel->setText(index.data(TextualPreviewRole).toString());
+    if (previewLabel != nullptr) {
+        previewLabel->setText(index.data(PDFListModel::TextualPreviewRole).toString());
         previewLabel->move(margin * 2 + KIconLoader::SizeMedium, margin * 2 + labelHeight);
         previewLabel->resize(labelWidth, labelHeight);
     }
 
     /// setup the view button
-    KPushButton *viewButton = qobject_cast<KPushButton *>(widgets[posViewButton]);
-    if (viewButton != NULL) {
+    QPushButton *viewButton = qobject_cast<QPushButton *>(widgets[posViewButton]);
+    if (viewButton != nullptr) {
         const QSize hint = viewButton->sizeHint();
         const int h = hint.isValid() ? qMin(buttonHeight, hint.height()) : buttonHeight;
         viewButton->move(margin * 2 + KIconLoader::SizeMedium, option.rect.height() - margin - h);
@@ -193,13 +194,13 @@ void PDFItemDelegate::updateItemWidgets(const QList<QWidget *> widgets, const QS
     /// setup each of the three radio buttons
     for (int i = 0; i < 3; ++i) {
         QRadioButton *radioButton = qobject_cast<QRadioButton *>(widgets[posRadioNoDownload + i]);
-        if (radioButton != NULL) {
+        if (radioButton != nullptr) {
             const QSize hint = radioButton->sizeHint();
             const int h = hint.isValid() ? qMin(buttonHeight, hint.height()) : buttonHeight;
             radioButton->move(option.rect.width() - margin - (3 - i) * (buttonWidth + margin), option.rect.height() - margin - h);
             radioButton->resize(buttonWidth, h);
             bool ok = false;
-            radioButton->setChecked(i + FindPDF::NoDownload == index.data(DownloadModeRole).toInt(&ok) && ok);
+            radioButton->setChecked(i + FindPDF::NoDownload == index.data(PDFListModel::DownloadModeRole).toInt(&ok) && ok);
         }
     }
 }
@@ -209,7 +210,7 @@ QSize PDFItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModel
     /// set a size that is suiteable
     QSize size;
     size.setWidth(option.fontMetrics.width(i18n("Download")) * 6);
-    size.setHeight(qMax(option.fontMetrics.height() * 16 / 3, (int)KIconLoader::SizeMedium));
+    size.setHeight(qMax(option.fontMetrics.height() * 16 / 3, static_cast<int>(KIconLoader::SizeMedium))); ///< KIconLoader::SizeMedium should be 32
     return size;
 }
 
@@ -222,21 +223,29 @@ void PDFItemDelegate::slotViewPDF()
     QModelIndex index = focusedIndex();
 
     if (index.isValid()) {
-        const QString tempfileName = index.data(TempFileNameRole).toString();
-        const QUrl url = index.data(URLRole).toUrl();
+        const QString tempfileName = index.data(PDFListModel::TempFileNameRole).toString();
+        const QUrl url = index.data(PDFListModel::URLRole).toUrl();
         if (!tempfileName.isEmpty()) {
             /// Guess mime type for url to open
-            KUrl tempUrl(tempfileName);
-            KMimeType::Ptr mimeType = FileInfo::mimeTypeForUrl(tempUrl);
-            const QString mimeTypeName = mimeType->name();
+            QUrl tempUrl(tempfileName);
+            QMimeType mimeType = FileInfo::mimeTypeForUrl(tempUrl);
+            const QString mimeTypeName = mimeType.name();
             /// Ask KDE subsystem to open url in viewer matching mime type
-            KRun::runUrl(tempUrl, mimeTypeName, NULL, false, false, url.toString());
+#if KIO_VERSION < 0x051f00 // < 5.31.0
+            KRun::runUrl(tempUrl, mimeTypeName, itemView(), false, false, url.toDisplayString());
+#else // KIO_VERSION < 0x051f00 // >= 5.31.0
+            KRun::runUrl(tempUrl, mimeTypeName, itemView(), KRun::RunFlags(), url.toDisplayString());
+#endif // KIO_VERSION < 0x051f00
         } else if (url.isValid()) {
             /// Guess mime type for url to open
-            KMimeType::Ptr mimeType = FileInfo::mimeTypeForUrl(url);
-            const QString mimeTypeName = mimeType->name();
+            QMimeType mimeType = FileInfo::mimeTypeForUrl(url);
+            const QString mimeTypeName = mimeType.name();
             /// Ask KDE subsystem to open url in viewer matching mime type
-            KRun::runUrl(url, mimeTypeName, NULL, false, false);
+#if KIO_VERSION < 0x051f00 // < 5.31.0
+            KRun::runUrl(url, mimeTypeName, itemView(), false, false);
+#else // KIO_VERSION < 0x051f00 // >= 5.31.0
+            KRun::runUrl(url, mimeTypeName, itemView(), KRun::RunFlags());
+#endif // KIO_VERSION < 0x051f00
         }
     }
 }
@@ -249,7 +258,7 @@ void PDFItemDelegate::slotRadioNoDownloadToggled(bool checked)
     QModelIndex index = focusedIndex();
 
     if (index.isValid() && checked) {
-        m_parent->model()->setData(index, FindPDF::NoDownload, DownloadModeRole);
+        m_parent->model()->setData(index, FindPDF::NoDownload, PDFListModel::DownloadModeRole);
     }
 }
 
@@ -261,7 +270,7 @@ void PDFItemDelegate::slotRadioDownloadToggled(bool checked)
     QModelIndex index = focusedIndex();
 
     if (index.isValid() && checked) {
-        m_parent->model()->setData(index, FindPDF::Download, DownloadModeRole);
+        m_parent->model()->setData(index, FindPDF::Download, PDFListModel::DownloadModeRole);
     }
 }
 
@@ -273,14 +282,14 @@ void PDFItemDelegate::slotRadioURLonlyToggled(bool checked)
     QModelIndex index = focusedIndex();
 
     if (index.isValid() && checked) {
-        m_parent->model()->setData(index, FindPDF::URLonly, DownloadModeRole);
+        m_parent->model()->setData(index, FindPDF::URLonly, PDFListModel::DownloadModeRole);
     }
 }
 
 PDFListModel::PDFListModel(QList<FindPDF::ResultItem> &resultList, QObject *parent)
         : QAbstractListModel(parent), m_resultList(resultList)
 {
-    // nothing
+    /// nothing
 }
 
 int PDFListModel::rowCount(const QModelIndex &parent) const
@@ -294,15 +303,15 @@ QVariant PDFListModel::data(const QModelIndex &index, int role) const
 {
     if (index != QModelIndex() && index.parent() == QModelIndex() && index.row() < m_resultList.count()) {
         if (role == Qt::DisplayRole)
-            return m_resultList[index.row()].url.toString();
+            return m_resultList[index.row()].url.toDisplayString();
         else if (role == URLRole)
             return  m_resultList[index.row()].url;
         else if (role == TextualPreviewRole)
             return  m_resultList[index.row()].textPreview;
         else if (role == Qt::ToolTipRole)
-            return  QLatin1String("<qt>") + m_resultList[index.row()].textPreview + QLatin1String("</qt>"); ///<  'qt' tags required for word wrap
+            return  QStringLiteral("<qt>") + m_resultList[index.row()].textPreview + QStringLiteral("</qt>"); ///<  'qt' tags required for word wrap
         else if (role == TempFileNameRole) {
-            if (m_resultList[index.row()].tempFilename != NULL)
+            if (m_resultList[index.row()].tempFilename != nullptr)
                 return m_resultList[index.row()].tempFilename->fileName();
             else
                 return QVariant();
@@ -310,9 +319,9 @@ QVariant PDFListModel::data(const QModelIndex &index, int role) const
             return m_resultList[index.row()].downloadMode;
         else if (role == Qt::DecorationRole) {
             /// make an educated guess on the icon, based on URL or path
-            QString iconName = FileInfo::mimeTypeForUrl(m_resultList[index.row()].url)->iconName();
-            iconName = iconName == QLatin1String("application-octet-stream") ? QLatin1String("application-pdf") : iconName;
-            return KIcon(iconName).pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
+            QString iconName = FileInfo::mimeTypeForUrl(m_resultList[index.row()].url).iconName();
+            iconName = iconName == QStringLiteral("application-octet-stream") ? QStringLiteral("application-pdf") : iconName;
+            return QIcon::fromTheme(iconName).pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
         } else
             return QVariant();
     }
@@ -323,7 +332,7 @@ bool PDFListModel::setData(const QModelIndex &index, const QVariant &value, int
 {
     if (index != QModelIndex() && index.row() < m_resultList.count() && role == DownloadModeRole) {
         bool ok = false;
-        FindPDF::DownloadMode downloadMode = (FindPDF::DownloadMode)value.toInt(&ok);
+        const FindPDF::DownloadMode downloadMode = static_cast<FindPDF::DownloadMode>(value.toInt(&ok));
         if (ok) {
             m_resultList[index.row()].downloadMode = downloadMode;
             return true;
@@ -392,8 +401,8 @@ FindPDFUI::FindPDFUI(Entry &entry, QWidget *parent)
     d->labelMessage->show();
     d->labelMessage->setText(i18n("Starting to search..."));
 
-    connect(d->findpdf, SIGNAL(finished()), this, SLOT(searchFinished()));
-    connect(d->findpdf, SIGNAL(progress(int,int,int)), this, SLOT(searchProgress(int,int,int)));
+    connect(d->findpdf, &FindPDF::finished, this, &FindPDFUI::searchFinished);
+    connect(d->findpdf, &FindPDF::progress, this, &FindPDFUI::searchProgress);
     d->findpdf->search(entry);
 }
 
@@ -407,17 +416,24 @@ FindPDFUI::~FindPDFUI()
 
 void FindPDFUI::interactiveFindPDF(Entry &entry, const File &bibtexFile, QWidget *parent)
 {
-    QPointer<KDialog> dlg = new KDialog(parent);
+    QPointer<QDialog> dlg = new QDialog(parent);
     QPointer<FindPDFUI> widget = new FindPDFUI(entry, dlg);
-    dlg->setCaption(i18n("Find PDF"));
-    dlg->setMainWidget(widget);
-    dlg->enableButtonOk(false);
+    dlg->setWindowTitle(i18n("Find PDF"));
+    QBoxLayout *layout = new QVBoxLayout(dlg);
+    layout->addWidget(widget);
+    QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Abort | QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, dlg);
+    layout->addWidget(buttonBox);
+    dlg->setLayout(layout);
 
-    connect(widget, SIGNAL(resultAvailable(bool)), dlg, SLOT(enableButtonOk(bool)));
+    buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+    connect(widget.data(), &FindPDFUI::resultAvailable, buttonBox->button(QDialogButtonBox::Ok), &QWidget::setEnabled);
+    connect(widget.data(), &FindPDFUI::resultAvailable, buttonBox->button(QDialogButtonBox::Abort), &QWidget::setDisabled);
+    connect(buttonBox->button(QDialogButtonBox::Ok), &QPushButton::clicked, dlg.data(), &QDialog::accept);
+    connect(buttonBox->button(QDialogButtonBox::Cancel), &QPushButton::clicked, dlg.data(), &QDialog::reject);
+    connect(buttonBox->button(QDialogButtonBox::Abort), &QPushButton::clicked, widget.data(), &FindPDFUI::stopSearch);
 
-    if (dlg->exec() == KDialog::Accepted) {
+    if (dlg->exec() == QDialog::Accepted)
         widget->apply(entry, bibtexFile);
-    }
     delete dlg;
 }
 
@@ -426,28 +442,28 @@ void FindPDFUI::apply(Entry &entry, const File &bibtexFile)
     QAbstractItemModel *model = d->listViewResult->model();
     for (int i = 0; i < model->rowCount(); ++i) {
         bool ok = false;
-        FindPDF::DownloadMode downloadMode = (FindPDF::DownloadMode)model->data(model->index(i, 0), DownloadModeRole).toInt(&ok);
+        FindPDF::DownloadMode downloadMode = static_cast<FindPDF::DownloadMode>(model->data(model->index(i, 0), PDFListModel::DownloadModeRole).toInt(&ok));
         if (!ok) {
-            kDebug() << "Could not interprete download mode";
+            qCDebug(LOG_KBIBTEX_GUI) << "Could not interprete download mode";
             downloadMode = FindPDF::NoDownload;
         }
 
-        QUrl url = model->data(model->index(i, 0), URLRole).toUrl();
-        QString tempfileName = model->data(model->index(i, 0), TempFileNameRole).toString();
+        QUrl url = model->data(model->index(i, 0), PDFListModel::URLRole).toUrl();
+        QString tempfileName = model->data(model->index(i, 0), PDFListModel::TempFileNameRole).toString();
 
         if (downloadMode == FindPDF::URLonly && url.isValid()) {
             bool alreadyContained = false;
             for (QMap<QString, Value>::ConstIterator it = entry.constBegin(); !alreadyContained && it != entry.constEnd(); ++it)
                 // FIXME this will terribly break if URLs in an entry's URL field are separated with semicolons
-                alreadyContained |= it.key().toLower().startsWith(Entry::ftUrl) && PlainTextValue::text(it.value()) == url.toString();
+                alreadyContained |= it.key().toLower().startsWith(Entry::ftUrl) && PlainTextValue::text(it.value()) == url.toDisplayString();
             if (!alreadyContained) {
                 Value value;
-                value.append(QSharedPointer<VerbatimText>(new VerbatimText(url.toString())));
+                value.append(QSharedPointer<VerbatimText>(new VerbatimText(url.toDisplayString())));
                 if (!entry.contains(Entry::ftUrl))
                     entry.insert(Entry::ftUrl, value);
                 else
                     for (int i = 2; i < 256; ++i) {
-                        const QString keyName = QString(QLatin1String("%1%2")).arg(Entry::ftUrl).arg(i);
+                        const QString keyName = QString(QStringLiteral("%1%2")).arg(Entry::ftUrl).arg(i);
                         if (!entry.contains(keyName)) {
                             entry.insert(keyName, value);
                             break;
@@ -455,26 +471,27 @@ void FindPDFUI::apply(Entry &entry, const File &bibtexFile)
                     }
             }
         } else if (downloadMode == FindPDF::Download && !tempfileName.isEmpty()) {
-            KUrl startUrl = bibtexFile.property(File::Url, QUrl()).toUrl();
-            const QString absoluteFilename = KFileDialog::getSaveFileName(KUrl::fromLocalFile(startUrl.directory()), QLatin1String("application/pdf"), this, i18n("Save URL '%1'", url.toString()));
+            QUrl startUrl = bibtexFile.property(File::Url, QUrl()).toUrl();
+            const QString absoluteFilename = QFileDialog::getSaveFileName(this, i18n("Save URL '%1'", url.url(QUrl::PreferLocalFile)), startUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path(), QStringLiteral("application/pdf"));
             if (!absoluteFilename.isEmpty()) {
                 const QString visibleFilename = UrlListEdit::askRelativeOrStaticFilename(this, absoluteFilename, startUrl);
 
-                kDebug() << "Saving PDF from " << url << " to file " << absoluteFilename << " known as " << visibleFilename;
+                qCDebug(LOG_KBIBTEX_GUI) << "Saving PDF from " << url << " to file " << absoluteFilename << " known as " << visibleFilename;
                 // FIXME test for overwrite
-                KIO::NetAccess::file_copy(KUrl::fromLocalFile(tempfileName), KUrl::fromLocalFile(absoluteFilename), this);
+                QFile::copy(tempfileName, absoluteFilename);
 
                 bool alreadyContained = false;
                 for (QMap<QString, Value>::ConstIterator it = entry.constBegin(); !alreadyContained && it != entry.constEnd(); ++it)
-                    alreadyContained |= (it.key().toLower().startsWith(Entry::ftLocalFile) || it.key().toLower().startsWith(Entry::ftUrl)) && PlainTextValue::text(it.value()) == url.toString();
+                    alreadyContained |= (it.key().toLower().startsWith(Entry::ftFile) || it.key().toLower().startsWith(Entry::ftLocalFile) || it.key().toLower().startsWith(Entry::ftUrl)) && PlainTextValue::text(it.value()) == url.toDisplayString();
                 if (!alreadyContained) {
                     Value value;
                     value.append(QSharedPointer<VerbatimText>(new VerbatimText(visibleFilename)));
-                    if (!entry.contains(Entry::ftLocalFile))
-                        entry.insert(Entry::ftLocalFile, value);
+                    const QString fieldNameStem = Preferences::bibliographySystem() == Preferences::BibTeX ? Entry::ftLocalFile : Entry::ftFile;
+                    if (!entry.contains(fieldNameStem))
+                        entry.insert(fieldNameStem, value);
                     else
                         for (int i = 2; i < 256; ++i) {
-                            const QString keyName = QString(QLatin1String("%1%2")).arg(Entry::ftLocalFile).arg(i);
+                            const QString keyName = QString(QStringLiteral("%1%2")).arg(fieldNameStem).arg(i);
                             if (!entry.contains(keyName)) {
                                 entry.insert(keyName, value);
                                 break;
@@ -507,3 +524,12 @@ void FindPDFUI::searchProgress(int visitedPages, int runningJobs, int foundDocum
     d->labelMessage->show();
     d->labelMessage->setText(i18n("<qt>Number of visited pages: <b>%1</b><br/>Number of running downloads: <b>%2</b><br/>Number of found documents: <b>%3</b></qt>", visitedPages, runningJobs, foundDocuments));
 }
+
+void FindPDFUI::stopSearch() {
+    d->findpdf->abort();
+    searchFinished();
+}
+
+void FindPDFUI::abort() {
+    d->findpdf->abort();
+}
