@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "fieldlistedit.h"
@@ -27,19 +27,20 @@
 #include <QCheckBox>
 #include <QDragEnterEvent>
 #include <QDropEvent>
+#include <QMimeData>
 #include <QUrl>
 #include <QTimer>
 #include <QAction>
+#include <QPushButton>
+#include <QFontDatabase>
+#include <QFileDialog>
+#include <QMenu>
 
 #include <KMessageBox>
-#include <KLocale>
-#include <KPushButton>
-#include <KFileDialog>
-#include <KInputDialog>
+#include <KLocalizedString>
 #include <KIO/CopyJob>
-#include <KGlobalSettings>
-#include <KMenu>
-#include <KDebug>
+#include <KSharedConfig>
+#include <KConfigGroup>
 
 #include "fileinfo.h"
 #include "file.h"
@@ -49,6 +50,7 @@
 #include "fieldlineedit.h"
 #include "associatedfiles.h"
 #include "associatedfilesui.h"
+#include "logging_gui.h"
 
 class FieldListEdit::FieldListEditProtected
 {
@@ -64,7 +66,7 @@ public:
     QList<FieldLineEdit *> lineEditList;
     QWidget *pushButtonContainer;
     QBoxLayout *pushButtonContainerLayout;
-    KPushButton *addLineButton;
+    QPushButton *addLineButton;
     const File *file;
     QString fieldKey;
     QWidget *container;
@@ -73,18 +75,15 @@ public:
     QStringList completionItems;
 
     FieldListEditProtected(KBibTeX::TypeFlag ptf, KBibTeX::TypeFlags tf, FieldListEdit *parent)
-            : p(parent), innerSpacing(4), preferredTypeFlag(ptf), typeFlags(tf), file(NULL), m_isReadOnly(false) {
+            : p(parent), innerSpacing(4), preferredTypeFlag(ptf), typeFlags(tf), file(nullptr), m_isReadOnly(false) {
         smRemove = new QSignalMapper(parent);
         smGoUp = new QSignalMapper(parent);
         smGoDown = new QSignalMapper(parent);
         setupGUI();
     }
 
-    ~FieldListEditProtected() {
-        delete smRemove;
-        delete smGoUp;
-        delete smGoDown;
-    }
+    FieldListEditProtected(const FieldListEditProtected &other) = delete;
+    FieldListEditProtected &operator= (const FieldListEditProtected &other) = delete;
 
     void setupGUI() {
         QBoxLayout *outerLayout = new QVBoxLayout(p);
@@ -105,24 +104,24 @@ public:
         pushButtonContainerLayout->setMargin(0);
         layout->addWidget(pushButtonContainer);
 
-        addLineButton = new KPushButton(KIcon("list-add"), i18n("Add"), pushButtonContainer);
-        addLineButton->setObjectName(QLatin1String("addButton"));
-        connect(addLineButton, SIGNAL(clicked()), p, SLOT(lineAdd()));
-        connect(addLineButton, SIGNAL(clicked()), p, SIGNAL(modified()));
+        addLineButton = new QPushButton(QIcon::fromTheme(QStringLiteral("list-add")), i18n("Add"), pushButtonContainer);
+        addLineButton->setObjectName(QStringLiteral("addButton"));
+        connect(addLineButton, &QPushButton::clicked, p, static_cast<void(FieldListEdit::*)()>(&FieldListEdit::lineAdd));
+        connect(addLineButton, &QPushButton::clicked, p, &FieldListEdit::modified);
         pushButtonContainerLayout->addWidget(addLineButton);
 
         layout->addStretch(100);
 
-        connect(smRemove, SIGNAL(mapped(QWidget*)), p, SLOT(lineRemove(QWidget*)));
-        connect(smGoDown, SIGNAL(mapped(QWidget*)), p, SLOT(lineGoDown(QWidget*)));
-        connect(smGoUp, SIGNAL(mapped(QWidget*)), p, SLOT(lineGoUp(QWidget*)));
+        connect(smRemove, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), p, &FieldListEdit::lineRemove);
+        connect(smGoDown, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), p, &FieldListEdit::lineGoDown);
+        connect(smGoUp, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), p, &FieldListEdit::lineGoUp);
 
         scrollArea->setBackgroundRole(QPalette::Base);
         scrollArea->ensureWidgetVisible(container);
         scrollArea->setWidgetResizable(true);
     }
 
-    void addButton(KPushButton *button) {
+    void addButton(QPushButton *button) {
         button->setParent(pushButtonContainer);
         pushButtonContainerLayout->addWidget(button);
     }
@@ -130,8 +129,8 @@ public:
     int recommendedHeight() {
         int heightHint = 0;
 
-        for (QList<FieldLineEdit *>::ConstIterator it = lineEditList.constBegin(); it != lineEditList.constEnd(); ++it)
-            heightHint += (*it)->sizeHint().height();
+        for (const auto *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(lineEditList))
+            heightHint += fieldLineEdit->sizeHint().height();
 
         heightHint += lineEditList.count() * innerSpacing;
         heightHint += addLineButton->sizeHint().height();
@@ -148,25 +147,25 @@ public:
         layout->insertWidget(layout->count() - 2, le);
         lineEditList.append(le);
 
-        KPushButton *remove = new KPushButton(KIcon("list-remove"), QLatin1String(""), le);
+        QPushButton *remove = new QPushButton(QIcon::fromTheme(QStringLiteral("list-remove")), QString(), le);
         remove->setToolTip(i18n("Remove value"));
         le->appendWidget(remove);
-        connect(remove, SIGNAL(clicked()), smRemove, SLOT(map()));
+        connect(remove, &QPushButton::clicked, smRemove, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
         smRemove->setMapping(remove, le);
 
-        KPushButton *goDown = new KPushButton(KIcon("go-down"), QLatin1String(""), le);
+        QPushButton *goDown = new QPushButton(QIcon::fromTheme(QStringLiteral("go-down")), QString(), le);
         goDown->setToolTip(i18n("Move value down"));
         le->appendWidget(goDown);
-        connect(goDown, SIGNAL(clicked()), smGoDown, SLOT(map()));
+        connect(goDown, &QPushButton::clicked, smGoDown, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
         smGoDown->setMapping(goDown, le);
 
-        KPushButton *goUp = new KPushButton(KIcon("go-up"), QLatin1String(""), le);
+        QPushButton *goUp = new QPushButton(QIcon::fromTheme(QStringLiteral("go-up")), QString(), le);
         goUp->setToolTip(i18n("Move value up"));
         le->appendWidget(goUp);
-        connect(goUp, SIGNAL(clicked()), smGoUp, SLOT(map()));
+        connect(goUp, &QPushButton::clicked, smGoUp, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
         smGoUp->setMapping(goUp, le);
 
-        connect(le, SIGNAL(textChanged(QString)), p, SIGNAL(modified()));
+        connect(le, &FieldLineEdit::textChanged, p, &FieldListEdit::modified);
 
         return le;
     }
@@ -230,9 +229,9 @@ FieldListEdit::~FieldListEdit()
 bool FieldListEdit::reset(const Value &value)
 {
     d->removeAllFieldLineEdits();
-    for (Value::ConstIterator it = value.constBegin(); it != value.constEnd(); ++it) {
+    for (const auto &valueItem : value) {
         Value v;
-        v.append(*it);
+        v.append(valueItem);
         FieldLineEdit *fieldLineEdit = addFieldLineEdit();
         fieldLineEdit->setFile(d->file);
         fieldLineEdit->reset(v);
@@ -247,16 +246,25 @@ bool FieldListEdit::apply(Value &value) const
 {
     value.clear();
 
-    for (QList<FieldLineEdit *>::ConstIterator it = d->lineEditList.constBegin(); it != d->lineEditList.constEnd(); ++it) {
+    for (const auto *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(d->lineEditList)) {
         Value v;
-        (*it)->apply(v);
-        for (Value::ConstIterator itv = v.constBegin(); itv != v.constEnd(); ++itv)
-            value.append(*itv);
+        fieldLineEdit->apply(v);
+        for (const auto &valueItem : const_cast<const Value &>(v))
+            value.append(valueItem);
     }
 
     return true;
 }
 
+bool FieldListEdit::validate(QWidget **widgetWithIssue, QString &message) const
+{
+    for (const auto *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(d->lineEditList)) {
+        const bool v = fieldLineEdit->validate(widgetWithIssue, message);
+        if (!v) return false;
+    }
+    return true;
+}
+
 void FieldListEdit::clear()
 {
     d->removeAllFieldLineEdits();
@@ -265,37 +273,37 @@ void FieldListEdit::clear()
 void FieldListEdit::setReadOnly(bool isReadOnly)
 {
     d->m_isReadOnly = isReadOnly;
-    for (QList<FieldLineEdit *>::ConstIterator it = d->lineEditList.constBegin(); it != d->lineEditList.constEnd(); ++it)
-        (*it)->setReadOnly(isReadOnly);
+    for (FieldLineEdit *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(d->lineEditList))
+        fieldLineEdit->setReadOnly(isReadOnly);
     d->addLineButton->setEnabled(!isReadOnly);
 }
 
 void FieldListEdit::setFile(const File *file)
 {
     d->file = file;
-    for (QList<FieldLineEdit *>::ConstIterator it = d->lineEditList.constBegin(); it != d->lineEditList.constEnd(); ++it)
-        (*it)->setFile(file);
+    for (FieldLineEdit *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(d->lineEditList))
+        fieldLineEdit->setFile(file);
 }
 
 void FieldListEdit::setElement(const Element *element)
 {
     m_element = element;
-    for (QList<FieldLineEdit *>::ConstIterator it = d->lineEditList.constBegin(); it != d->lineEditList.constEnd(); ++it)
-        (*it)->setElement(element);
+    for (FieldLineEdit *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(d->lineEditList))
+        fieldLineEdit->setElement(element);
 }
 
 void FieldListEdit::setFieldKey(const QString &fieldKey)
 {
     d->fieldKey = fieldKey;
-    for (QList<FieldLineEdit *>::ConstIterator it = d->lineEditList.constBegin(); it != d->lineEditList.constEnd(); ++it)
-        (*it)->setFieldKey(fieldKey);
+    for (FieldLineEdit *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(d->lineEditList))
+        fieldLineEdit->setFieldKey(fieldKey);
 }
 
 void FieldListEdit::setCompletionItems(const QStringList &items)
 {
     d->completionItems = items;
-    for (QList<FieldLineEdit *>::Iterator it = d->lineEditList.begin(); it != d->lineEditList.end(); ++it)
-        (*it)->setCompletionItems(items);
+    for (FieldLineEdit *fieldLineEdit : const_cast<const QList<FieldLineEdit *> &>(d->lineEditList))
+        fieldLineEdit->setCompletionItems(items);
 }
 
 FieldLineEdit *FieldListEdit::addFieldLineEdit()
@@ -303,14 +311,14 @@ FieldLineEdit *FieldListEdit::addFieldLineEdit()
     return d->addFieldLineEdit();
 }
 
-void FieldListEdit::addButton(KPushButton *button)
+void FieldListEdit::addButton(QPushButton *button)
 {
     d->addButton(button);
 }
 
 void FieldListEdit::dragEnterEvent(QDragEnterEvent *event)
 {
-    if (event->mimeData()->hasFormat("text/plain") || event->mimeData()->hasFormat("text/x-bibtex"))
+    if (event->mimeData()->hasFormat(QStringLiteral("text/plain")) || event->mimeData()->hasFormat(QStringLiteral("text/x-bibtex")))
         event->acceptProposedAction();
 }
 
@@ -319,18 +327,19 @@ void FieldListEdit::dropEvent(QDropEvent *event)
     const QString clipboardText = event->mimeData()->text();
     if (clipboardText.isEmpty()) return;
 
-    const File *file = NULL;
-    if (!d->fieldKey.isEmpty() && clipboardText.startsWith(QLatin1String("@"))) {
-        FileImporterBibTeX importer;
+    const File *file = nullptr;
+    if (!d->fieldKey.isEmpty() && clipboardText.startsWith(QStringLiteral("@"))) {
+        FileImporterBibTeX importer(this);
         file = importer.fromString(clipboardText);
-        const QSharedPointer<Entry> entry = (file != NULL && file->count() == 1) ? file->first().dynamicCast<Entry>() : QSharedPointer<Entry>();
+        const QSharedPointer<Entry> entry = (file != nullptr && file->count() == 1) ? file->first().dynamicCast<Entry>() : QSharedPointer<Entry>();
 
-        if (file != NULL && !entry.isNull() && d->fieldKey == QLatin1String("^external")) {
+        if (file != nullptr && !entry.isNull() && d->fieldKey == QStringLiteral("^external")) {
             /// handle "external" list differently
-            QList<KUrl> urlList = FileInfo::entryUrls(entry.data(), KUrl(file->property(File::Url).toUrl()), FileInfo::TestExistenceNo);
+            const auto urlList = FileInfo::entryUrls(entry, QUrl(file->property(File::Url).toUrl()), FileInfo::TestExistenceNo);
             Value v;
-            foreach (const KUrl &url, urlList) {
-                v.append(QSharedPointer<VerbatimText>(new VerbatimText(url.pathOrUrl())));
+            v.reserve(urlList.size());
+            for (const QUrl &url : urlList) {
+                v.append(QSharedPointer<VerbatimText>(new VerbatimText(url.url(QUrl::PreferLocalFile))));
             }
             reset(v);
             emit modified();
@@ -343,7 +352,7 @@ void FieldListEdit::dropEvent(QDropEvent *event)
         }
     }
 
-    if (file == NULL || file->count() == 0) {
+    if (file == nullptr || file->count() == 0) {
         /// fall-back case: single field line edit with text
         d->removeAllFieldLineEdits();
         FieldLineEdit *fle = addFieldLineEdit();
@@ -356,7 +365,7 @@ void FieldListEdit::lineAdd(Value *value)
 {
     FieldLineEdit *le = addFieldLineEdit();
     le->setCompletionItems(d->completionItems);
-    if (value != NULL)
+    if (value != nullptr)
         le->reset(*value);
 }
 
@@ -396,14 +405,15 @@ PersonListEdit::PersonListEdit(KBibTeX::TypeFlag preferredTypeFlag, KBibTeX::Typ
         : FieldListEdit(preferredTypeFlag, typeFlags, parent)
 {
     m_checkBoxOthers = new QCheckBox(i18n("... and others (et al.)"), this);
-    connect(m_checkBoxOthers, SIGNAL(toggled(bool)), this, SIGNAL(modified()));
+    connect(m_checkBoxOthers, &QCheckBox::toggled, this, &PersonListEdit::modified);
     QBoxLayout *boxLayout = static_cast<QBoxLayout *>(layout());
     boxLayout->addWidget(m_checkBoxOthers);
 
-    m_buttonAddNamesFromClipboard = new KPushButton(KIcon("edit-paste"), i18n("Add from Clipboard"), this);
+    m_buttonAddNamesFromClipboard = new QPushButton(QIcon::fromTheme(QStringLiteral("edit-paste")), i18n("Add from Clipboard"), this);
     m_buttonAddNamesFromClipboard->setToolTip(i18n("Add a list of names from clipboard"));
     addButton(m_buttonAddNamesFromClipboard);
-    connect(m_buttonAddNamesFromClipboard, SIGNAL(clicked()), this, SLOT(slotAddNamesFromClipboard()));
+
+    connect(m_buttonAddNamesFromClipboard, &QPushButton::clicked, this, &PersonListEdit::slotAddNamesFromClipboard);
 }
 
 bool PersonListEdit::reset(const Value &value)
@@ -413,7 +423,7 @@ bool PersonListEdit::reset(const Value &value)
     m_checkBoxOthers->setCheckState(Qt::Unchecked);
     QSharedPointer<PlainText> pt;
     if (!internal.isEmpty() && !(pt = internal.last().dynamicCast<PlainText>()).isNull()) {
-        if (pt->text() == QLatin1String("others")) {
+        if (pt->text() == QStringLiteral("others")) {
             internal.erase(internal.end() - 1);
             m_checkBoxOthers->setCheckState(Qt::Checked);
         }
@@ -427,7 +437,7 @@ bool PersonListEdit::apply(Value &value) const
     bool result = FieldListEdit::apply(value);
 
     if (result && m_checkBoxOthers->checkState() == Qt::Checked)
-        value.append(QSharedPointer<PlainText>(new PlainText(QLatin1String("others"))));
+        value.append(QSharedPointer<PlainText>(new PlainText(QStringLiteral("others"))));
 
     return result;
 }
@@ -447,7 +457,7 @@ void PersonListEdit::slotAddNamesFromClipboard()
         text = clipboard->text(QClipboard::Selection);
     if (!text.isEmpty()) {
         const QList<QSharedPointer<Person> > personList = FileImporterBibTeX::splitNames(text);
-        foreach (const QSharedPointer<Person> &person, personList) {
+        for (const QSharedPointer<Person> &person : personList) {
             Value *value = new Value();
             value->append(person);
             lineAdd(value);
@@ -463,48 +473,42 @@ UrlListEdit::UrlListEdit(QWidget *parent)
         : FieldListEdit(KBibTeX::tfVerbatim, KBibTeX::tfVerbatim, parent)
 {
     m_signalMapperSaveLocallyButtonClicked = new QSignalMapper(this);
-    connect(m_signalMapperSaveLocallyButtonClicked, SIGNAL(mapped(QWidget*)), this, SLOT(slotSaveLocally(QWidget*)));
+    connect(m_signalMapperSaveLocallyButtonClicked, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), this, &UrlListEdit::slotSaveLocally);
     m_signalMapperFieldLineEditTextChanged = new QSignalMapper(this);
-    connect(m_signalMapperFieldLineEditTextChanged, SIGNAL(mapped(QWidget*)), this, SLOT(textChanged(QWidget*)));
+    connect(m_signalMapperFieldLineEditTextChanged, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), this, &UrlListEdit::textChanged);
 
-    m_buttonAddFile = new KPushButton(KIcon("list-add"), i18n("Add file..."), this);
+    m_buttonAddFile = new QPushButton(QIcon::fromTheme(QStringLiteral("list-add")), i18n("Add file..."), this);
     addButton(m_buttonAddFile);
-    KMenu *menuAddFile = new KMenu(m_buttonAddFile);
+    QMenu *menuAddFile = new QMenu(m_buttonAddFile);
     m_buttonAddFile->setMenu(menuAddFile);
-    connect(m_buttonAddFile, SIGNAL(clicked()), m_buttonAddFile, SLOT(showMenu()));
+    connect(m_buttonAddFile, &QPushButton::clicked, m_buttonAddFile, &QPushButton::showMenu);
 
-    menuAddFile->addAction(KIcon("emblem-symbolic-link"), i18n("Add reference ..."), this, SLOT(slotAddReference()));
-    menuAddFile->addAction(KIcon("emblem-symbolic-link"), i18n("Add reference from clipboard"), this, SLOT(slotAddReferenceFromClipboard()));
-}
-
-UrlListEdit::~UrlListEdit()
-{
-    delete m_signalMapperSaveLocallyButtonClicked;
-    delete m_signalMapperFieldLineEditTextChanged;
+    menuAddFile->addAction(QIcon::fromTheme(QStringLiteral("emblem-symbolic-link")), i18n("Add reference ..."), this, SLOT(slotAddReference()));
+    menuAddFile->addAction(QIcon::fromTheme(QStringLiteral("emblem-symbolic-link")), i18n("Add reference from clipboard"), this, SLOT(slotAddReferenceFromClipboard()));
 }
 
 void UrlListEdit::slotAddReference()
 {
-    QUrl bibtexUrl(d->file != NULL ? d->file->property(File::Url, QVariant()).toUrl() : QUrl());
+    QUrl bibtexUrl(d->file != nullptr ? d->file->property(File::Url, QVariant()).toUrl() : QUrl());
     if (!bibtexUrl.isEmpty()) {
         const QFileInfo fi(bibtexUrl.path());
         bibtexUrl.setPath(fi.absolutePath());
     }
-    const KUrl documentUrl = KFileDialog::getOpenUrl(bibtexUrl, QString(), this, i18n("File to Associate"));
+    const QUrl documentUrl = QFileDialog::getOpenFileUrl(this, i18n("File to Associate"), bibtexUrl);
     if (!documentUrl.isEmpty())
         addReference(documentUrl);
 }
 
 void UrlListEdit::slotAddReferenceFromClipboard()
 {
-    const KUrl url = KUrl::fromUserInput(QApplication::clipboard()->text());
+    const QUrl url = QUrl::fromUserInput(QApplication::clipboard()->text());
     if (!url.isEmpty())
         addReference(url);
 }
 
 void UrlListEdit::addReference(const QUrl &url) {
     const Entry *entry = dynamic_cast<const Entry *>(m_element);
-    const QString entryId = entry != NULL ? entry->id() : QString();
+    const QString entryId = entry != nullptr ? entry->id() : QString();
     const QString visibleFilename = AssociatedFilesUI::associateUrl(url, entryId, d->file, this);
     if (!visibleFilename.isEmpty()) {
         Value *value = new Value();
@@ -520,18 +524,18 @@ void UrlListEdit::slotSaveLocally(QWidget *widget)
     /// Determine FieldLineEdit widget
     FieldLineEdit *fieldLineEdit = qobject_cast<FieldLineEdit *>(widget);
     /// Build Url from line edit's content
-    const KUrl url = KUrl::fromUserInput(fieldLineEdit->text());
+    const QUrl url = QUrl::fromUserInput(fieldLineEdit->text());
 
     /// Only proceed if Url is valid and points to a remote location
     if (url.isValid() && !urlIsLocal(url)) {
         /// Get filename from url (without any path/directory part)
         QString filename = url.fileName();
         /// Build QFileInfo from current BibTeX file if available
-        QFileInfo bibFileinfo = d->file != NULL ? QFileInfo(d->file->property(File::Url).toUrl().path()) : QFileInfo();
+        QFileInfo bibFileinfo = d->file != nullptr ? QFileInfo(d->file->property(File::Url).toUrl().path()) : QFileInfo();
         /// Build proposal to a local filename for remote file
         filename = bibFileinfo.isFile() ? bibFileinfo.absolutePath() + QDir::separator() + filename : filename;
         /// Ask user for actual local filename to save remote file to
-        filename = KFileDialog::getSaveFileName(filename, QLatin1String("application/pdf application/postscript image/vnd.djvu"), this, i18n("Save file locally"));
+        filename = QFileDialog::getSaveFileName(this, i18n("Save file locally"), filename, QStringLiteral("application/pdf application/postscript image/vnd.djvu"));
         /// Check if user entered a valid filename ...
         if (!filename.isEmpty()) {
             /// Ask user if reference to local file should be
@@ -542,12 +546,11 @@ void UrlListEdit::slotSaveLocally(QWidget *widget)
                 visibleFilename = askRelativeOrStaticFilename(this, absoluteFilename, d->file->property(File::Url).toUrl());
 
             /// Download remote file and save it locally
-            // FIXME: KIO::NetAccess::download is blocking
             setEnabled(false);
             setCursor(Qt::WaitCursor);
             KIO::CopyJob *job = KIO::copy(url, QUrl::fromLocalFile(absoluteFilename), KIO::Overwrite);
             job->setProperty("visibleFilename", QVariant::fromValue<QString>(visibleFilename));
-            connect(job, SIGNAL(result(KJob*)), this, SLOT(downloadFinished(KJob*)));
+            connect(job, &KJob::result, this, &UrlListEdit::downloadFinished);
         }
     }
 }
@@ -561,7 +564,7 @@ void UrlListEdit::downloadFinished(KJob *j) {
         lineAdd(value);
         delete value;
     } else {
-        kWarning() << "Downloading" << (*job->srcUrls().constBegin()).pathOrUrl() << "failed with error" << job->error() << job->errorString();
+        qCWarning(LOG_KBIBTEX_GUI) << "Downloading" << (*job->srcUrls().constBegin()).toDisplayString() << "failed with error" << job->error() << job->errorString();
     }
     setEnabled(true);
     unsetCursor();
@@ -569,13 +572,13 @@ void UrlListEdit::downloadFinished(KJob *j) {
 
 void UrlListEdit::textChanged(QWidget *widget)
 {
-    /// Determine associated KPushButton "Save locally"
-    KPushButton *buttonSaveLocally = qobject_cast<KPushButton *>(widget);
-    if (buttonSaveLocally == NULL) return; ///< should never happen!
+    /// Determine associated QPushButton "Save locally"
+    QPushButton *buttonSaveLocally = qobject_cast<QPushButton *>(widget);
+    if (buttonSaveLocally == nullptr) return; ///< should never happen!
 
     /// Assume a FieldLineEdit was the sender of this signal
     FieldLineEdit *fieldLineEdit = qobject_cast<FieldLineEdit *>(m_signalMapperFieldLineEditTextChanged->mapping(widget));
-    if (fieldLineEdit == NULL) return; ///< should never happen!
+    if (fieldLineEdit == nullptr) return; ///< should never happen!
 
     /// Create URL from new text to make some tests on it
     /// Only remote URLs are of interest, therefore no tests
@@ -586,9 +589,9 @@ void UrlListEdit::textChanged(QWidget *widget)
     /// Enable button only if Url is valid and points to a remote
     /// DjVu, PDF, or PostScript file
     // TODO more file types?
-    const bool canBeSaved = lowerText.contains(QLatin1String("://")) && (lowerText.endsWith(QLatin1String(".djvu")) || lowerText.endsWith(QLatin1String(".pdf")) || lowerText.endsWith(QLatin1String(".ps")));
+    const bool canBeSaved = lowerText.contains(QStringLiteral("://")) && (lowerText.endsWith(QStringLiteral(".djvu")) || lowerText.endsWith(QStringLiteral(".pdf")) || lowerText.endsWith(QStringLiteral(".ps")));
     buttonSaveLocally->setEnabled(canBeSaved);
-    buttonSaveLocally->setToolTip(canBeSaved ? i18n("Save file '%1' locally", newText) : QLatin1String(""));
+    buttonSaveLocally->setToolTip(canBeSaved ? i18n("Save file '%1' locally", newText) : QString());
 }
 
 QString UrlListEdit::askRelativeOrStaticFilename(QWidget *parent, const QString &absoluteFilename, const QUrl &baseUrl)
@@ -598,8 +601,8 @@ QString UrlListEdit::askRelativeOrStaticFilename(QWidget *parent, const QString
     if (!baseUrl.isEmpty() && (filenameInfo.absolutePath() == baseUrlInfo.absolutePath() || filenameInfo.absolutePath().startsWith(baseUrlInfo.absolutePath() + QDir::separator()))) {
         // TODO cover level-up cases like "../../test.pdf"
         const QString relativePath = filenameInfo.absolutePath().mid(baseUrlInfo.absolutePath().length() + 1);
-        const QString relativeFilename = relativePath + (relativePath.isEmpty() ? QLatin1String("") : QString(QDir::separator())) + filenameInfo.fileName();
-        if (KMessageBox::questionYesNo(parent, i18n("<qt><p>Use a filename relative to the bibliography file?</p><p>The relative path would be<br/><tt style=\"font-family: %3;\">%1</tt></p><p>The absolute path would be<br/><tt style=\"font-family: %3;\">%2</tt></p></qt>", relativeFilename, absoluteFilename, KGlobalSettings::fixedFont().family()), i18n("Relative Path"), KGuiItem(i18n("Relative Path")), KGuiItem(i18n("Absolute Path"))) == KMessageBox::Yes)
+        const QString relativeFilename = relativePath + (relativePath.isEmpty() ? QString() : QString(QDir::separator())) + filenameInfo.fileName();
+        if (KMessageBox::questionYesNo(parent, i18n("<qt><p>Use a filename relative to the bibliography file?</p><p>The relative path would be<br/><tt style=\"font-family: %3;\">%1</tt></p><p>The absolute path would be<br/><tt style=\"font-family: %3;\">%2</tt></p></qt>", relativeFilename, absoluteFilename, QFontDatabase::systemFont(QFontDatabase::FixedFont).family()), i18n("Relative Path"), KGuiItem(i18n("Relative Path")), KGuiItem(i18n("Absolute Path"))) == KMessageBox::Yes)
             return relativeFilename;
     }
     return absoluteFilename;
@@ -610,7 +613,7 @@ bool UrlListEdit::urlIsLocal(const QUrl &url)
     // FIXME same function as in AssociateFiles; move to common code base?
     const QString scheme = url.scheme();
     /// Test various schemes such as "http", "https", "ftp", ...
-    return !scheme.startsWith(QLatin1String("http")) && !scheme.startsWith(QLatin1String("ftp")) && !scheme.startsWith(QLatin1String("sftp")) && !scheme.startsWith(QLatin1String("fish")) && !scheme.startsWith(QLatin1String("webdav")) && scheme != QLatin1String("smb");
+    return !scheme.startsWith(QStringLiteral("http")) && !scheme.startsWith(QStringLiteral("ftp")) && !scheme.startsWith(QStringLiteral("sftp")) && !scheme.startsWith(QStringLiteral("fish")) && !scheme.startsWith(QStringLiteral("webdav")) && scheme != QStringLiteral("smb");
 }
 
 FieldLineEdit *UrlListEdit::addFieldLineEdit()
@@ -619,7 +622,7 @@ FieldLineEdit *UrlListEdit::addFieldLineEdit()
     FieldLineEdit *fieldLineEdit = FieldListEdit::addFieldLineEdit();
 
     /// Create a new "save locally" button
-    KPushButton *buttonSaveLocally = new KPushButton(KIcon("document-save"), QLatin1String(""), fieldLineEdit);
+    QPushButton *buttonSaveLocally = new QPushButton(QIcon::fromTheme(QStringLiteral("document-save")), QString(), fieldLineEdit);
     buttonSaveLocally->setToolTip(i18n("Save file locally"));
     buttonSaveLocally->setEnabled(false);
     /// Append button to new FieldLineEdit
@@ -628,8 +631,8 @@ FieldLineEdit *UrlListEdit::addFieldLineEdit()
     /// or changes in the FieldLineEdit's text
     m_signalMapperSaveLocallyButtonClicked->setMapping(buttonSaveLocally, fieldLineEdit);
     m_signalMapperFieldLineEditTextChanged->setMapping(fieldLineEdit, buttonSaveLocally);
-    connect(buttonSaveLocally, SIGNAL(clicked()), m_signalMapperSaveLocallyButtonClicked, SLOT(map()));
-    connect(fieldLineEdit, SIGNAL(textChanged(QString)), m_signalMapperFieldLineEditTextChanged, SLOT(map()));
+    connect(buttonSaveLocally, &QPushButton::clicked, m_signalMapperSaveLocallyButtonClicked, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
+    connect(fieldLineEdit, &FieldLineEdit::textChanged, m_signalMapperFieldLineEditTextChanged, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
 
     return fieldLineEdit;
 }
@@ -641,19 +644,19 @@ void UrlListEdit::setReadOnly(bool isReadOnly)
 }
 
 
-const QString KeywordListEdit::keyGlobalKeywordList = QLatin1String("globalKeywordList");
+const QString KeywordListEdit::keyGlobalKeywordList = QStringLiteral("globalKeywordList");
 
 KeywordListEdit::KeywordListEdit(QWidget *parent)
-        : FieldListEdit(KBibTeX::tfKeyword, KBibTeX::tfKeyword | KBibTeX::tfSource, parent), m_config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), m_configGroupName(QLatin1String("Global Keywords"))
+        : FieldListEdit(KBibTeX::tfKeyword, KBibTeX::tfKeyword | KBibTeX::tfSource, parent), m_config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), m_configGroupName(QStringLiteral("Global Keywords"))
 {
-    m_buttonAddKeywordsFromList = new KPushButton(KIcon("list-add"), i18n("Add Keywords from List"), this);
+    m_buttonAddKeywordsFromList = new QPushButton(QIcon::fromTheme(QStringLiteral("list-add")), i18n("Add Keywords from List"), this);
     m_buttonAddKeywordsFromList->setToolTip(i18n("Add keywords as selected from a pre-defined list of keywords"));
     addButton(m_buttonAddKeywordsFromList);
-    connect(m_buttonAddKeywordsFromList, SIGNAL(clicked()), this, SLOT(slotAddKeywordsFromList()));
-    m_buttonAddKeywordsFromClipboard = new KPushButton(KIcon("edit-paste"), i18n("Add Keywords from Clipboard"), this);
+    connect(m_buttonAddKeywordsFromList, &QPushButton::clicked, this, &KeywordListEdit::slotAddKeywordsFromList);
+    m_buttonAddKeywordsFromClipboard = new QPushButton(QIcon::fromTheme(QStringLiteral("edit-paste")), i18n("Add Keywords from Clipboard"), this);
     m_buttonAddKeywordsFromClipboard->setToolTip(i18n("Add a punctuation-separated list of keywords from clipboard"));
     addButton(m_buttonAddKeywordsFromClipboard);
-    connect(m_buttonAddKeywordsFromClipboard, SIGNAL(clicked()), this, SLOT(slotAddKeywordsFromClipboard()));
+    connect(m_buttonAddKeywordsFromClipboard, &QPushButton::clicked, this, &KeywordListEdit::slotAddKeywordsFromClipboard);
 }
 
 void KeywordListEdit::slotAddKeywordsFromList()
@@ -666,18 +669,20 @@ void KeywordListEdit::slotAddKeywordsFromList()
     /// (recommended by Qt's documentation)
     QMap<QString, QString> forCaseInsensitiveSorting;
     /// insert all stored, global keywords
-    foreach (const QString &keyword, keywords)
+    for (const QString &keyword : const_cast<const QStringList &>(keywords))
         forCaseInsensitiveSorting.insert(keyword.toLower(), keyword);
     /// insert all unique keywords used in this file
-    foreach (const QString &keyword, m_keywordsFromFile)
+    for (const QString &keyword : const_cast<const QSet<QString> &>(m_keywordsFromFile))
         forCaseInsensitiveSorting.insert(keyword.toLower(), keyword);
     /// re-create string list from map's values
     keywords = forCaseInsensitiveSorting.values();
 
+    // FIXME QInputDialog does not have a 'getItemList'
+    /*
     bool ok = false;
     const QStringList newKeywordList = KInputDialog::getItemList(i18n("Add Keywords"), i18n("Select keywords to add:"), keywords, QStringList(), true, &ok, this);
     if (ok) {
-        foreach (const QString &newKeywordText, newKeywordList) {
+        for(const QString &newKeywordText : newKeywordList) {
             Value *value = new Value();
             value->append(QSharedPointer<Keyword>(new Keyword(newKeywordText)));
             lineAdd(value);
@@ -686,6 +691,7 @@ void KeywordListEdit::slotAddKeywordsFromList()
         if (!newKeywordList.isEmpty())
             emit modified();
     }
+    */
 }
 
 void KeywordListEdit::slotAddKeywordsFromClipboard()
@@ -696,9 +702,9 @@ void KeywordListEdit::slotAddKeywordsFromClipboard()
         text = clipboard->text(QClipboard::Selection);
     if (!text.isEmpty()) {
         const QList<QSharedPointer<Keyword> > keywords = FileImporterBibTeX::splitKeywords(text);
-        for (QList<QSharedPointer<Keyword> >::ConstIterator it = keywords.constBegin(); it != keywords.constEnd(); ++it) {
+        for (const auto &keyword : keywords) {
             Value *value = new Value();
-            value->append(*it);
+            value->append(keyword);
             lineAdd(value);
             delete value;
         }
@@ -716,7 +722,7 @@ void KeywordListEdit::setReadOnly(bool isReadOnly)
 
 void KeywordListEdit::setFile(const File *file)
 {
-    if (file == NULL)
+    if (file == nullptr)
         m_keywordsFromFile.clear();
     else
         m_keywordsFromFile = file->uniqueEntryValuesSet(Entry::ftKeywords);
@@ -734,10 +740,10 @@ void KeywordListEdit::setCompletionItems(const QStringList &items)
     /// (recommended by Qt's documentation)
     QMap<QString, QString> forCaseInsensitiveSorting;
     /// insert all stored, global keywords
-    foreach (const QString &keyword, keywords)
+    for (const QString &keyword : const_cast<const QStringList &>(keywords))
         forCaseInsensitiveSorting.insert(keyword.toLower(), keyword);
     /// insert all unique keywords used in this file
-    foreach (const QString &keyword, items)
+    for (const QString &keyword : const_cast<const QStringList &>(items))
         forCaseInsensitiveSorting.insert(keyword.toLower(), keyword);
     /// re-create string list from map's values
     keywords = forCaseInsensitiveSorting.values();
