@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #ifndef KBIBTEX_GUI_SETTINGSIDSUGGESTIONSEDITOR_H
@@ -22,10 +22,9 @@
 
 #include <QWidget>
 #include <QLayout>
+#include <QDialog>
 #include <QGroupBox>
 
-#include <KDialog>
-
 #include "idsuggestions.h"
 #include "entry.h"
 
@@ -33,110 +32,48 @@ class QFormLayout;
 class QSpinBox;
 class QLabel;
 class QCheckBox;
+class QPushButton;
 
 class KLineEdit;
 class KComboBox;
 
-class QxtSpanSlider;
-
 class IdSuggestionsEditWidget;
+class RangeWidget;
 
 /**
  * @author Thomas Fischer
  */
-class TokenWidget : public QGroupBox
-{
-    Q_OBJECT
-
-protected:
-    QGridLayout *gridLayout;
-    QFormLayout *formLayout;
-
-public:
-    explicit TokenWidget(QWidget *parent);
-
-    void addButtons(KPushButton *buttonUp, KPushButton *buttonDown, KPushButton *buttonRemove);
-
-    virtual QString toString() const = 0;
-};
-
-/**
- * @author Thomas Fischer
- */
-class AuthorWidget : public TokenWidget
-{
-    Q_OBJECT
-
-private:
-    QxtSpanSlider *spanSliderAuthor;
-    QCheckBox *checkBoxLastAuthor;
-    QLabel *labelAuthorRange;
-    KComboBox *comboBoxChangeCase;
-    KLineEdit *lineEditTextInBetween;
-    QSpinBox *spinBoxLength;
-
-private slots:
-    void updateRangeLabel();
-
-public:
-    AuthorWidget(const struct IdSuggestions::IdSuggestionTokenInfo &info, IdSuggestionsEditWidget *isew, QWidget *parent);
-
-    QString toString() const;
-};
-
-/**
- * @author Thomas Fischer
- */
-class TitleWidget : public TokenWidget
-{
-    Q_OBJECT
-
-private:
-    QxtSpanSlider *spanSliderWords;
-    QLabel *labelWordsRange;
-    QCheckBox *checkBoxRemoveSmallWords;
-    KComboBox *comboBoxChangeCase;
-    KLineEdit *lineEditTextInBetween;
-    QSpinBox *spinBoxLength;
-
-private slots:
-    void updateRangeLabel();
-
-public:
-    TitleWidget(const struct IdSuggestions::IdSuggestionTokenInfo &info, bool removeSmallWords, IdSuggestionsEditWidget *isew, QWidget *parent);
-
-    QString toString() const;
-};
-
-
-/**
- * @author Thomas Fischer
- */
-class KBIBTEXGUI_EXPORT IdSuggestionsEditDialog : public KDialog
+class KBIBTEXGUI_EXPORT IdSuggestionsEditDialog : public QDialog
 {
     Q_OBJECT
 
 public:
-    virtual ~IdSuggestionsEditDialog();
+    ~IdSuggestionsEditDialog() override;
 
     static QString editSuggestion(const Entry *previewEntry, const QString &suggestion, QWidget *parent);
+
 protected:
-    explicit IdSuggestionsEditDialog(QWidget *parent = 0, Qt::WFlags flags = 0);
+    explicit IdSuggestionsEditDialog(QWidget *parent = nullptr, Qt::WindowFlags flags = 0);
 };
 
+/**
+ * @author Thomas Fischer
+ */
 class IdSuggestionsEditWidget : public QWidget, public IdSuggestions
 {
     Q_OBJECT
 
 public:
-    explicit IdSuggestionsEditWidget(const Entry *previewEntry, QWidget *parent = 0, Qt::WindowFlags f = 0);
-    virtual ~IdSuggestionsEditWidget();
+    explicit IdSuggestionsEditWidget(const Entry *previewEntry, QWidget *parent = nullptr, Qt::WindowFlags f = 0);
+    ~IdSuggestionsEditWidget() override;
 
     void setFormatString(const QString &formatString);
     QString formatString() const;
 
-private slots:
+public slots:
     void updatePreview();
+
+private slots:
     void moveUpToken(QWidget *);
     void moveDownToken(QWidget *);
     void removeToken(QWidget *);
