@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,30 +12,29 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "settingsidsuggestionswidget.h"
 
 #include <QLayout>
 #include <QTreeView>
+#include <QPushButton>
+#include <QFontDatabase>
 
 #include <KSharedConfig>
 #include <KConfigGroup>
-#include <KPushButton>
-#include <KLocale>
-#include <KGlobalSettings>
+#include <KLocalizedString>
 #include <KMessageBox>
 
 #include "fileimporterbibtex.h"
 #include "idsuggestions.h"
 #include "settingsidsuggestionseditor.h"
 
-const int FormatStringRole = Qt::UserRole + 7811;
-const int IsDefaultFormatStringRole = Qt::UserRole + 7812;
-
 class IdSuggestionsModel : public QAbstractListModel
 {
+    Q_OBJECT
+
 private:
     QStringList m_formatStringList;
     int m_defaultFormatStringRow;
@@ -44,15 +43,15 @@ private:
     static QSharedPointer<const Entry> exampleBibTeXEntry;
 
 public:
-    IdSuggestionsModel(QObject *parent = NULL)
+    IdSuggestionsModel(QObject *parent = nullptr)
             : QAbstractListModel(parent) {
         m_idSuggestions = new IdSuggestions();
         m_defaultFormatStringRow = -1;
 
         if (exampleBibTeXEntry.isNull()) {
-            static FileImporterBibTeX fileImporterBibTeX;
+            FileImporterBibTeX fileImporterBibTeX(this);
             File *file = fileImporterBibTeX.fromString(exampleBibTeXEntryString);
-            if (file != NULL) {
+            if (file != nullptr) {
                 if (!file->isEmpty())
                     exampleBibTeXEntry = file->first().dynamicCast<const Entry>();
                 delete file;
@@ -60,18 +59,26 @@ public:
         }
     }
 
-    ~IdSuggestionsModel() {
+    ~IdSuggestionsModel() override {
         delete m_idSuggestions;
     }
 
+    enum IdSuggestionsModelRole {
+        /// Raw format string as QString
+        FormatStringRole = Qt::UserRole + 7811,
+        /// Flag whether current index is the default one; boolean value
+        IsDefaultFormatStringRole = Qt::UserRole + 7812
+    };
+
     QSharedPointer<const Entry> previewEntry() {
         return exampleBibTeXEntry;
     }
 
     void setFormatStringList(const QStringList &formatStringList, const QString &defaultString = QString()) {
+        beginResetModel();
         m_formatStringList = formatStringList;
         m_defaultFormatStringRow = m_formatStringList.indexOf(defaultString);
-        reset();
+        endResetModel();
     }
 
     QStringList formatStringList() const {
@@ -85,31 +92,31 @@ public:
             return QString();
     }
 
-    int rowCount(const QModelIndex &parent = QModelIndex()) const {
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override {
         if (parent == QModelIndex())
             return m_formatStringList.count();
         else
             return 0;
     }
 
-    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const {
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override {
         if (index.row() < 0 || index.row() >= m_formatStringList.count())
             return QVariant();
 
         switch (role) {
         case Qt::FontRole: {
-            QFont defaultFont = KGlobalSettings::generalFont();
+            QFont defaultFont = QFontDatabase::systemFont(QFontDatabase::GeneralFont);
             if (index.row() == m_defaultFormatStringRow)
                 defaultFont.setBold(true);
             return defaultFont;
         }
         case Qt::DecorationRole:
             if (index.row() == m_defaultFormatStringRow)
-                return KIcon("favorites");
+                return QIcon::fromTheme(QStringLiteral("favorites"));
             else
-                return KIcon("view-filter");
+                return QIcon::fromTheme(QStringLiteral("view-filter"));
         case Qt::ToolTipRole:
-            return i18n("<qt>Structure:<ul><li>%1</li></ul>Example: %2</qt>", m_idSuggestions->formatStrToHuman(m_formatStringList[index.row()]).join(QLatin1String("</li><li>")), m_idSuggestions->formatId(*exampleBibTeXEntry, m_formatStringList[index.row()]));
+            return i18n("<qt>Structure:<ul><li>%1</li></ul>Example: %2</qt>", m_idSuggestions->formatStrToHuman(m_formatStringList[index.row()]).join(QStringLiteral("</li><li>")), m_idSuggestions->formatId(*exampleBibTeXEntry, m_formatStringList[index.row()]));
         case Qt::DisplayRole:
             return m_idSuggestions->formatId(*exampleBibTeXEntry, m_formatStringList[index.row()]);
         case Qt::UserRole:
@@ -122,7 +129,7 @@ public:
         }
     }
 
-    bool setData(const QModelIndex &idx, const QVariant &value, int role) {
+    bool setData(const QModelIndex &idx, const QVariant &value, int role) override {
         if (idx.row() < 0 || idx.row() >= m_formatStringList.count())
             return false;
 
@@ -131,18 +138,18 @@ public:
                 if (idx.row() != m_defaultFormatStringRow) {
                     QModelIndex oldDefaultIndex = index(m_defaultFormatStringRow, 0);
                     m_defaultFormatStringRow = idx.row();
-                    dataChanged(oldDefaultIndex, oldDefaultIndex);
-                    dataChanged(idx, idx);
+                    emit dataChanged(oldDefaultIndex, oldDefaultIndex);
+                    emit dataChanged(idx, idx);
                 }
             } else {
                 m_defaultFormatStringRow = -1;
-                dataChanged(idx, idx);
+                emit dataChanged(idx, idx);
             }
 
             return true;
         } else if (role == FormatStringRole && value.canConvert<QString>()) {
             m_formatStringList[idx.row()] = value.toString();
-            dataChanged(idx, idx);
+            emit dataChanged(idx, idx);
             return true;
         }
 
@@ -153,13 +160,13 @@ public:
         if (parent != QModelIndex()) return false;
 
         beginInsertRows(parent, row, row);
-        m_formatStringList.insert(row, QLatin1String("T"));
+        m_formatStringList.insert(row, QStringLiteral("T"));
         endInsertRows();
 
         return true;
     }
 
-    QVariant headerData(int section, Qt::Orientation, int role = Qt::DisplayRole) const {
+    QVariant headerData(int section, Qt::Orientation, int role = Qt::DisplayRole) const override {
         if (role == Qt::DisplayRole && section == 0)
             return i18n("Id Suggestions");
 
@@ -210,7 +217,7 @@ public:
     }
 };
 
-const QString IdSuggestionsModel::exampleBibTeXEntryString = QLatin1String("@Article{ dijkstra1983terminationdetect,\nauthor = {Edsger W. Dijkstra and W. H. J. Feijen and A. J. M. {van Gasteren}},\ntitle = {{Derivation of a Termination Detection Algorithm for Distributed Computations}},\njournal = {Information Processing Letters},\nvolume = 16,\nnumber = 5,\npages = {217--219},\nmonth = jun,\nyear = 1983\n}");
+const QString IdSuggestionsModel::exampleBibTeXEntryString = QStringLiteral("@Article{ dijkstra1983terminationdetect,\nauthor = {Edsger W. Dijkstra and W. H. J. Feijen and A. J. M. {van Gasteren}},\ntitle = {{Derivation of a Termination Detection Algorithm for Distributed Computations}},\njournal = {Information Processing Letters},\nvolume = 16,\nnumber = 5,\npages = {217--219},\nmonth = jun,\nyear = 1983\n}");
 QSharedPointer<const Entry> IdSuggestionsModel::exampleBibTeXEntry;
 
 class SettingsIdSuggestionsWidget::SettingsIdSuggestionsWidgetPrivate
@@ -224,10 +231,10 @@ private:
 public:
     QTreeView *treeViewSuggestions;
     IdSuggestionsModel *idSuggestionsModel;
-    KPushButton *buttonNewSuggestion, *buttonEditSuggestion, *buttonDeleteSuggestion, *buttonSuggestionUp, *buttonSuggestionDown, *buttonToggleDefaultString;
+    QPushButton *buttonNewSuggestion, *buttonEditSuggestion, *buttonDeleteSuggestion, *buttonSuggestionUp, *buttonSuggestionDown, *buttonToggleDefaultString;
 
     SettingsIdSuggestionsWidgetPrivate(SettingsIdSuggestionsWidget *parent)
-            : p(parent), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), configGroup(config, IdSuggestions::configGroupName) {
+            : p(parent), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), configGroup(config, IdSuggestions::configGroupName) {
         setupGUI();
     }
 
@@ -253,34 +260,34 @@ public:
         idSuggestionsModel = new IdSuggestionsModel(treeViewSuggestions);
         treeViewSuggestions->setModel(idSuggestionsModel);
         treeViewSuggestions->setRootIsDecorated(false);
-        connect(treeViewSuggestions->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), p, SLOT(itemChanged(QModelIndex)));
+        connect(treeViewSuggestions->selectionModel(), &QItemSelectionModel::currentChanged, p, &SettingsIdSuggestionsWidget::itemChanged);
         treeViewSuggestions->setMinimumSize(treeViewSuggestions->fontMetrics().width(QChar('W')) * 25, treeViewSuggestions->fontMetrics().height() * 15);
 
-        buttonNewSuggestion = new KPushButton(KIcon("list-add"), i18n("Add..."), p);
+        buttonNewSuggestion = new QPushButton(QIcon::fromTheme(QStringLiteral("list-add")), i18n("Add..."), p);
         layout->addWidget(buttonNewSuggestion, 0, 1, 1, 1);
 
-        buttonEditSuggestion = new KPushButton(KIcon("document-edit"), i18n("Edit..."), p);
+        buttonEditSuggestion = new QPushButton(QIcon::fromTheme(QStringLiteral("document-edit")), i18n("Edit..."), p);
         layout->addWidget(buttonEditSuggestion, 1, 1, 1, 1);
 
-        buttonDeleteSuggestion = new KPushButton(KIcon("list-remove"), i18n("Remove"), p);
+        buttonDeleteSuggestion = new QPushButton(QIcon::fromTheme(QStringLiteral("list-remove")), i18n("Remove"), p);
         layout->addWidget(buttonDeleteSuggestion, 2, 1, 1, 1);
 
-        buttonSuggestionUp = new KPushButton(KIcon("go-up"), i18n("Up"), p);
+        buttonSuggestionUp = new QPushButton(QIcon::fromTheme(QStringLiteral("go-up")), i18n("Up"), p);
         layout->addWidget(buttonSuggestionUp, 3, 1, 1, 1);
 
-        buttonSuggestionDown = new KPushButton(KIcon("go-down"), i18n("Down"), p);
+        buttonSuggestionDown = new QPushButton(QIcon::fromTheme(QStringLiteral("go-down")), i18n("Down"), p);
         layout->addWidget(buttonSuggestionDown, 4, 1, 1, 1);
 
-        buttonToggleDefaultString = new KPushButton(KIcon("favorites"), i18n("Toggle Default"), p);
+        buttonToggleDefaultString = new QPushButton(QIcon::fromTheme(QStringLiteral("favorites")), i18n("Toggle Default"), p);
         layout->addWidget(buttonToggleDefaultString, 5, 1, 1, 1);
 
-        connect(buttonNewSuggestion, SIGNAL(clicked()), p, SLOT(buttonClicked()));
-        connect(buttonEditSuggestion, SIGNAL(clicked()), p, SLOT(buttonClicked()));
-        connect(buttonDeleteSuggestion, SIGNAL(clicked()), p, SLOT(buttonClicked()));
-        connect(buttonSuggestionUp, SIGNAL(clicked()), p, SLOT(buttonClicked()));
-        connect(buttonSuggestionDown, SIGNAL(clicked()), p, SLOT(buttonClicked()));
-        connect(buttonToggleDefaultString, SIGNAL(clicked()), p, SLOT(toggleDefault()));
-        connect(treeViewSuggestions, SIGNAL(doubleClicked(QModelIndex)), p, SLOT(editItem(QModelIndex)));
+        connect(buttonNewSuggestion, &QPushButton::clicked, p, &SettingsIdSuggestionsWidget::buttonClicked);
+        connect(buttonEditSuggestion, &QPushButton::clicked, p, &SettingsIdSuggestionsWidget::buttonClicked);
+        connect(buttonDeleteSuggestion, &QPushButton::clicked, p, &SettingsIdSuggestionsWidget::buttonClicked);
+        connect(buttonSuggestionUp, &QPushButton::clicked, p, &SettingsIdSuggestionsWidget::buttonClicked);
+        connect(buttonSuggestionDown, &QPushButton::clicked, p, &SettingsIdSuggestionsWidget::buttonClicked);
+        connect(buttonToggleDefaultString, &QPushButton::clicked, p, &SettingsIdSuggestionsWidget::toggleDefault);
+        connect(treeViewSuggestions, &QTreeView::doubleClicked, p, &SettingsIdSuggestionsWidget::editItem);
     }
 };
 
@@ -302,9 +309,9 @@ QString SettingsIdSuggestionsWidget::label() const
     return i18n("Id Suggestions");
 }
 
-KIcon SettingsIdSuggestionsWidget::icon() const
+QIcon SettingsIdSuggestionsWidget::icon() const
 {
-    return KIcon("view-filter");
+    return QIcon::fromTheme(QStringLiteral("view-filter"));
 }
 
 void SettingsIdSuggestionsWidget::loadState()
@@ -324,16 +331,16 @@ void SettingsIdSuggestionsWidget::resetToDefaults()
 
 void SettingsIdSuggestionsWidget::buttonClicked()
 {
-    KPushButton *button = qobject_cast<KPushButton *>(sender());
+    QPushButton *button = qobject_cast<QPushButton *>(sender());
     QModelIndex selectedIndex = d->treeViewSuggestions->selectionModel()->currentIndex();
 
     if (button == d->buttonNewSuggestion) {
-        const QString newSuggestion = IdSuggestionsEditDialog::editSuggestion(d->idSuggestionsModel->previewEntry().data(), QLatin1String(""), this);
+        const QString newSuggestion = IdSuggestionsEditDialog::editSuggestion(d->idSuggestionsModel->previewEntry().data(), QString(), this);
         const int row = d->treeViewSuggestions->model()->rowCount(QModelIndex());
         if (!newSuggestion.isEmpty() && d->idSuggestionsModel->insertRow(row)) {
             QModelIndex index = d->idSuggestionsModel->index(row, 0, QModelIndex());
             d->treeViewSuggestions->setCurrentIndex(index);
-            if (d->idSuggestionsModel->setData(index, newSuggestion, FormatStringRole))
+            if (d->idSuggestionsModel->setData(index, newSuggestion, IdSuggestionsModel::FormatStringRole))
                 emit changed();
         }
     } else if (button == d->buttonEditSuggestion) {
@@ -370,14 +377,14 @@ void SettingsIdSuggestionsWidget::itemChanged(const QModelIndex &index)
 void SettingsIdSuggestionsWidget::editItem(const QModelIndex &index)
 {
     QString suggestion;
-    if (index != QModelIndex() && !(suggestion = index.data(FormatStringRole).toString()).isEmpty()) {
+    if (index != QModelIndex() && !(suggestion = index.data(IdSuggestionsModel::FormatStringRole).toString()).isEmpty()) {
         const QString newSuggestion = IdSuggestionsEditDialog::editSuggestion(d->idSuggestionsModel->previewEntry().data(), suggestion, this);
         if (newSuggestion.isEmpty()) {
-            if (KMessageBox::questionYesNo(this, i18n("All token have been removed from this suggestion. Remove suggestion itself or restore original suggestion?"), i18n("Remove suggestion?"), KGuiItem(i18n("Remove suggestion"), KIcon("list-remove")), KGuiItem(i18n("Revert changes"), KIcon("edit-undo"))) == KMessageBox::Yes && d->idSuggestionsModel->remove(index)) {
+            if (KMessageBox::questionYesNo(this, i18n("All token have been removed from this suggestion. Remove suggestion itself or restore original suggestion?"), i18n("Remove suggestion?"), KGuiItem(i18n("Remove suggestion"), QIcon::fromTheme(QStringLiteral("list-remove"))), KGuiItem(i18n("Revert changes"), QIcon::fromTheme(QStringLiteral("edit-undo")))) == KMessageBox::Yes && d->idSuggestionsModel->remove(index)) {
                 emit changed();
             }
         } else if (newSuggestion != suggestion) {
-            if (d->idSuggestionsModel->setData(index, newSuggestion, FormatStringRole))
+            if (d->idSuggestionsModel->setData(index, newSuggestion, IdSuggestionsModel::FormatStringRole))
                 emit changed();
         }
     }
@@ -387,7 +394,9 @@ void SettingsIdSuggestionsWidget::editItem(const QModelIndex &index)
 void SettingsIdSuggestionsWidget::toggleDefault()
 {
     QModelIndex curIndex = d->treeViewSuggestions->currentIndex();
-    bool current = d->idSuggestionsModel->data(curIndex, IsDefaultFormatStringRole).toBool();
-    d->idSuggestionsModel->setData(curIndex, !current, IsDefaultFormatStringRole);
+    bool current = d->idSuggestionsModel->data(curIndex, IdSuggestionsModel::IsDefaultFormatStringRole).toBool();
+    d->idSuggestionsModel->setData(curIndex, !current, IdSuggestionsModel::IsDefaultFormatStringRole);
     emit changed();
 }
+
+#include "settingsidsuggestionswidget.moc"
