@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -97,10 +97,9 @@ public:
         comboBoxField->addItem(i18n("any field"), QVariant());
         comboBoxField->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 
-        const BibTeXFields *bf = BibTeXFields::self();
         /// Use a hash map to get an alphabetically sorted list
         QHash<QString, QString> fielddescs;
-        for (const auto &fd : const_cast<const BibTeXFields &>(*bf))
+        for (const auto &fd : const_cast<const BibTeXFields &>(BibTeXFields::instance()))
             if (fd.upperCamelCaseAlt.isEmpty())
                 fielddescs.insert(fd.label, fd.upperCamelCase);
         /// Sort locale-aware
@@ -128,7 +127,7 @@ public:
         QStringList completionListDate = configGroup.readEntry(QStringLiteral("PreviousSearches"), QStringList());
         for (QStringList::Iterator it = completionListDate.begin(); it != completionListDate.end(); ++it)
             comboBoxFilterText->addItem((*it).mid(12));
-        comboBoxFilterText->lineEdit()->setText(QStringLiteral(""));
+        comboBoxFilterText->lineEdit()->setText(QString());
         comboBoxCombination->setCurrentIndex(configGroup.readEntry("CurrentCombination", 0));
         comboBoxField->setCurrentIndex(configGroup.readEntry("CurrentField", 0));
 
@@ -149,8 +148,10 @@ public:
         result.terms.clear();
         if (comboBoxCombination->currentIndex() == 2) /// exact phrase
             result.terms << comboBoxFilterText->lineEdit()->text();
-        else /// any or every word
-            result.terms = comboBoxFilterText->lineEdit()->text().split(QRegExp(QStringLiteral("\\s+")), QString::SkipEmptyParts);
+        else { /// any or every word
+            static const QRegularExpression sequenceOfSpacesRegExp(QStringLiteral("\\s+"));
+            result.terms = comboBoxFilterText->lineEdit()->text().split(sequenceOfSpacesRegExp, QString::SkipEmptyParts);
+        }
         result.field = comboBoxField->currentIndex() == 0 ? QString() : comboBoxField->itemData(comboBoxField->currentIndex(), Qt::UserRole).toString();
         result.searchPDFfiles = buttonSearchPDFfiles->isChecked();
 
