@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -67,7 +67,7 @@ public:
 
     FileExporterBibTeXPrivate(FileExporterBibTeX *parent)
             : p(parent), keywordCasing(KBibTeX::cLowerCase), quoteComment(Preferences::qcNone), protectCasing(Qt::PartiallyChecked), cancelFlag(false), destinationCodec(nullptr), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), configGroupName(QStringLiteral("FileExporterBibTeX")), configGroupNameGeneral(QStringLiteral("General")) {
-        // nothing
+        /// nothing
     }
 
     void loadState() {
@@ -84,9 +84,9 @@ public:
         stringOpenDelimiter = stringDelimiter[0];
         stringCloseDelimiter = stringDelimiter[1];
 #ifdef HAVE_KF5
-        keywordCasing = (KBibTeX::Casing)configGroup.readEntry(Preferences::keyKeywordCasing, (int)Preferences::defaultKeywordCasing);
-        quoteComment = (Preferences::QuoteComment)configGroup.readEntry(Preferences::keyQuoteComment, (int)Preferences::defaultQuoteComment);
-        protectCasing = (Qt::CheckState)configGroup.readEntry(Preferences::keyProtectCasing, (int)Preferences::defaultProtectCasing);
+        keywordCasing = static_cast<KBibTeX::Casing>(configGroup.readEntry(Preferences::keyKeywordCasing, static_cast<int>(Preferences::defaultKeywordCasing)));
+        quoteComment = static_cast<Preferences::QuoteComment>(configGroup.readEntry(Preferences::keyQuoteComment, static_cast<int>(Preferences::defaultQuoteComment)));
+        protectCasing = static_cast<Qt::CheckState>(configGroup.readEntry(Preferences::keyProtectCasing, static_cast<int>(Preferences::defaultProtectCasing)));
         personNameFormatting = configGroup.readEntry(Preferences::keyPersonNameFormatting, QString());
         listSeparator = configGroup.readEntry(Preferences::keyListSeparator, Preferences::defaultListSeparator);
 
@@ -124,11 +124,11 @@ public:
             stringCloseDelimiter = stringDelimiter[1];
         }
         if (bibtexfile->hasProperty(File::QuoteComment))
-            quoteComment = (Preferences::QuoteComment)bibtexfile->property(File::QuoteComment).toInt();
+            quoteComment = static_cast<Preferences::QuoteComment>(bibtexfile->property(File::QuoteComment).toInt());
         if (bibtexfile->hasProperty(File::KeywordCasing))
-            keywordCasing = (KBibTeX::Casing)bibtexfile->property(File::KeywordCasing).toInt();
+            keywordCasing = static_cast<KBibTeX::Casing>(bibtexfile->property(File::KeywordCasing).toInt());
         if (bibtexfile->hasProperty(File::ProtectCasing))
-            protectCasing = (Qt::CheckState)bibtexfile->property(File::ProtectCasing).toInt();
+            protectCasing = static_cast<Qt::CheckState>(bibtexfile->property(File::ProtectCasing).toInt());
         if (bibtexfile->hasProperty(File::NameFormatting)) {
             /// if the user set "use global default", this property is an empty string
             /// in this case, keep default value
@@ -140,13 +140,11 @@ public:
     }
 
     bool writeEntry(QIODevice *iodevice, const Entry &entry) {
-        const BibTeXEntries *be = BibTeXEntries::self();
-        const BibTeXFields *bf = BibTeXFields::self();
         const EncoderLaTeX &laTeXEncoder = EncoderLaTeX::instance();
 
         /// write start of a entry (entry type and id) in plain ASCII
         iodevice->putChar('@');
-        iodevice->write(be->format(entry.type(), keywordCasing).toLatin1().data());
+        iodevice->write(BibTeXEntries::instance().format(entry.type(), keywordCasing).toLatin1().data());
         iodevice->putChar('{');
         iodevice->write(laTeXEncoder.convertToPlainAscii(entry.id()).toLatin1());
 
@@ -174,7 +172,7 @@ public:
             iodevice->putChar(',');
             iodevice->putChar('\n');
             iodevice->putChar('\t');
-            iodevice->write(laTeXEncoder.convertToPlainAscii(bf->format(key, keywordCasing)).toLatin1());
+            iodevice->write(laTeXEncoder.convertToPlainAscii(BibTeXFields::instance().format(key, keywordCasing)).toLatin1());
             iodevice->putChar(' ');
             iodevice->putChar('=');
             iodevice->putChar(' ');
@@ -189,8 +187,6 @@ public:
     }
 
     bool writeMacro(QIODevice *iodevice, const Macro &macro) {
-        const BibTeXEntries *be = BibTeXEntries::self();
-
         QString text = p->internalValueToBibTeX(macro.value(), QString(), leUTF8);
         if (protectCasing == Qt::Checked)
             addProtectiveCasing(text);
@@ -198,7 +194,7 @@ public:
             removeProtectiveCasing(text);
 
         iodevice->putChar('@');
-        iodevice->write(be->format(QStringLiteral("String"), keywordCasing).toLatin1().data());
+        iodevice->write(BibTeXEntries::instance().format(QStringLiteral("String"), keywordCasing).toLatin1().data());
         iodevice->putChar('{');
         iodevice->write(TextEncoder::encode(macro.key(), destinationCodec));
         iodevice->putChar(' ');
@@ -213,13 +209,11 @@ public:
     }
 
     bool writeComment(QIODevice *iodevice, const Comment &comment) {
-        const BibTeXEntries *be = BibTeXEntries::self();
-
         QString text = comment.text() ;
 
         if (comment.useCommand() || quoteComment == Preferences::qcCommand) {
             iodevice->putChar('@');
-            iodevice->write(be->format(QStringLiteral("Comment"), keywordCasing).toLatin1().data());
+            iodevice->write(BibTeXEntries::instance().format(QStringLiteral("Comment"), keywordCasing).toLatin1().data());
             iodevice->putChar('{');
             iodevice->write(TextEncoder::encode(text, destinationCodec));
             iodevice->putChar('}');
@@ -248,10 +242,8 @@ public:
     }
 
     bool writePreamble(QIODevice *iodevice, const Preamble &preamble) {
-        const BibTeXEntries *be = BibTeXEntries::self();
-
         iodevice->putChar('@');
-        iodevice->write(be->format(QStringLiteral("Preamble"), keywordCasing).toLatin1().data());
+        iodevice->write(BibTeXEntries::instance().format(QStringLiteral("Preamble"), keywordCasing).toLatin1().data());
         iodevice->putChar('{');
         /// Remember: strings from preamble do not get encoded,
         /// may contain raw LaTeX commands and code
@@ -368,7 +360,7 @@ public:
 FileExporterBibTeX::FileExporterBibTeX(QObject *parent)
         : FileExporter(parent), d(new FileExporterBibTeXPrivate(this))
 {
-    // nothing
+    /// nothing
 }
 
 FileExporterBibTeX::~FileExporterBibTeX()
@@ -381,15 +373,17 @@ void FileExporterBibTeX::setEncoding(const QString &encoding)
     d->forcedEncoding = encoding;
 }
 
-bool FileExporterBibTeX::save(QIODevice *iodevice, const File *bibtexfile, QStringList * /*errorLog*/)
+bool FileExporterBibTeX::save(QIODevice *iodevice, const File *bibtexfile, QStringList *errorLog)
 {
+    Q_UNUSED(errorLog)
+
     if (!iodevice->isWritable() && !iodevice->open(QIODevice::WriteOnly)) {
         qCWarning(LOG_KBIBTEX_IO) << "Output device not writable";
         return false;
     }
 
     bool result = true;
-    int totalElements = (int) bibtexfile->count();
+    const int totalElements = bibtexfile->count();
     int currentPos = 0;
 
     d->loadState();
@@ -476,8 +470,10 @@ bool FileExporterBibTeX::save(QIODevice *iodevice, const File *bibtexfile, QStri
     return result && !d->cancelFlag;
 }
 
-bool FileExporterBibTeX::save(QIODevice *iodevice, const QSharedPointer<const Element> element, const File *bibtexfile, QStringList * /*errorLog*/)
+bool FileExporterBibTeX::save(QIODevice *iodevice, const QSharedPointer<const Element> element, const File *bibtexfile, QStringList *errorLog)
 {
+    Q_UNUSED(errorLog)
+
     if (!iodevice->isWritable() && !iodevice->open(QIODevice::WriteOnly)) {
         qCWarning(LOG_KBIBTEX_IO) << "Output device not writable";
         return false;
@@ -583,7 +579,7 @@ QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QStr
                         result.append(d->stringOpenDelimiter);
                     } else if (!prev.dynamicCast<const VerbatimText>().isNull()) {
                         const QString keyToLower(key.toLower());
-                        if (keyToLower.startsWith(Entry::ftUrl) || keyToLower.startsWith(Entry::ftLocalFile) || keyToLower.startsWith(Entry::ftDOI))
+                        if (keyToLower.startsWith(Entry::ftUrl) || keyToLower.startsWith(Entry::ftLocalFile) || keyToLower.startsWith(Entry::ftFile) || keyToLower.startsWith(Entry::ftDOI))
                             /// Filenames and alike have be separated by a semicolon,
                             /// as a plain comma may be part of the filename or URL
                             result.append(QStringLiteral("; "));
