@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "fileexporterpdf.h"
@@ -23,8 +23,6 @@
 #include <QTextStream>
 #include <QDir>
 
-#include <KDebug>
-#include <KLocale>
 #include <KSharedConfig>
 #include <KConfigGroup>
 
@@ -32,34 +30,33 @@
 #include "element.h"
 #include "entry.h"
 #include "fileexporterbibtex.h"
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
+#include "logging_io.h"
 
-FileExporterPDF::FileExporterPDF(FileEmbedding fileEmbedding)
-        : FileExporterToolchain(), m_fileEmbedding(fileEmbedding)
+FileExporterPDF::FileExporterPDF(QObject *parent)
+        : FileExporterToolchain(parent)
 {
-    m_fileBasename = QLatin1String("bibtex-to-pdf");
-    m_fileStem = tempDir.name() + QDir::separator() + m_fileBasename;
+    m_fileBasename = QStringLiteral("bibtex-to-pdf");
+    m_fileStem = tempDir.path() + QDir::separator() + m_fileBasename;
 
-    /// If there is not embedfile.sty file, disable embedding
-    /// irrespective of user's wishes
-    if (!kpsewhich("embedfile.sty")) m_fileEmbedding = NoFileEmbedding;
+    setFileEmbedding(FileExporterPDF::EmbedBibTeXFileAndReferences);
 
     reloadConfig();
 }
 
 FileExporterPDF::~FileExporterPDF()
 {
-    // nothing
+    /// nothing
 }
 
 void FileExporterPDF::reloadConfig()
 {
-    KSharedConfigPtr config = KSharedConfig::openConfig(QLatin1String("kbibtexrc"));
-    KConfigGroup configGroup(config, QLatin1String("FileExporterPDFPS"));
+    KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("kbibtexrc"));
+    KConfigGroup configGroup(config, QStringLiteral("FileExporterPDFPS"));
     m_babelLanguage = configGroup.readEntry(keyBabelLanguage, defaultBabelLanguage);
     m_bibliographyStyle = configGroup.readEntry(keyBibliographyStyle, defaultBibliographyStyle);
 
-    KConfigGroup configGroupGeneral(config, QLatin1String("General"));
+    KConfigGroup configGroupGeneral(config, QStringLiteral("General"));
     m_paperSize = configGroupGeneral.readEntry(keyPaperSize, defaultPaperSize);
     m_font = configGroupGeneral.readEntry(keyFont, defaultFont);
 }
@@ -67,31 +64,30 @@ void FileExporterPDF::reloadConfig()
 bool FileExporterPDF::save(QIODevice *iodevice, const File *bibtexfile, QStringList *errorLog)
 {
     if (!iodevice->isWritable() && !iodevice->open(QIODevice::WriteOnly)) {
-        kDebug() << "Output device not writable";
+        qCWarning(LOG_KBIBTEX_IO) << "Output device not writable";
         return false;
     }
 
     bool result = false;
     m_embeddedFileList.clear();
     if (m_fileEmbedding & EmbedBibTeXFile)
-        m_embeddedFileList.append(QString(QLatin1String("%1|%2|%3")).arg(QLatin1String("BibTeX source"), m_fileStem + KBibTeX::extensionBibTeX, m_fileBasename + KBibTeX::extensionBibTeX));
+        m_embeddedFileList.append(QString(QStringLiteral("%1|%2|%3")).arg(QStringLiteral("BibTeX source"), m_fileStem + KBibTeX::extensionBibTeX, m_fileBasename + KBibTeX::extensionBibTeX));
     if (m_fileEmbedding & EmbedReferences)
         fillEmbeddedFileList(bibtexfile);
 
     QFile output(m_fileStem + KBibTeX::extensionBibTeX);
     if (output.open(QIODevice::WriteOnly)) {
-        FileExporterBibTeX *bibtexExporter = new FileExporterBibTeX();
-        bibtexExporter->setEncoding(QLatin1String("latex"));
-        result = bibtexExporter->save(&output, bibtexfile, errorLog);
+        FileExporterBibTeX bibtexExporter(this);
+        bibtexExporter.setEncoding(QStringLiteral("latex"));
+        result = bibtexExporter.save(&output, bibtexfile, errorLog);
         output.close();
-        delete bibtexExporter;
     }
 
     if (result)
         result = generatePDF(iodevice, errorLog);
 
-    if (errorLog != NULL)
-        kDebug() << "errorLog" << errorLog->join(";");
+    if (errorLog != nullptr)
+        qCDebug(LOG_KBIBTEX_IO) << "errorLog" << errorLog->join(QStringLiteral(";"));
 
     iodevice->close();
     return result;
@@ -100,7 +96,7 @@ bool FileExporterPDF::save(QIODevice *iodevice, const File *bibtexfile, QStringL
 bool FileExporterPDF::save(QIODevice *iodevice, const QSharedPointer<const Element> element, const File *bibtexfile, QStringList *errorLog)
 {
     if (!iodevice->isWritable() && !iodevice->open(QIODevice::WriteOnly)) {
-        kDebug() << "Output device not writable";
+        qCWarning(LOG_KBIBTEX_IO) << "Output device not writable";
         return false;
     }
 
@@ -111,11 +107,10 @@ bool FileExporterPDF::save(QIODevice *iodevice, const QSharedPointer<const Eleme
 
     QFile output(m_fileStem + KBibTeX::extensionBibTeX);
     if (output.open(QIODevice::WriteOnly)) {
-        FileExporterBibTeX *bibtexExporter = new FileExporterBibTeX();
-        bibtexExporter->setEncoding(QLatin1String("latex"));
-        result = bibtexExporter->save(&output, element, bibtexfile, errorLog);
+        FileExporterBibTeX bibtexExporter(this);
+        bibtexExporter.setEncoding(QStringLiteral("latex"));
+        result = bibtexExporter.save(&output, element, bibtexfile, errorLog);
         output.close();
-        delete bibtexExporter;
     }
 
     if (result)
@@ -130,9 +125,18 @@ void FileExporterPDF::setDocumentSearchPaths(const QStringList &searchPaths)
     m_searchPaths = searchPaths;
 }
 
+void FileExporterPDF::setFileEmbedding(FileEmbedding fileEmbedding) {
+    /// If there is not embedfile.sty file, disable embedding
+    /// irrespective of user's wishes
+    if (!kpsewhich(QStringLiteral("embedfile.sty")))
+        m_fileEmbedding = NoFileEmbedding;
+    else
+        m_fileEmbedding = fileEmbedding;
+}
+
 bool FileExporterPDF::generatePDF(QIODevice *iodevice, QStringList *errorLog)
 {
-    QStringList cmdLines = QStringList() << QLatin1String("pdflatex -halt-on-error ") + m_fileStem + KBibTeX::extensionTeX << QLatin1String("bibtex ") + m_fileStem + KBibTeX::extensionAux << QLatin1String("pdflatex -halt-on-error ") + m_fileStem + KBibTeX::extensionTeX << QLatin1String("pdflatex -halt-on-error ") + m_fileStem + KBibTeX::extensionTeX;
+    QStringList cmdLines {QStringLiteral("pdflatex -halt-on-error ") + m_fileStem + KBibTeX::extensionTeX, QStringLiteral("bibtex ") + m_fileStem + KBibTeX::extensionAux, QStringLiteral("pdflatex -halt-on-error ") + m_fileStem + KBibTeX::extensionTeX, QStringLiteral("pdflatex -halt-on-error ") + m_fileStem + KBibTeX::extensionTeX};
 
     return writeLatexFile(m_fileStem + KBibTeX::extensionTeX) && runProcesses(cmdLines, errorLog) && writeFileToIODevice(m_fileStem + KBibTeX::extensionPDF, iodevice, errorLog);
 }
@@ -146,28 +150,28 @@ bool FileExporterPDF::writeLatexFile(const QString &filename)
         ts << "\\documentclass{article}" << endl;
         ts << "\\usepackage[T1]{fontenc}" << endl;
         ts << "\\usepackage[utf8]{inputenc}" << endl;
-        if (kpsewhich("babel.sty"))
+        if (kpsewhich(QStringLiteral("babel.sty")))
             ts << "\\usepackage[" << m_babelLanguage << "]{babel}" << endl;
-        if (kpsewhich("hyperref.sty"))
+        if (kpsewhich(QStringLiteral("hyperref.sty")))
             ts << "\\usepackage[pdfborder={0 0 0},pdfproducer={KBibTeX: https://userbase.kde.org/KBibTeX},pdftex]{hyperref}" << endl;
-        else if (kpsewhich("url.sty"))
+        else if (kpsewhich(QStringLiteral("url.sty")))
             ts << "\\usepackage{url}" << endl;
-        if (m_bibliographyStyle.startsWith(QLatin1String("apacite")) && kpsewhich("apacite.sty"))
+        if (m_bibliographyStyle.startsWith(QStringLiteral("apacite")) && kpsewhich(QStringLiteral("apacite.sty")))
             ts << "\\usepackage[bibnewpage]{apacite}" << endl;
-        if ((m_bibliographyStyle == QLatin1String("agsm") || m_bibliographyStyle == QLatin1String("dcu") || m_bibliographyStyle == QLatin1String("jmr") || m_bibliographyStyle == QLatin1String("jphysicsB") || m_bibliographyStyle == QLatin1String("kluwer") || m_bibliographyStyle == QLatin1String("nederlands") || m_bibliographyStyle == QLatin1String("dcu") || m_bibliographyStyle == QLatin1String("dcu")) && kpsewhich("harvard.sty") && kpsewhich("html.sty"))
+        if ((m_bibliographyStyle == QStringLiteral("agsm") || m_bibliographyStyle == QStringLiteral("dcu") || m_bibliographyStyle == QStringLiteral("jmr") || m_bibliographyStyle == QStringLiteral("jphysicsB") || m_bibliographyStyle == QStringLiteral("kluwer") || m_bibliographyStyle == QStringLiteral("nederlands") || m_bibliographyStyle == QStringLiteral("dcu") || m_bibliographyStyle == QStringLiteral("dcu")) && kpsewhich(QStringLiteral("harvard.sty")) && kpsewhich(QStringLiteral("html.sty")))
             ts << "\\usepackage{html}" << endl << "\\usepackage[dcucite]{harvard}" << endl << "\\renewcommand{\\harvardurl}{URL: \\url}" << endl;
-        if (kpsewhich("embedfile.sty"))
+        if (kpsewhich(QStringLiteral("embedfile.sty")))
             ts << "\\usepackage{embedfile}" << endl;
-        if (kpsewhich("geometry.sty"))
+        if (kpsewhich(QStringLiteral("geometry.sty")))
             ts << "\\usepackage[paper=" << m_paperSize << (m_paperSize.length() <= 2 ? "paper" : "") << "]{geometry}" << endl;
-        if (!m_font.isEmpty() && kpsewhich(m_font + QLatin1String(".sty")))
+        if (!m_font.isEmpty() && kpsewhich(m_font + QStringLiteral(".sty")))
             ts << "\\usepackage{" << m_font << "}" << endl;
         ts << "\\bibliographystyle{" << m_bibliographyStyle << "}" << endl;
         ts << "\\begin{document}" << endl;
 
         if (!m_embeddedFileList.isEmpty())
-            for (QStringList::ConstIterator it = m_embeddedFileList.constBegin(); it != m_embeddedFileList.constEnd(); ++it) {
-                QStringList param = (*it).split(QLatin1String("|"));
+            for (const QString &embeddedFile : const_cast<const QStringList &>(m_embeddedFileList)) {
+                const QStringList param = embeddedFile.split(QStringLiteral("|"));
                 QFile file(param[1]);
                 if (file.exists())
                     ts << "\\embedfile[desc={" << param[0] << "}";
@@ -180,7 +184,7 @@ bool FileExporterPDF::writeLatexFile(const QString &filename)
             }
 
         ts << "\\nocite{*}" << endl;
-        ts << QLatin1String("\\bibliography{") << m_fileBasename << QLatin1String("}") << endl;
+        ts << QStringLiteral("\\bibliography{") << m_fileBasename << QStringLiteral("}") << endl;
         ts << "\\end{document}" << endl;
         latexFile.close();
         return true;
@@ -190,13 +194,13 @@ bool FileExporterPDF::writeLatexFile(const QString &filename)
 
 void FileExporterPDF::fillEmbeddedFileList(const File *bibtexfile)
 {
-    for (File::ConstIterator it = bibtexfile->constBegin(); it != bibtexfile->constEnd(); ++it)
-        fillEmbeddedFileList(*it, bibtexfile);
+    for (const auto &element : const_cast<const File &>(*bibtexfile))
+        fillEmbeddedFileList(element, bibtexfile);
 }
 
 void FileExporterPDF::fillEmbeddedFileList(const QSharedPointer<const Element> element, const File *bibtexfile)
 {
-    if (bibtexfile == NULL || !bibtexfile->hasProperty(File::Url)) {
+    if (bibtexfile == nullptr || !bibtexfile->hasProperty(File::Url)) {
         /// If no valid File was provided or File is not saved, do not append files
         return;
     }
@@ -204,12 +208,12 @@ void FileExporterPDF::fillEmbeddedFileList(const QSharedPointer<const Element> e
     const QSharedPointer<const Entry> entry = element.dynamicCast<const Entry>();
     if (!entry.isNull()) {
         const QString title = PlainTextValue::text(entry->value(Entry::ftTitle));
-        QList<KUrl> urlList = FileInfo::entryUrls(entry.data(), bibtexfile->property(File::Url).toUrl(), FileInfo::TestExistenceYes);
-        foreach (const KUrl &url, urlList) {
+        const auto urlList = FileInfo::entryUrls(entry, bibtexfile->property(File::Url).toUrl(), FileInfo::TestExistenceYes);
+        for (const QUrl &url : urlList) {
             if (!url.isLocalFile()) continue;
-            const QString filename = url.pathOrUrl();
+            const QString filename = url.toLocalFile();
             const QString basename = QFileInfo(filename).fileName();
-            m_embeddedFileList.append(QString(QLatin1String("%1|%2|%3")).arg(title, filename, basename));
+            m_embeddedFileList.append(QString(QStringLiteral("%1|%2|%3")).arg(title, filename, basename));
         }
     }
 }
