@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,13 +23,15 @@
 #include <QDir>
 #include <QTextStream>
 #include <QStandardPaths>
-#include <QtConcurrent/QtConcurrent>
+#include <QRegularExpression>
+#include <QtConcurrentRun>
 
 #include <KSharedConfig>
 #include <KConfigGroup>
 
 #include "kbibtex.h"
 #include "entry.h"
+#include "logging_io.h"
 
 FileInfo::FileInfo()
 {
@@ -44,7 +46,12 @@ const QString FileInfo::mimetypePDF = QStringLiteral("application/pdf");
 
 QMimeType FileInfo::mimeTypeForUrl(const QUrl &url)
 {
-    static QMimeDatabase db;
+    if (!url.isValid() || url.isEmpty()) {
+        qCWarning(LOG_KBIBTEX_IO) << "Cannot determine mime type for empty or invalid QUrl";
+        return QMimeType(); ///< invalid input gives invalid mime type
+    }
+
+    static const QMimeDatabase db;
     static const QMimeType mtHTML(db.mimeTypeForName(mimetypeHTML));
     static const QMimeType mtOctetStream(db.mimeTypeForName(mimetypeOctetStream));
     static const QMimeType mtBibTeX(db.mimeTypeForName(mimetypeBibTeX));
@@ -90,7 +97,7 @@ QMimeType FileInfo::mimeTypeForUrl(const QUrl &url)
     return result;
 }
 
-void FileInfo::urlsInText(const QString &text, TestExistence testExistence, const QString &baseDirectory, QList<QUrl> &result)
+void FileInfo::urlsInText(const QString &text, const TestExistence testExistence, const QString &baseDirectory, QSet<QUrl> &result)
 {
     if (text.isEmpty())
         return;
@@ -100,9 +107,13 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
     /// a DOI in between.
     QString internalText = text;
     int pos = 0;
-    while ((pos = KBibTeX::doiRegExp.indexIn(internalText, pos)) != -1) {
-        QString match = KBibTeX::doiRegExp.cap(0);
-        QUrl url(doiUrlPrefix() + match.remove(QStringLiteral("\\")));
+    QRegularExpressionMatch doiRegExpMatch;
+    while ((doiRegExpMatch = KBibTeX::doiRegExp.match(internalText, pos)).hasMatch()) {
+        pos = doiRegExpMatch.capturedStart(0);
+        QString doiMatch = doiRegExpMatch.captured(0);
+        const int semicolonHttpPos = doiMatch.indexOf(QStringLiteral(";http"));
+        if (semicolonHttpPos > 0) doiMatch = doiMatch.left(semicolonHttpPos);
+        const QUrl url(doiUrlPrefix() + QString(doiMatch).remove(QStringLiteral("\\")));
         if (url.isValid() && !result.contains(url))
             result << url;
         /// remove match from internal text to avoid duplicates
@@ -112,13 +123,13 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
         ///   'Lore ipsum http://doi.example.org/10.1000/38-abc Lore ipsum'
         /// also remove 'http://doi.example.org/' from the text, keeping only
         ///   'Lore ipsum  Lore ipsum'
-        static const QRegExp genericDoiUrlPrefix(QStringLiteral("http[s]?://[a-z0-9./]+/")); ///< looks like an URL
-        const int urlStartPos = genericDoiUrlPrefix.lastIndexIn(internalText, pos);
-        if (urlStartPos >= 0 && genericDoiUrlPrefix.cap(0).length() > pos - urlStartPos)
-            /// genericDoiUrlPrefix.cap(0) may contain (parts of) DOI
-            internalText = internalText.left(urlStartPos) + internalText.mid(pos + match.length());
+        static const QRegularExpression genericDoiUrlPrefix(QStringLiteral("http[s]?://[a-z0-9./-]+/$")); ///< looks like an URL
+        const QRegularExpressionMatch genericDoiUrlPrefixMatch = genericDoiUrlPrefix.match(internalText.left(pos));
+        if (genericDoiUrlPrefixMatch.hasMatch())
+            /// genericDoiUrlPrefixMatch.captured(0) may contain (parts of) DOI
+            internalText = internalText.left(genericDoiUrlPrefixMatch.capturedStart(0)) + internalText.mid(pos + doiMatch.length());
         else
-            internalText = internalText.left(pos) + internalText.mid(pos + match.length());
+            internalText = internalText.left(pos) + internalText.mid(pos + doiMatch.length());
     }
 
     const QStringList fileList = internalText.split(KBibTeX::fileListSeparatorRegExp, QString::SkipEmptyParts);
@@ -139,11 +150,11 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
                     continue;
                 }
             } else if (!baseDirectory.isEmpty() &&
-                    // TODO the following test assumes that absolute paths start
-                    // with a dir separator, which may only be true on Unix/Linux,
-                    // but not Windows. May be a test for 'first character is a letter,
-                    // second is ":", third is "\"' may be necessary.
-                    !internalText.startsWith(QDir::separator())) {
+                       // TODO the following test assumes that absolute paths start
+                       // with a dir separator, which may only be true on Unix/Linux,
+                       // but not Windows. May be a test for 'first character is a letter,
+                       // second is ":", third is "\"' may be necessary.
+                       !internalText.startsWith(QDir::separator())) {
                 /// To get the absolute path, prepend filename fragment with base directory
                 const QString fullFilename = baseDirectory + QDir::separator() + internalText;
                 const QFileInfo fileInfo(fullFilename);
@@ -168,8 +179,10 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
 
         /// extract URL from current field
         pos = 0;
-        while ((pos = KBibTeX::urlRegExp.indexIn(internalText, pos)) != -1) {
-            const QString match = KBibTeX::urlRegExp.cap(0);
+        QRegularExpressionMatch urlRegExpMatch;
+        while ((urlRegExpMatch = KBibTeX::urlRegExp.match(internalText, pos)).hasMatch()) {
+            pos = urlRegExpMatch.capturedStart(0);
+            const QString match = urlRegExpMatch.captured(0);
             QUrl url(match);
             if (url.isValid() && (testExistence == TestExistenceNo || !url.isLocalFile() || QFileInfo::exists(url.toLocalFile())) && !result.contains(url))
                 result << url;
@@ -179,11 +192,13 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
 
         /// explicitly check URL entry, may be an URL even if http:// or alike is missing
         pos = 0;
-        while ((pos = KBibTeX::domainNameRegExp.indexIn(internalText, pos)) > -1) {
+        QRegularExpressionMatch domainNameRegExpMatch;
+        while ((domainNameRegExpMatch = KBibTeX::domainNameRegExp.match(internalText, pos)).hasMatch()) {
+            pos = domainNameRegExpMatch.capturedStart(0);
             int pos2 = internalText.indexOf(QStringLiteral(" "), pos + 1);
             if (pos2 < 0) pos2 = internalText.length();
             QString match = internalText.mid(pos, pos2 - pos);
-            QUrl url("http://" + match);
+            const QUrl url(QStringLiteral("http://") + match); // FIXME what about HTTPS?
             if (url.isValid() && !result.contains(url))
                 result << url;
             /// remove match from internal text to avoid duplicates
@@ -192,8 +207,10 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
 
         /// extract general file-like patterns
         pos = 0;
-        while ((pos = KBibTeX::fileRegExp.indexIn(internalText, pos)) != -1) {
-            QString match = KBibTeX::fileRegExp.cap(0);
+        QRegularExpressionMatch fileRegExpMatch;
+        while ((fileRegExpMatch = KBibTeX::fileRegExp.match(internalText, pos)).hasMatch()) {
+            pos = fileRegExpMatch.capturedStart(0);
+            const QString match = fileRegExpMatch.captured(0);
             QUrl url(match);
             if (url.isValid() && (testExistence == TestExistenceNo || !url.isLocalFile() || QFileInfo::exists(url.toLocalFile())) && !result.contains(url))
                 result << url;
@@ -203,18 +220,19 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
     }
 }
 
-QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const QUrl &bibTeXUrl, TestExistence testExistence)
+QSet<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const QUrl &bibTeXUrl, TestExistence testExistence)
 {
-    QList<QUrl> result;
+    QSet<QUrl> result;
     if (entry.isNull() || entry->isEmpty())
         return result;
 
     if (entry->contains(Entry::ftDOI)) {
         const QString doi = PlainTextValue::text(entry->value(Entry::ftDOI));
-        if (!doi.isEmpty() && KBibTeX::doiRegExp.indexIn(doi) == 0) {
-            QString match = KBibTeX::doiRegExp.cap(0);
+        QRegularExpressionMatch doiRegExpMatch;
+        if (!doi.isEmpty() && (doiRegExpMatch = KBibTeX::doiRegExp.match(doi)).hasMatch()) {
+            QString match = doiRegExpMatch.captured(0);
             QUrl url(doiUrlPrefix() + match.remove(QStringLiteral("\\")));
-            result.append(url);
+            result.insert(url);
         }
     }
     static const QString etPMID = QStringLiteral("pmid");
@@ -224,7 +242,7 @@ QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const
         ok &= pmid.toInt(&ok) > 0;
         if (ok) {
             QUrl url(QStringLiteral("https://www.ncbi.nlm.nih.gov/pubmed/") + pmid);
-            result.append(url);
+            result.insert(url);
         }
     }
     static const QString etEPrint = QStringLiteral("eprint");
@@ -232,13 +250,12 @@ QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const
         const QString eprint = PlainTextValue::text(entry->value(etEPrint));
         if (!eprint.isEmpty()) {
             QUrl url(QStringLiteral("http://arxiv.org/search?query=") + eprint);
-            result.append(url);
+            result.insert(url);
         }
     }
 
     const QString baseDirectory = bibTeXUrl.isValid() ? bibTeXUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path() : QString();
 
-    static const QRegExp regExpEscapedChars = QRegExp(QStringLiteral("\\\\+([&_~])"));
     for (Entry::ConstIterator it = entry->constBegin(); it != entry->constEnd(); ++it) {
         /// skip abstracts, they contain sometimes strange text fragments
         /// that are mistaken for URLs
@@ -248,9 +265,8 @@ QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const
         for (const auto &valueItem : v) {
             QString plainText = PlainTextValue::text(*valueItem);
 
-            int pos = -1;
-            while ((pos = regExpEscapedChars.indexIn(plainText, pos + 1)) != -1)
-                plainText = plainText.replace(regExpEscapedChars.cap(0), regExpEscapedChars.cap(1));
+            static const QRegularExpression regExpEscapedChars = QRegularExpression(QStringLiteral("\\\\+([&_~])"));
+            plainText.replace(regExpEscapedChars, QStringLiteral("\\1"));
 
             urlsInText(plainText, testExistence, baseDirectory, result);
         }
@@ -258,7 +274,7 @@ QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const
 
     if (!baseDirectory.isEmpty()) {
         /// File types supported by "document preview"
-        static const QStringList documentFileExtensions = QStringList() << QStringLiteral(".pdf") << QStringLiteral(".pdf.gz") << QStringLiteral(".pdf.bz2") << QStringLiteral(".ps") << QStringLiteral(".ps.gz") << QStringLiteral(".ps.bz2") << QStringLiteral(".eps") << QStringLiteral(".eps.gz") << QStringLiteral(".eps.bz2") << QStringLiteral(".html") << QStringLiteral(".xhtml") << QStringLiteral(".htm") << QStringLiteral(".dvi") << QStringLiteral(".djvu") << QStringLiteral(".wwf") << QStringLiteral(".jpeg") << QStringLiteral(".jpg") << QStringLiteral(".png") << QStringLiteral(".gif") << QStringLiteral(".tif") << QStringLiteral(".tiff");
+        static const QStringList documentFileExtensions {QStringLiteral(".pdf"), QStringLiteral(".pdf.gz"), QStringLiteral(".pdf.bz2"), QStringLiteral(".ps"), QStringLiteral(".ps.gz"), QStringLiteral(".ps.bz2"), QStringLiteral(".eps"), QStringLiteral(".eps.gz"), QStringLiteral(".eps.bz2"), QStringLiteral(".html"), QStringLiteral(".xhtml"), QStringLiteral(".htm"), QStringLiteral(".dvi"), QStringLiteral(".djvu"), QStringLiteral(".wwf"), QStringLiteral(".jpeg"), QStringLiteral(".jpg"), QStringLiteral(".png"), QStringLiteral(".gif"), QStringLiteral(".tif"), QStringLiteral(".tiff")};
         result.reserve(result.size() + documentFileExtensions.size() * 2);
 
         /// check if in the same directory as the BibTeX file
@@ -275,7 +291,8 @@ QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const
         /// check if in the same directory as the BibTeX file there is a subdirectory
         /// similar to the BibTeX file's name and which contains a PDF file exists
         /// which filename is based on the entry's id
-        QString basename = bibTeXUrl.fileName().remove(QRegExp("\\.[^.]{2,5}$"));
+        static const QRegularExpression filenameExtension(QStringLiteral("\\.[^.]{2,5}$"));
+        const QString basename = bibTeXUrl.fileName().remove(filenameExtension);
         QString directory = baseDirectory + QDir::separator() + basename;
         for (const QString &extension : documentFileExtensions) {
             const QFileInfo fi(directory + QDir::separator() + entry->id() + extension);
@@ -293,11 +310,11 @@ QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const
 QString FileInfo::pdfToText(const QString &pdfFilename)
 {
     /// Build filename for text file where PDF file's plain text is cached
-    static const QRegExp invalidChars(QStringLiteral("[^-a-z0-9_]"), Qt::CaseInsensitive);
     const QString cacheDirectory = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/pdftotext");
     if (!QDir(cacheDirectory).exists() && !QDir::home().mkdir(cacheDirectory))
         /// Could not create cache directory
         return QString();
+    static const QRegularExpression invalidChars(QStringLiteral("[^-a-z0-9_]"), QRegularExpression::CaseInsensitiveOption);
     const QString textFilename = QString(pdfFilename).remove(invalidChars).append(QStringLiteral(".txt")).prepend(QStringLiteral("/")).prepend(cacheDirectory);
 
     /// First, check if there is a cache text file
@@ -305,8 +322,7 @@ QString FileInfo::pdfToText(const QString &pdfFilename)
         /// Load text from cache file
         QFile f(textFilename);
         if (f.open(QFile::ReadOnly)) {
-            QTextStream ts(&f);
-            const QString text = ts.readAll();
+            const QString text = QString::fromUtf8(f.readAll());
             f.close();
             return text;
         }
@@ -341,14 +357,16 @@ void FileInfo::extractPDFTextToCache(const QString &pdfFilename, const QString &
     QFile f(cacheFilename);
     if (f.open(QFile::WriteOnly)) {
         static const int maxCharacters = 1 << 18;
-        QTextStream ts(&f);
-        ts << text.left(maxCharacters); ///< keep only the first 2^18 many characters
+        f.write(text.left(maxCharacters).toUtf8()); ///< keep only the first 2^18 many characters
 
         if (text.length() > maxCharacters)
             msgList << QString(QStringLiteral("### Text too long, skipping %1 characters ###")).arg(text.length() - maxCharacters);
         /// Write all messages (warnings) to end of text file
-        for (const QString &msg : const_cast<const QStringList &>(msgList))
-            ts << endl << msg;
+        for (const QString &msg : const_cast<const QStringList &>(msgList)) {
+            static const char linebreak = '\n';
+            f.write(&linebreak, 1);
+            f.write(msg.toUtf8());
+        }
 
         f.close();
     }
