@@ -20,7 +20,7 @@
 
 #include "kbibtexio_export.h"
 
-#include <QList>
+#include <QSet>
 #include <QUrl>
 #include <QMimeDatabase>
 #include <QMimeType>
@@ -69,7 +69,7 @@ public:
      * @param baseDirectory base directory for tests on relative path names
      * @param addTo add found URLs/filenames to this list
      */
-    static void urlsInText(const QString &text, TestExistence testExistence, const QString &baseDirectory, QList<QUrl> &addTo);
+    static void urlsInText(const QString &text, const TestExistence testExistence, const QString &baseDirectory, QSet<QUrl> &addTo);
 
     /**
      * Find all file or URL references in the given entry. Found filenames or
@@ -83,7 +83,7 @@ public:
      * @param testExistence shall be tested for file existence?
      * @return list of found URLs/filenames (duplicates are avoided)
      */
-    static QList<QUrl> entryUrls(const QSharedPointer<const Entry> &entry, const QUrl &bibTeXUrl, TestExistence testExistence);
+    static QSet<QUrl> entryUrls(const QSharedPointer<const Entry> &entry, const QUrl &bibTeXUrl, TestExistence testExistence);
 
     /**
      * Load the given PDF file and return the contained plain text.
