@@ -18,7 +18,9 @@
 #ifndef KBIBTEX_NETWORKING_INTERNALNETWORKACCESSMANAGER_H
 #define KBIBTEX_NETWORKING_INTERNALNETWORKACCESSMANAGER_H
 
+#ifdef HAVE_KF5
 #include "kbibtexnetworking_export.h"
+#endif // HAVE_KF5
 
 #include <QNetworkAccessManager>
 #include <QUrl>
@@ -57,6 +59,14 @@ public:
      */
     static QString reverseObfuscate(const QByteArray &obfuscated);
 
+    /**
+     * Remove API keys from an URL.
+     *
+     * @param url URL where API keys have to be removed
+     * @return a reference to the URL passed to this function
+     */
+    static QUrl removeApiKey(QUrl url);
+
 protected:
     InternalNetworkAccessManager(QObject *parent = nullptr);
     class HTTPEquivCookieJar;
@@ -72,7 +82,7 @@ private:
 private slots:
     void networkReplyTimeout();
     void networkReplyFinished();
-
+    void logSslErrors(const QList<QSslError> &errors);
 };
 
 #endif // KBIBTEX_NETWORKING_INTERNALNETWORKACCESSMANAGER_H
