@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,12 +23,13 @@
 #endif // HAVE_QTWIDGETS
 #include <QNetworkRequest>
 #include <QNetworkReply>
+#include <QRegularExpression>
 
-#ifdef HAVE_QTWIDGETS
+#ifdef HAVE_KF5
 #include <KLineEdit>
 #include <KConfigGroup>
-#endif // HAVE_QTWIDGETS
 #include <KLocalizedString>
+#endif // HAVE_KF5
 
 #include "kbibtex.h"
 #include "internalnetworkaccessmanager.h"
@@ -88,20 +89,17 @@ public:
 
 class OnlineSearchDOI::OnlineSearchDOIPrivate
 {
-private:
-    // UNUSED OnlineSearchDOI *p;
-
 public:
 #ifdef HAVE_QTWIDGETS
     OnlineSearchQueryFormDOI *form;
 #endif // HAVE_QTWIDGETS
 
-    OnlineSearchDOIPrivate(OnlineSearchDOI */* UNUSED parent*/)
+    OnlineSearchDOIPrivate(OnlineSearchDOI *parent)
 #ifdef HAVE_QTWIDGETS
             : form(nullptr)
 #endif // HAVE_QTWIDGETS
     {
-        // nothing
+        Q_UNUSED(parent)
     }
 
 #ifdef HAVE_QTWIDGETS
@@ -118,8 +116,9 @@ public:
     QUrl buildQueryUrl(const QMap<QString, QString> &query, int numResults) {
         Q_UNUSED(numResults)
 
-        if (KBibTeX::doiRegExp.indexIn(query[queryKeyFreeText]) >= 0) {
-            return QUrl(QStringLiteral("https://dx.doi.org/") + KBibTeX::doiRegExp.cap(0));
+        const QRegularExpressionMatch doiRegExpMatch = KBibTeX::doiRegExp.match(query[queryKeyFreeText]);
+        if (doiRegExpMatch.hasMatch()) {
+            return QUrl(QStringLiteral("https://dx.doi.org/") + doiRegExpMatch.captured(0));
         }
 
         return QUrl();
@@ -155,6 +154,8 @@ void OnlineSearchDOI::startSearchFromForm()
         d->form->saveState();
     } else
         delayedStoppedSearch(resultNoError);
+
+    refreshBusyProperty();
 }
 #endif // HAVE_QTWIDGETS
 
@@ -170,6 +171,8 @@ void OnlineSearchDOI::startSearch(const QMap<QString, QString> &query, int numRe
         QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
         InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
         connect(reply, &QNetworkReply::finished, this, &OnlineSearchDOI::downloadDone);
+
+        refreshBusyProperty();
     } else
         delayedStoppedSearch(resultNoError);
 }
@@ -232,7 +235,7 @@ void OnlineSearchDOI::downloadDone()
 
                     delete bibtexFile;
                 } else {
-                    qCWarning(LOG_KBIBTEX_NETWORKING) << "No valid BibTeX file results returned on request on" << reply->url().toDisplayString();
+                    qCWarning(LOG_KBIBTEX_NETWORKING) << "No valid BibTeX file results returned on request on" << InternalNetworkAccessManager::removeApiKey(reply->url()).toDisplayString();
                     stopSearch(resultUnspecifiedError);
                 }
             } else {
@@ -240,8 +243,9 @@ void OnlineSearchDOI::downloadDone()
                 stopSearch(resultNoError);
             }
         }
-    } else
-        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
+    }
+
+    refreshBusyProperty();
 }
 
 #include "onlinesearchdoi.moc"
