@@ -21,9 +21,12 @@
 #include <QIcon>
 #include <QUrlQuery>
 #include <QRegularExpression>
+#include <QTimer>
 
 #ifdef HAVE_KF5
 #include <KLocalizedString>
+#else // HAVE_KF5
+#define i18n(text) QObject::tr(text)
 #endif // HAVE_KF5
 
 #include "fileimporterbibtex.h"
@@ -32,12 +35,9 @@
 
 class OnlineSearchGoogleScholar::OnlineSearchGoogleScholarPrivate
 {
-private:
-    // UNUSED OnlineSearchGoogleScholar *p;
-
 public:
     int numResults;
-    QMap<QString, QString> listBibTeXurls;
+    QMap<QString, QPair<QString, QString>> listBibTeXurls;
     QString queryFreetext, queryAuthor, queryYear;
     QString startPageUrl;
     QString advancedSearchPageUrl;
@@ -45,7 +45,7 @@ public:
     FileImporterBibTeX *importer;
 
     OnlineSearchGoogleScholarPrivate(OnlineSearchGoogleScholar *parent)
-        : /* UNUSED p(parent), */ numResults(0)
+            : numResults(0)
     {
         importer = new FileImporterBibTeX(parent);
         startPageUrl = QStringLiteral("http://scholar.google.com/");
@@ -58,7 +58,7 @@ public:
 
     QString documentUrlForBibTeXEntry(const QString &htmlText, int bibLinkPos) {
         /// Regular expression to detect text of a link to a document
-        static const QRegExp documentLinkIndicator(QStringLiteral("\\[(PDF|HTML)\\]"), Qt::CaseSensitive);
+        static const QRegularExpression documentLinkIndicator(QStringLiteral("\\[(PDF|HTML)\\]"), QRegularExpression::CaseInsensitiveOption);
 
         /// Text for link is *before* the BibTeX link in Google's HTML code
         int posDocumentLinkText = htmlText.lastIndexOf(documentLinkIndicator, bibLinkPos);
@@ -110,7 +110,7 @@ public:
 OnlineSearchGoogleScholar::OnlineSearchGoogleScholar(QObject *parent)
         : OnlineSearchAbstract(parent), d(new OnlineSearchGoogleScholar::OnlineSearchGoogleScholarPrivate(this))
 {
-    // nothing
+    /// nothing
 }
 
 OnlineSearchGoogleScholar::~OnlineSearchGoogleScholar()
@@ -149,6 +149,8 @@ void OnlineSearchGoogleScholar::startSearch(const QMap<QString, QString> &query,
     QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
     InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
     connect(reply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingStartPage);
+
+    refreshBusyProperty();
 }
 
 void OnlineSearchGoogleScholar::doneFetchingStartPage()
@@ -170,6 +172,7 @@ void OnlineSearchGoogleScholar::doneFetchingStartPage()
             /// landed on country-specific domain
             static const QRegularExpression pathToSettingsPage(QStringLiteral(" href=\"(/scholar_settings[^ \"]*)"));
             const QString htmlCode = QString::fromUtf8(reply->readAll());
+            // dumpToFile(QStringLiteral("01-doneFetchingStartPage.html"),htmlCode);
             const QRegularExpressionMatch pathToSettingsPageMatch = pathToSettingsPage.match(htmlCode);
             if (!pathToSettingsPageMatch.hasMatch() || pathToSettingsPageMatch.captured(1).isEmpty()) {
                 qCWarning(LOG_KBIBTEX_NETWORKING) << "No link to Google Scholar settings found";
@@ -185,13 +188,17 @@ void OnlineSearchGoogleScholar::doneFetchingStartPage()
             query.addQueryItem(QStringLiteral("as_sdt"), QStringLiteral("0,5"));
             url.setQuery(query);
 
-            QNetworkRequest request(url);
-            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply->url());
-            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
-            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingConfigPage);
+            const QUrl replyUrl = reply->url();
+            QTimer::singleShot(250, [this, url, replyUrl]() {
+                QNetworkRequest request(url);
+                QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, replyUrl);
+                InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+                connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingConfigPage);
+            });
         }
-    } else
-        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
+    }
+
+    refreshBusyProperty();
 }
 
 void OnlineSearchGoogleScholar::doneFetchingConfigPage()
@@ -205,12 +212,14 @@ void OnlineSearchGoogleScholar::doneFetchingConfigPage()
         if (redirUrl.isValid()) {
             /// Redirection to another url
             ++numSteps;
+
             QNetworkRequest request(redirUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
             InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
             connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingConfigPage);
         } else {
             const QString htmlText = QString::fromUtf8(reply->readAll().constData());
+            // dumpToFile(QStringLiteral("02-doneFetchingConfigPage.html"), htmlText);
             static const QRegularExpression formOpeningTag(QStringLiteral("<form [^>]+action=\"([^\"]*scholar_setprefs[^\"]*)"));
             const QRegularExpressionMatch formOpeningTagMatch = formOpeningTag.match(htmlText);
             const int formOpeningTagPos = formOpeningTagMatch.capturedStart(0);
@@ -225,7 +234,7 @@ void OnlineSearchGoogleScholar::doneFetchingConfigPage()
             inputMap[QStringLiteral("scis")] = QStringLiteral("yes");
             inputMap[QStringLiteral("scisf")] = QStringLiteral("4");
             inputMap[QStringLiteral("num")] = QString::number(d->numResults);
-            inputMap[QStringLiteral("submit")] = QStringLiteral("");
+            inputMap[QStringLiteral("submit")] = QString();
 
             QUrl url = reply->url().resolved(QUrl(decodeURL(formOpeningTagMatch.captured(1))));
             QUrlQuery query(url);
@@ -235,13 +244,16 @@ void OnlineSearchGoogleScholar::doneFetchingConfigPage()
             }
             url.setQuery(query);
 
-            QNetworkRequest request(url);
-            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
-            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
-            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingSetConfigPage);
+            QTimer::singleShot(250, [this, url, reply]() {
+                QNetworkRequest request(url);
+                QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
+                InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+                connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingSetConfigPage);
+            });
         }
-    } else
-        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
+    }
+
+    refreshBusyProperty();
 }
 
 void OnlineSearchGoogleScholar::doneFetchingSetConfigPage()
@@ -261,6 +273,9 @@ void OnlineSearchGoogleScholar::doneFetchingSetConfigPage()
             InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
             connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingSetConfigPage);
         } else {
+            // const QString htmlText = QString::fromUtf8(reply->readAll().constData());
+            // dumpToFile(QStringLiteral("03-doneFetchingSetConfigPage.html"), htmlText);
+
             QUrl url(QString(d->queryPageUrl).arg(reply->url().host()));
             QUrlQuery query(url);
             query.addQueryItem(QStringLiteral("as_q"), d->queryFreetext);
@@ -272,13 +287,16 @@ void OnlineSearchGoogleScholar::doneFetchingSetConfigPage()
             query.addQueryItem(QStringLiteral("btnG"), QStringLiteral("Search Scholar"));
             url.setQuery(query);
 
-            QNetworkRequest request(url);
-            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
-            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
-            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingQueryPage);
+            QTimer::singleShot(250, [this, url, reply]() {
+                QNetworkRequest request(url);
+                QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
+                InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+                connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingQueryPage);
+            });
         }
-    } else
-        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
+    }
+
+    refreshBusyProperty();
 }
 
 void OnlineSearchGoogleScholar::doneFetchingQueryPage()
@@ -296,57 +314,62 @@ void OnlineSearchGoogleScholar::doneFetchingQueryPage()
             QNetworkRequest request(redirUrl);
             QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
             InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
-            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingBibTeX);
             connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingQueryPage);
         } else {
             const QString htmlText = QString::fromUtf8(reply->readAll().constData());
+            // dumpToFile(QStringLiteral("04-doneFetchingQueryPage.html"), htmlText);
 
             d->listBibTeXurls.clear();
 
 #ifdef HAVE_KF5
             if (htmlText.contains(QStringLiteral("enable JavaScript")) || htmlText.contains(QStringLiteral("re not a robot"))) {
+                qCInfo(LOG_KBIBTEX_NETWORKING) << "'Google Scholar' denied scrapping data because it thinks KBibTeX is a robot.";
                 sendVisualNotification(i18n("'Google Scholar' denied scrapping data because it thinks you are a robot."), label(), QStringLiteral("kbibtex"), 7 * 1000);
             } else {
 #endif // HAVE_KF5
-                static const QRegExp linkToBib("/scholar.bib\\?[^\" >]+");
-                int pos = 0;
-                while ((pos = linkToBib.indexIn(htmlText, pos)) != -1) {
+                static const QRegularExpression linkToBib("/scholar.bib\\?[^\" >]+");
+                QRegularExpressionMatchIterator linkToBibMatchIterator = linkToBib.globalMatch(htmlText);
+                while (linkToBibMatchIterator.hasNext()) {
+                    const QRegularExpressionMatch linkToBibMatch = linkToBibMatchIterator.next();
+                    const int pos = linkToBibMatch.capturedStart();
                     /// Try to figure out [PDF] or [HTML] link associated with BibTeX entry
                     const QString documentUrl = d->documentUrlForBibTeXEntry(htmlText, pos);
                     /// Extract primary link associated with BibTeX entry
                     const QString primaryUrl = d->mainUrlForBibTeXEntry(htmlText, pos);
 
-                    const QString bibtexUrl("https://" + reply->url().host() + linkToBib.cap(0).replace(QStringLiteral("&amp;"), QStringLiteral("&")));
-                    d->listBibTeXurls.insert(bibtexUrl, primaryUrl + QLatin1Char('|') + documentUrl);
-                    pos += linkToBib.matchedLength();
+                    const QString bibtexUrl(QStringLiteral("https://") + reply->url().host() + linkToBibMatch.captured().replace(QStringLiteral("&amp;"), QStringLiteral("&")));
+                    d->listBibTeXurls.insert(bibtexUrl, qMakePair(primaryUrl, documentUrl));
                 }
 #ifdef HAVE_KF5
             }
 #endif // HAVE_KF5
 
             if (!d->listBibTeXurls.isEmpty()) {
-                const QString bibtexUrl = d->listBibTeXurls.constBegin().key();
-                const QStringList urls = d->listBibTeXurls.constBegin().value().split(QStringLiteral("|"), QString::KeepEmptyParts);
-                const QString primaryUrl = urls.first();
-                const QString documentUrl = urls.last();
-                QNetworkRequest request(bibtexUrl);
-                QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
-                if (!primaryUrl.isEmpty()) {
-                    /// Store primary URL as a property of the request/reply
-                    newReply->setProperty("primaryurl", QVariant::fromValue<QString>(primaryUrl));
-                }
-                if (!documentUrl.isEmpty()) {
-                    /// Store URL to document as a property of the request/reply
-                    newReply->setProperty("documenturl", QVariant::fromValue<QString>(documentUrl));
-                }
-                InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
-                connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingBibTeX);
-                d->listBibTeXurls.erase(d->listBibTeXurls.begin());
+                const auto listBibTeXurlsFront = d->listBibTeXurls.begin();
+                const QString bibtexUrl = listBibTeXurlsFront.key();
+                const QString primaryUrl = listBibTeXurlsFront.value().first;
+                const QString documentUrl = listBibTeXurlsFront.value().second;
+                QTimer::singleShot(250, [this, bibtexUrl, primaryUrl, documentUrl, reply]() {
+                    QNetworkRequest request(bibtexUrl);
+                    QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
+                    if (!primaryUrl.isEmpty()) {
+                        /// Store primary URL as a property of the request/reply
+                        newReply->setProperty("primaryurl", QVariant::fromValue<QString>(primaryUrl));
+                    }
+                    if (!documentUrl.isEmpty()) {
+                        /// Store URL to document as a property of the request/reply
+                        newReply->setProperty("documenturl", QVariant::fromValue<QString>(documentUrl));
+                    }
+                    InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+                    connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingBibTeX);
+                });
+                d->listBibTeXurls.erase(listBibTeXurlsFront);
             } else
                 stopSearch(resultNoError);
         }
-    } else
-        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
+    }
+
+    refreshBusyProperty();
 }
 
 void OnlineSearchGoogleScholar::doneFetchingBibTeX()
@@ -370,6 +393,7 @@ void OnlineSearchGoogleScholar::doneFetchingBibTeX()
         } else {
             /// ensure proper treatment of UTF-8 characters
             const QString rawText = QString::fromUtf8(reply->readAll());
+            // dumpToFile(QStringLiteral("05-doneFetchingBibTeX.bib"),rawText);
             File *bibtexFile = d->importer->fromString(rawText);
 
             bool hasEntry = false;
@@ -403,14 +427,11 @@ void OnlineSearchGoogleScholar::doneFetchingBibTeX()
             if (!hasEntry) {
                 qCWarning(LOG_KBIBTEX_NETWORKING) << "Searching" << label() << "resulted in invalid BibTeX data:" << rawText;
                 stopSearch(resultUnspecifiedError);
-                return;
-            }
-
-            if (!d->listBibTeXurls.isEmpty()) {
-                const QString bibtexUrl = d->listBibTeXurls.constBegin().key();
-                const QStringList urls = d->listBibTeXurls.constBegin().value().split(QStringLiteral("|"), QString::KeepEmptyParts);
-                const QString primaryUrl = urls.first();
-                const QString documentUrl = urls.last();
+            } else if (!d->listBibTeXurls.isEmpty()) {
+                const auto listBibTeXurlsFront = d->listBibTeXurls.begin();
+                const QString bibtexUrl = listBibTeXurlsFront.key();
+                const QString primaryUrl = listBibTeXurlsFront.value().first;
+                const QString documentUrl = listBibTeXurlsFront.value().second;
                 QNetworkRequest request(bibtexUrl);
                 QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
                 InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
@@ -423,14 +444,13 @@ void OnlineSearchGoogleScholar::doneFetchingBibTeX()
                     newReply->setProperty("documenturl", QVariant::fromValue<QString>(documentUrl));
                 }
                 connect(newReply, &QNetworkReply::finished, this, &OnlineSearchGoogleScholar::doneFetchingBibTeX);
-                d->listBibTeXurls.erase(d->listBibTeXurls.begin());
-            } else {
+                d->listBibTeXurls.erase(listBibTeXurlsFront);
+            } else
                 stopSearch(resultNoError);
-                emit progress(curStep = numSteps, numSteps);
-            }
         }
-    } else
-        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
+    }
+
+    refreshBusyProperty();
 }
 
 QString OnlineSearchGoogleScholar::label() const
