@@ -1,6 +1,7 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                 2014 Pavel Zorin-Kranich <pzorin@math.uni-bonn.de>      *
+ *                 2018 Alexander Dunlap <alexander.dunlap@gmail.com>      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -13,7 +14,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "onlinesearchmrlookup.h"
@@ -22,15 +23,16 @@
 #include <QNetworkRequest>
 #include <QNetworkAccessManager>
 #include <QMap>
+#include <QUrlQuery>
 
-#include <KLocale>
-#include <KDebug>
+#include <KLocalizedString>
 
 #include "fileimporterbibtex.h"
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
 #include "internalnetworkaccessmanager.h"
+#include "logging_networking.h"
 
-const QString OnlineSearchMRLookup::queryUrlStem = QLatin1String("http://www.ams.org/mrlookup");
+const QString OnlineSearchMRLookup::queryUrlStem = QStringLiteral("https://mathscinet.ams.org/mrlookup");
 
 OnlineSearchMRLookup::OnlineSearchMRLookup(QWidget *parent)
         : OnlineSearchAbstract(parent)
@@ -41,33 +43,30 @@ OnlineSearchMRLookup::OnlineSearchMRLookup(QWidget *parent)
 void OnlineSearchMRLookup::startSearch(const QMap<QString, QString> &query, int)
 {
     m_hasBeenCanceled = false;
+    emit progress(curStep = 0, numSteps = 1);
 
-    KUrl url(queryUrlStem);
+    QUrl url(queryUrlStem);
+    QUrlQuery q(url);
 
     const QString title = query[queryKeyTitle];
-    url.addQueryItem(QLatin1String("ti"), title);
+    q.addQueryItem(QStringLiteral("ti"), title);
 
     const QString authors = query[queryKeyAuthor];
-    url.addQueryItem(QLatin1String("au"), authors);
+    q.addQueryItem(QStringLiteral("au"), authors);
 
     const QString year = query[queryKeyYear];
     if (!year.isEmpty())
-        url.addQueryItem(QLatin1String("year"), year);
+        q.addQueryItem(QStringLiteral("year"), year);
 
-    url.addQueryItem(QLatin1String("format"), QLatin1String("bibtex"));
-
-    emit progress(0, 1);
+    q.addQueryItem(QStringLiteral("format"), QStringLiteral("bibtex"));
 
+    url.setQuery(q);
     QNetworkRequest request(url);
-    QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-    connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingResultPage()));
-}
+    QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
+    InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+    connect(reply, &QNetworkReply::finished, this, &OnlineSearchMRLookup::doneFetchingResultPage);
 
-void OnlineSearchMRLookup::startSearch()
-{
-    m_hasBeenCanceled = false;
-    delayedStoppedSearch(resultNoError);
+    refreshBusyProperty();
 }
 
 QString OnlineSearchMRLookup::label() const
@@ -77,29 +76,19 @@ QString OnlineSearchMRLookup::label() const
 
 QString OnlineSearchMRLookup::favIconUrl() const
 {
-    return QLatin1String("http://www.ams.org/favicon.ico");
+    return QStringLiteral("http://www.ams.org/favicon.ico");
 }
 
-OnlineSearchQueryFormAbstract *OnlineSearchMRLookup::customWidget(QWidget *)
+QUrl OnlineSearchMRLookup::homepage() const
 {
-    return NULL;
-}
-
-KUrl OnlineSearchMRLookup::homepage() const
-{
-    return KUrl("http://www.ams.org/mrlookup");
-}
-
-void OnlineSearchMRLookup::cancel()
-{
-    OnlineSearchAbstract::cancel();
+    return QUrl(QStringLiteral("https://mathscinet.ams.org/mrlookup"));
 }
 
 void OnlineSearchMRLookup::doneFetchingResultPage()
 {
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
-    emit progress(1, 1);
+    emit progress(curStep = numSteps, numSteps);
 
     if (handleErrors(reply)) {
         /// ensure proper treatment of UTF-8 characters
@@ -107,33 +96,36 @@ void OnlineSearchMRLookup::doneFetchingResultPage()
 
         QString bibtexCode;
         int p1 = -1, p2 = -1;
-        while ((p1 = htmlCode.indexOf(QLatin1String("<pre>"), p2 + 1)) >= 0 && (p2 = htmlCode.indexOf(QLatin1String("</pre>"), p1 + 1)) >= 0) {
+        while ((p1 = htmlCode.indexOf(QStringLiteral("<pre>"), p2 + 1)) >= 0 && (p2 = htmlCode.indexOf(QStringLiteral("</pre>"), p1 + 1)) >= 0) {
             bibtexCode += htmlCode.midRef(p1 + 5, p2 - p1 - 5);
-            bibtexCode + QLatin1Char('\n');
+            bibtexCode += QLatin1Char('\n');
         }
 
-        FileImporterBibTeX importer;
+        FileImporterBibTeX importer(this);
         File *bibtexFile = importer.fromString(bibtexCode);
 
         bool hasEntry = false;
-        if (bibtexFile != NULL) {
-            for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
-                QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
+        if (bibtexFile != nullptr) {
+            for (const auto &element : const_cast<const File &>(*bibtexFile)) {
+                const QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
                 hasEntry |= publishEntry(entry);
             }
             delete bibtexFile;
         }
 
-        emit stoppedSearch(hasEntry ? resultNoError : resultUnspecifiedError);
-    } else
-        kDebug() << "url was" << reply->url().toString();
+        stopSearch(hasEntry ? resultNoError : resultUnspecifiedError);
+    }
+
+    refreshBusyProperty();
 }
 
 void OnlineSearchMRLookup::sanitizeEntry(QSharedPointer<Entry> entry)
 {
+    OnlineSearchAbstract::sanitizeEntry(entry);
+
     /// Rewrite 'fjournal' fields to become 'journal' fields
     /// (overwriting them if necessary)
-    const QString ftFJournal = QLatin1String("fjournal");
+    const QString ftFJournal = QStringLiteral("fjournal");
     if (entry->contains(ftFJournal)) {
         Value v = entry->value(ftFJournal);
         entry->remove(Entry::ftJournal);
@@ -144,7 +136,7 @@ void OnlineSearchMRLookup::sanitizeEntry(QSharedPointer<Entry> entry)
     /// Remove URL from entry if contains a DOI and the DOI field is present
     if (entry->contains(Entry::ftDOI) && entry->contains(Entry::ftUrl)) {
         Value v = entry->value(Entry::ftUrl);
-        if (v.containsPattern("http://dx.doi.org")) {
+        if (v.containsPattern(QStringLiteral("http://dx.doi.org"))) {
             entry->remove(Entry::ftUrl);
         }
     }
