@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,14 +12,14 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "tagmodel.h"
 
 #include <QHash>
 
-#include <KIcon>
+#include <QIcon>
 
 #include "zotero/tags.h"
 
@@ -27,13 +27,10 @@ using namespace Zotero;
 
 class Zotero::TagModel::Private
 {
-private:
-    // UNUSED Zotero::TagModel *p;
-
 public:
-    Private(Tags *t, Zotero::TagModel */* UNUSED parent*/)
-        : /* UNUSED p(parent),*/ tags(t) {
-        /// nothing
+    Private(Tags *t, Zotero::TagModel *parent)
+            : tags(t) {
+        Q_UNUSED(parent)
     }
 
     Tags *tags;
@@ -45,7 +42,7 @@ TagModel::TagModel(Tags *tags, QObject *parent)
         : QAbstractItemModel(parent), d(new Zotero::TagModel::Private(tags, this))
 {
     beginResetModel();
-    connect(tags, SIGNAL(finishedLoading()), this, SLOT(fetchingDone()));
+    connect(tags, &Tags::finishedLoading, this, &TagModel::fetchingDone);
 }
 
 QVariant TagModel::data(const QModelIndex &index, int role) const
@@ -68,7 +65,7 @@ QVariant TagModel::data(const QModelIndex &index, int role) const
             return data.value(dataKeys[index.row()]);
     } else if (role == Qt::DecorationRole) {
         if (index.column() == 0)
-            return KIcon("mail-tagged");
+            return QIcon::fromTheme(QStringLiteral("mail-tagged"));
     } else if (role == TagRole)
         return dataKeys[index.row()];
     else if (role == TagCountRole)
