@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "findduplicates.h"
@@ -20,18 +20,20 @@
 #include <typeinfo>
 
 #include <QLinkedList>
+#include <QProgressDialog>
+#include <QApplication>
+#include <QDate>
+#include <QRegularExpression>
 
-#include <KDebug>
-#include <KProgressDialog>
-#include <KLocale>
-#include <KApplication>
+#include <KLocalizedString>
 
 #include "file.h"
+#include "models/filemodel.h"
 #include "entry.h"
 
 EntryClique::EntryClique()
 {
-    // nothing
+    /// nothing
 }
 
 int EntryClique::entryCount() const
@@ -65,12 +67,12 @@ QList<QString> EntryClique::fieldList() const
     return valueMap.keys();
 }
 
-QList<Value> EntryClique::values(const QString &field) const
+QVector<Value> EntryClique::values(const QString &field) const
 {
     return valueMap[field];
 }
 
-QList<Value> &EntryClique::values(const QString &field)
+QVector<Value> &EntryClique::values(const QString &field)
 {
     return valueMap[field];
 }
@@ -81,12 +83,12 @@ Value EntryClique::chosenValue(const QString &field) const
     return chosenValueMap[field].first();
 }
 
-QList<Value> EntryClique::chosenValues(const QString &field) const
+QVector<Value> EntryClique::chosenValues(const QString &field) const
 {
     return chosenValueMap[field];
 }
 
-void EntryClique::setChosenValue(const QString &field, Value &value, ValueOperation valueOperation)
+void EntryClique::setChosenValue(const QString &field, const Value &value, ValueOperation valueOperation)
 {
     switch (valueOperation) {
     case SetValue: {
@@ -96,7 +98,7 @@ void EntryClique::setChosenValue(const QString &field, Value &value, ValueOperat
     }
     case AddValue: {
         QString text = PlainTextValue::text(value);
-        foreach (const Value &value, chosenValueMap[field])
+        for (const Value &value : const_cast<const QVector<Value> &>(chosenValueMap[field]))
             if (PlainTextValue::text(value) == text)
                 return;
         chosenValueMap[field] << value;
@@ -104,7 +106,7 @@ void EntryClique::setChosenValue(const QString &field, Value &value, ValueOperat
     }
     case RemoveValue: {
         QString text = PlainTextValue::text(value);
-        for (QList<Value>::Iterator it = chosenValueMap[field].begin(); it != chosenValueMap[field].end(); ++it)
+        for (QVector<Value>::Iterator it = chosenValueMap[field].begin(); it != chosenValueMap[field].end(); ++it)
             if (PlainTextValue::text(*it) == text) {
                 chosenValueMap[field].erase(it);
                 return;
@@ -126,18 +128,18 @@ void EntryClique::recalculateValueMap()
 
     /// go through each and every entry ...
     const QList<QSharedPointer<Entry> > el = entryList();
-    foreach (const QSharedPointer<Entry> &entry, el)
+    for (const auto &entry : el)
         if (isEntryChecked(entry)) {
 
             /// cover entry type
             Value v;
             v.append(QSharedPointer<VerbatimText>(new VerbatimText(entry->type())));
-            insertKeyValueToValueMap(QLatin1String("^type"), v, entry->type());
+            insertKeyValueToValueMap(QStringLiteral("^type"), v, entry->type(), Qt::CaseInsensitive /** entry types shall be compared case insensitive */);
 
             /// cover entry id
             v.clear();
             v.append(QSharedPointer<VerbatimText>(new VerbatimText(entry->id())));
-            insertKeyValueToValueMap(QLatin1String("^id"), v, entry->id());
+            insertKeyValueToValueMap(QStringLiteral("^id"), v, entry->id());
 
             /// go through each and every field of this entry
             for (Entry::ConstIterator fieldIt = entry->constBegin(); fieldIt != entry->constEnd(); ++fieldIt) {
@@ -146,7 +148,7 @@ void EntryClique::recalculateValueMap()
                 const Value fieldValue = fieldIt.value();
 
                 if (fieldName == Entry::ftKeywords || fieldName == Entry::ftUrl) {
-                    foreach (const QSharedPointer<ValueItem> &vi, fieldValue) {
+                    for (const auto &vi : fieldValue) {
                         const QString text = PlainTextValue::text(*vi);
                         Value v;
                         v << vi;
@@ -159,15 +161,15 @@ void EntryClique::recalculateValueMap()
             }
         }
 
-    QList<QString> fl = fieldList();
-    foreach (const QString &fieldName, fl)
+    const auto fl = fieldList();
+    for (const QString &fieldName : fl)
         if (valueMap[fieldName].count() < 2) {
             valueMap.remove(fieldName);
             chosenValueMap.remove(fieldName);
         }
 }
 
-void EntryClique::insertKeyValueToValueMap(const QString &fieldName, const Value &fieldValue, const QString &fieldValueText)
+void EntryClique::insertKeyValueToValueMap(const QString &fieldName, const Value &fieldValue, const QString &fieldValueText, const Qt::CaseSensitivity)
 {
     if (fieldValueText.isEmpty()) return;
 
@@ -176,8 +178,8 @@ void EntryClique::insertKeyValueToValueMap(const QString &fieldName, const Value
         /// to the current (as of fieldIt) value (to avoid duplicates)
 
         bool alreadyContained = false;
-        QList<Value> alternatives = valueMap[fieldName];
-        foreach (const Value &v, alternatives)
+        QVector<Value> alternatives = valueMap[fieldName];
+        for (const Value &v : const_cast<const QVector<Value> &>(alternatives))
             if (PlainTextValue::text(v) == fieldValueText) {
                 alreadyContained = true;
                 break;
@@ -188,10 +190,10 @@ void EntryClique::insertKeyValueToValueMap(const QString &fieldName, const Value
             valueMap[fieldName] = alternatives;
         }
     } else {
-        QList<Value> alternatives = valueMap[fieldName];
+        QVector<Value> alternatives = valueMap[fieldName];
         alternatives << fieldValue;
         valueMap.insert(fieldName, alternatives);
-        QList<Value> chosen;
+        QVector<Value> chosen;
         chosen << fieldValue;
         chosenValueMap.insert(fieldName, chosen);
     }
@@ -200,19 +202,17 @@ void EntryClique::insertKeyValueToValueMap(const QString &fieldName, const Value
 class FindDuplicates::FindDuplicatesPrivate
 {
 private:
-    // UNUSED FindDuplicates *p;
     const unsigned int maxDistance;
     int **d;
     static const int dsize;
 
 public:
-    bool gotCanceled;
     int sensitivity;
     QWidget *widget;
 
-    FindDuplicatesPrivate(FindDuplicates */* UNUSED parent*/, int sens, QWidget *w)
-        : /* UNUSED p(parent),*/ maxDistance(10000), gotCanceled(false), sensitivity(sens), widget(w) {
-        d = new int*[dsize];
+    FindDuplicatesPrivate(int sens, QWidget *w)
+            : maxDistance(10000), sensitivity(sens), widget(w == nullptr ? qApp->activeWindow() : w) {
+        d = new int *[dsize];
         for (int i = 0; i < dsize; ++i)
             d[i] = new int[dsize];
     }
@@ -230,7 +230,7 @@ public:
       * @return distance between both words
       */
     double levenshteinDistanceWord(const QString &s, const QString &t) {
-        int m = qMin(s.length(), dsize - 1), n = qMin(t.length(), dsize - 1);
+        const int m = qMin(s.length(), dsize - 1), n = qMin(t.length(), dsize - 1);
         if (m < 1 && n < 1) return 0.0;
         if (m < 1 || n < 1) return 1.0;
 
@@ -250,7 +250,7 @@ public:
 
         double result = d[m][n];
 
-        result = result / (double)qMax(m, n);
+        result = result / qMax(m, n);
         result *= result;
         return result;
     }
@@ -263,7 +263,7 @@ public:
      * @return distance between both sentences
      */
     double levenshteinDistance(const QStringList &s, const QStringList &t) {
-        int m = s.size(), n = t.size();
+        const int m = s.size(), n = t.size();
         if (m < 1 && n < 1) return 0.0;
         if (m < 1 || n < 1) return 1.0;
 
@@ -286,7 +286,7 @@ public:
         for (int i = 0; i <= m; ++i) delete[] d[i];
         delete [] d;
 
-        result = result / (double)qMax(m, n);
+        result = result / qMax(m, n);
 
         return result;
     }
@@ -301,7 +301,7 @@ public:
      * @return distance between both sentences
      */
     double levenshteinDistance(const QString &s, const QString &t) {
-        const QRegExp nonWordRegExp("[^a-z']+", Qt::CaseInsensitive);
+        static const QRegularExpression nonWordRegExp(QStringLiteral("[^a-z']+"), QRegularExpression::CaseInsensitiveOption);
         if (s.isEmpty() || t.isEmpty()) return 1.0;
         return levenshteinDistance(s.toLower().split(nonWordRegExp, QString::SkipEmptyParts), t.toLower().split(nonWordRegExp, QString::SkipEmptyParts));
     }
@@ -351,7 +351,7 @@ public:
          * Scale distance by maximum distance and round to int; result
          * will be in range 0 .. maxDistance.
          */
-        int distance = (unsigned int)(maxDistance * (titleDistance * 0.6 + authorDistance * 0.3 + yearDistance * 0.1) + 0.5);
+        int distance = static_cast<int>(maxDistance * (titleDistance * 0.6 + authorDistance * 0.3 + yearDistance * 0.1) + 0.5);
 
         return distance;
     }
@@ -362,9 +362,9 @@ const int FindDuplicates::FindDuplicatesPrivate::dsize = 32;
 
 
 FindDuplicates::FindDuplicates(QWidget *parent, int sensitivity)
-        : QObject(parent), d(new FindDuplicatesPrivate(this, sensitivity, parent))
+        : QObject(parent), d(new FindDuplicatesPrivate(sensitivity, parent))
 {
-    // nothing
+    /// nothing
 }
 
 FindDuplicates::~FindDuplicates()
@@ -372,25 +372,21 @@ FindDuplicates::~FindDuplicates()
     delete d;
 }
 
-bool FindDuplicates::findDuplicateEntries(File *file, QList<EntryClique *> &entryCliqueList)
+bool FindDuplicates::findDuplicateEntries(File *file, QVector<EntryClique *> &entryCliqueList)
 {
-    KProgressDialog *progressDlg = 0;
-    if (d->widget != NULL) {
-        KApplication::setOverrideCursor(Qt::WaitCursor);
-        progressDlg = new KProgressDialog(d->widget, i18n("Finding Duplicates"));
-        progressDlg->setModal(true);
-        progressDlg->setLabelText(i18n("Searching..."));
-        progressDlg->setMinimumWidth(d->widget->fontMetrics().averageCharWidth() * 48);
-        progressDlg->setAllowCancel(true);
-        connect(progressDlg, SIGNAL(cancelClicked()), this, SLOT(gotCanceled()));
-    }
+    QApplication::setOverrideCursor(Qt::WaitCursor);
+    QScopedPointer<QProgressDialog> progressDlg(new QProgressDialog(i18n("Searching ..."), i18n("Cancel"), 0, 100000 /* to be set later to actual value */, d->widget));
+    progressDlg->setModal(true);
+    progressDlg->setWindowTitle(i18n("Finding Duplicates"));
+    progressDlg->setMinimumWidth(d->widget->fontMetrics().averageCharWidth() * 48);
+    progressDlg->setAutoReset(false);
     entryCliqueList.clear();
-    d->gotCanceled = false;
 
     /// assemble list of entries only (ignoring comments, macros, ...)
-    QList<QSharedPointer<Entry> > listOfEntries;
-    for (File::ConstIterator it = file->constBegin(); it != file->constEnd(); ++it) {
-        QSharedPointer<Entry> e = (*it).dynamicCast<Entry>();
+    QVector<QSharedPointer<Entry> > listOfEntries;
+    listOfEntries.reserve(file->size());
+    for (const auto &element : const_cast<const File &>(*file)) {
+        QSharedPointer<Entry> e = element.dynamicCast<Entry>();
         if (!e.isNull() && !e->isEmpty())
             listOfEntries << e;
     }
@@ -398,37 +394,27 @@ bool FindDuplicates::findDuplicateEntries(File *file, QList<EntryClique *> &entr
     if (listOfEntries.isEmpty()) {
         /// no entries to compare found
         entryCliqueList.clear();
-        if (d->widget != NULL) {
-            progressDlg->deleteLater();
-            KApplication::restoreOverrideCursor();
-        }
-        return d->gotCanceled;
+        QApplication::restoreOverrideCursor();
+        return progressDlg->wasCanceled();
     }
 
     int curProgress = 0, maxProgress = listOfEntries.count() * (listOfEntries.count() - 1) / 2;
     int progressDelta = 1;
 
-    if (d->widget != NULL) {
-        progressDlg->progressBar()->setMaximum(maxProgress);
-        progressDlg->show();
-        progressDlg->setLabelText(i18n("Searching..."));
-    }
+    progressDlg->setMaximum(maxProgress);
+    progressDlg->show();
 
     emit maximumProgress(maxProgress);
 
     /// go through all entries ...
-    for (QList<QSharedPointer<Entry> >::ConstIterator eit = listOfEntries.constBegin(); eit != listOfEntries.constEnd(); ++eit) {
-        if (d->widget != NULL) {
-            KApplication::instance()->processEvents();
-        }
-        if (d->gotCanceled) {
+    for (const auto &entry : const_cast<const QVector<QSharedPointer<Entry> > &>(listOfEntries)) {
+        QApplication::instance()->processEvents();
+        if (progressDlg->wasCanceled()) {
             entryCliqueList.clear();
             break;
         }
 
-        if (d->widget != NULL) {
-            progressDlg->progressBar()->setValue(curProgress);
-        }
+        progressDlg->setValue(curProgress);
         emit currentProgress(curProgress);
         /// ... and find a "clique" of entries where it will match, i.e. distance is below sensitivity
 
@@ -436,48 +422,41 @@ bool FindDuplicates::findDuplicateEntries(File *file, QList<EntryClique *> &entr
         bool foundClique = false;
 
         /// go through all existing cliques
-        for (QList<EntryClique *>::Iterator cit = entryCliqueList.begin(); cit != entryCliqueList.end(); ++cit) {
+        for (QVector<EntryClique *>::Iterator cit = entryCliqueList.begin(); cit != entryCliqueList.end(); ++cit) {
             /// check distance between current entry and clique's first entry
-            if (d->entryDistance((*eit).data(), (*cit)->entryList().first().data()) < d->sensitivity) {
+            if (d->entryDistance(entry.data(), (*cit)->entryList().constFirst().data()) < d->sensitivity) {
                 /// if distance is below sensitivity, add current entry to clique
                 foundClique = true;
-                (*cit)->addEntry(*eit);
+                (*cit)->addEntry(entry);
                 break;
             }
 
-            if (d->widget != NULL) {
-                KApplication::instance()->processEvents();
-            }
-            if (d->gotCanceled) {
+            QApplication::instance()->processEvents();
+            if (progressDlg->wasCanceled()) {
                 entryCliqueList.clear();
                 break;
             }
         }
 
-        if (!d->gotCanceled && !foundClique) {
+        if (!progressDlg->wasCanceled() && !foundClique) {
             /// no clique matched to current entry, so create and add new clique
             /// consisting only of the current entry
             EntryClique *newClique = new EntryClique();
-            newClique->addEntry(*eit);
+            newClique->addEntry(entry);
             entryCliqueList << newClique;
         }
 
         curProgress += progressDelta;
         ++progressDelta;
-        if (d->widget != NULL) {
-            progressDlg->progressBar()->setValue(curProgress);
-        }
+        progressDlg->setValue(curProgress);
 
         emit currentProgress(curProgress);
     }
 
-    if (d->widget != NULL) {
-        progressDlg->progressBar()->setValue(progressDlg->progressBar()->maximum());
-        progressDlg->close();
-    }
+    progressDlg->setValue(progressDlg->maximum());
 
     /// remove cliques with only one element (nothing to merge here) from the list of cliques
-    for (QList<EntryClique *>::Iterator cit = entryCliqueList.begin(); cit != entryCliqueList.end();)
+    for (QVector<EntryClique *>::Iterator cit = entryCliqueList.begin(); cit != entryCliqueList.end();)
         if ((*cit)->entryCount() < 2) {
             EntryClique *ec = *cit;
             cit = entryCliqueList.erase(cit);
@@ -490,16 +469,8 @@ bool FindDuplicates::findDuplicateEntries(File *file, QList<EntryClique *> &entr
             ++cit;
         }
 
-    if (d->widget != NULL) {
-        progressDlg->deleteLater();
-        KApplication::restoreOverrideCursor();
-    }
-    return d->gotCanceled;
-}
-
-void FindDuplicates::gotCanceled()
-{
-    d->gotCanceled = true;
+    QApplication::restoreOverrideCursor();
+    return progressDlg->wasCanceled();
 }
 
 
@@ -508,26 +479,29 @@ MergeDuplicates::MergeDuplicates()
     /// nothing
 }
 
-bool MergeDuplicates::mergeDuplicateEntries(const QList<EntryClique *> &entryCliques, File *file)
+bool MergeDuplicates::mergeDuplicateEntries(const QVector<EntryClique *> &entryCliques, FileModel *fileModel)
 {
     bool didMerge = false;
 
-    foreach (EntryClique *entryClique, entryCliques) {
+    for (EntryClique *entryClique : entryCliques) {
         /// Avoid adding fields 20 lines below
         /// which have been remove (not added) 10 lines below
         QSet<QString> coveredFields;
 
         Entry *mergedEntry = new Entry(QString(), QString());
-        foreach (const QString &field, entryClique->fieldList()) {
+        const auto fieldList = entryClique->fieldList();
+        coveredFields.reserve(fieldList.size());
+        for (const auto &field : fieldList) {
             coveredFields << field;
-            if (field == QLatin1String("^id"))
+            if (field == QStringLiteral("^id"))
                 mergedEntry->setId(PlainTextValue::text(entryClique->chosenValue(field)));
-            else if (field == QLatin1String("^type"))
+            else if (field == QStringLiteral("^type"))
                 mergedEntry->setType(PlainTextValue::text(entryClique->chosenValue(field)));
             else {
                 Value combined;
-                foreach (const Value &v, entryClique->chosenValues(field)) {
-                    combined.merge(v);
+                const auto chosenValues = entryClique->chosenValues(field);
+                for (const Value &v : chosenValues) {
+                    combined.append(v);
                 }
                 if (!combined.isEmpty())
                     mergedEntry->insert(field, combined);
@@ -535,7 +509,9 @@ bool MergeDuplicates::mergeDuplicateEntries(const QList<EntryClique *> &entryCli
         }
 
         bool actuallyMerged = false;
-        foreach (const QSharedPointer<Entry> &entry, entryClique->entryList()) {
+        int preferredInsertionRow = -1;
+        const auto entryList = entryClique->entryList();
+        for (const auto &entry : entryList) {
             /// if merging entries with identical ids, the merged entry will not yet have an id (is null)
             if (mergedEntry->id().isEmpty())
                 mergedEntry->setId(entry->id());
@@ -553,14 +529,16 @@ bool MergeDuplicates::mergeDuplicateEntries(const QList<EntryClique *> &entryCli
                         mergedEntry->insert(it.key(), it.value());
                         coveredFields << it.key();
                     }
-                // TODO needs to be rewritten if File shall be based on QVector instead of QList
-                file->removeOne(QSharedPointer<Entry>(entry));
+                const int row = fileModel->row(entry);
+                if (preferredInsertionRow < 0) preferredInsertionRow = row;
+                fileModel->removeRow(row);
             }
         }
 
-        if (actuallyMerged)
-            file->append(QSharedPointer<Entry>(mergedEntry));
-        else
+        if (actuallyMerged) {
+            if (preferredInsertionRow < 0) preferredInsertionRow = fileModel->rowCount();
+            fileModel->insertRow(QSharedPointer<Entry>(mergedEntry), preferredInsertionRow);
+        } else
             delete mergedEntry;
         didMerge |= actuallyMerged;
     }
