@@ -1,5 +1,5 @@
 /*****************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de>   *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de>   *
  *                                                                           *
  *                                                                           *
  *   This program is free software; you can redistribute it and/or modify    *
@@ -13,7 +13,7 @@
  *   GNU General Public License for more details.                            *
  *                                                                           *
  *   You should have received a copy of the GNU General Public License       *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.    *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>.   *
  *****************************************************************************/
 
 #include "filesettings.h"
@@ -22,25 +22,29 @@
 #include <QCheckBox>
 
 #include <KComboBox>
-#include <KLocale>
+#include <KLocalizedString>
 
 #include "preferences.h"
 #include "guihelper.h"
 #include "italictextitemmodel.h"
 #include "fileview.h"
-#include "filemodel.h"
+#include "models/filemodel.h"
 #include "value.h"
-#include "iconvlatex.h"
 #include "file.h"
 #include "openfileinfo.h"
 
-FileSettings::FileSettings(OpenFileInfoManager *ofim, QWidget *parent)
-        : FileSettingsWidget(parent), m_fileView(NULL)
+FileSettings::FileSettings(QWidget *parent)
+        : FileSettingsWidget(parent), m_fileView(nullptr)
 {
     setEnabled(false);
 
-    connect(this, SIGNAL(widgetsChanged()), this, SLOT(widgetsChangedSlot()));
-    connect(ofim, SIGNAL(currentChanged(OpenFileInfo*,KService::Ptr)), this, SLOT(currentFileChangedSlot()));
+    connect(this, &FileSettings::widgetsChanged, this, &FileSettings::widgetsChangedSlot);
+
+    connect(&OpenFileInfoManager::instance(), &OpenFileInfoManager::currentChanged, this, &FileSettings::currentFileChangedSlot);
+    /// Monitoring file flag changes to get notified of
+    /// "Save As" operations where the file settings
+    /// may get changed (requires a reload of properties)
+    connect(&OpenFileInfoManager::instance(), &OpenFileInfoManager::flagsChanged, this, &FileSettings::resetToLoadedProperties);
 }
 
 void FileSettings::setFileView(FileView *fileView)
@@ -51,8 +55,8 @@ void FileSettings::setFileView(FileView *fileView)
 
 void FileSettings::widgetsChangedSlot()
 {
-    File *file = m_fileView != NULL && m_fileView->fileModel() != NULL ? m_fileView->fileModel()->bibliographyFile() : NULL;
-    if (file != NULL) {
+    File *file = m_fileView != nullptr && m_fileView->fileModel() != nullptr ? m_fileView->fileModel()->bibliographyFile() : nullptr;
+    if (file != nullptr) {
         saveProperties(file);
         /// Notify main view about change it its data
         m_fileView->externalModification();
@@ -60,7 +64,7 @@ void FileSettings::widgetsChangedSlot()
 }
 
 void FileSettings::currentFileChangedSlot() {
-    File *file = m_fileView != NULL && m_fileView->fileModel() != NULL ? m_fileView->fileModel()->bibliographyFile() : NULL;
+    File *file = m_fileView != nullptr && m_fileView->fileModel() != nullptr ? m_fileView->fileModel()->bibliographyFile() : nullptr;
     loadProperties(file);
-    setEnabled(file != NULL);
+    setEnabled(file != nullptr);
 }
