@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "mdiwidget.h"
@@ -27,53 +27,57 @@
 #include <QAbstractItemModel>
 #include <QSortFilterProxyModel>
 #include <QHeaderView>
+#include <QPushButton>
+#include <QIcon>
+#include <QDebug>
 
-#include <KDebug>
-#include <KPushButton>
-#include <KIcon>
-#include <KMessageBox>
-#include <kparts/part.h>
+#include <KParts/Part>
+#include <KParts/ReadOnlyPart>
 #include <KConfigGroup>
 #include <KSharedConfig>
+#include <KLocalizedString>
+#include <KMessageBox>
 
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
 #include "partwidget.h"
+#include "logging_program.h"
 
 class LRUItemModel : public QAbstractItemModel
 {
-private:
-    OpenFileInfoManager *ofim;
+    Q_OBJECT
 
 public:
     static const int URLRole = Qt::UserRole + 235;
     static const int SortRole = Qt::UserRole + 236;
 
-    LRUItemModel(OpenFileInfoManager *openFileInfoManager, QObject *parent = NULL)
-            : QAbstractItemModel(parent), ofim(openFileInfoManager) {
-        // nothing
+    LRUItemModel(QObject *parent = nullptr)
+            : QAbstractItemModel(parent)
+    {
+        /// nothing
     }
 
     void reset() {
-        QAbstractItemModel::reset();
+        beginResetModel();
+        endResetModel();
     }
 
-    int columnCount(const QModelIndex &) const {
+    int columnCount(const QModelIndex &) const override {
         return 3;
     }
 
-    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const {
-        OpenFileInfoManager::OpenFileInfoList ofiList = ofim->filteredItems(OpenFileInfo::RecentlyUsed);
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override {
+        OpenFileInfoManager::OpenFileInfoList ofiList = OpenFileInfoManager::instance().filteredItems(OpenFileInfo::RecentlyUsed);
         if (index.row() < ofiList.count()) {
             OpenFileInfo *ofiItem = ofiList[index.row()];
             if (index.column() == 0) {
                 if (role == Qt::DisplayRole || role == SortRole) {
-                    const KUrl url = ofiItem->url();
+                    const QUrl url = ofiItem->url();
                     const QString fileName = url.fileName();
-                    return fileName.isEmpty() ? squeeze_text(url.pathOrUrl(), 32) : fileName;
+                    return fileName.isEmpty() ? squeeze_text(url.url(QUrl::PreferLocalFile), 32) : fileName;
                 } else if (role == Qt::DecorationRole)
-                    return KIcon(ofiItem->mimeType().replace(QLatin1Char('/'), QLatin1Char('-')));
+                    return QIcon::fromTheme(ofiItem->mimeType().replace(QLatin1Char('/'), QLatin1Char('-')));
                 else if (role == Qt::ToolTipRole)
-                    return squeeze_text(ofiItem->url().pathOrUrl(), 64);
+                    return squeeze_text(ofiItem->url().url(QUrl::PreferLocalFile), 64);
             } else if (index.column() == 1) {
                 if (role == Qt::DisplayRole || role == Qt::ToolTipRole)
                     return ofiItem->lastAccess().toString(Qt::TextDate);
@@ -81,33 +85,33 @@ public:
                     return ofiItem->lastAccess().toTime_t();
             } else if (index.column() == 2) {
                 if (role == Qt::DisplayRole || role == Qt::ToolTipRole || role == SortRole)
-                    return ofiItem->url().pathOrUrl();
+                    return ofiItem->url().url(QUrl::PreferLocalFile);
             }
             if (role == URLRole)
-                return QVariant::fromValue<QUrl>(ofiItem->url());
+                return ofiItem->url();
         }
 
         return QVariant();
     }
 
-    QVariant headerData(int section, Qt::Orientation, int role = Qt::DisplayRole) const {
+    QVariant headerData(int section, Qt::Orientation, int role = Qt::DisplayRole) const override {
         if (role != Qt::DisplayRole || section > 2) return QVariant();
         else if (section == 0) return i18n("Filename");
         else if (section == 1) return i18n("Date/time of last use");
         else return i18n("Full filename");
     }
 
-    QModelIndex index(int row, int column, const QModelIndex &) const {
+    QModelIndex index(int row, int column, const QModelIndex &) const override {
         return createIndex(row, column, row);
     }
 
-    QModelIndex parent(const QModelIndex &) const {
+    QModelIndex parent(const QModelIndex &) const override {
         return QModelIndex();
     }
 
-    int rowCount(const QModelIndex &parent = QModelIndex()) const {
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override {
         if (parent == QModelIndex())
-            return ofim->filteredItems(OpenFileInfo::RecentlyUsed).count();
+            return OpenFileInfoManager::instance().filteredItems(OpenFileInfo::RecentlyUsed).count();
         else
             return 0;
     }
@@ -143,22 +147,22 @@ private:
         QLabel *label = new QLabel(i18n("<qt>Welcome to <b>KBibTeX</b></qt>"), welcomeWidget);
         layout->addWidget(label, 1, 2, 1, 3, Qt::AlignHCenter | Qt::AlignTop);
 
-        KPushButton *buttonNew = new KPushButton(KIcon("document-new"), i18n("New"), welcomeWidget);
+        QPushButton *buttonNew = new QPushButton(QIcon::fromTheme(QStringLiteral("document-new")), i18n("New"), welcomeWidget);
         layout->addWidget(buttonNew, 2, 2, 1, 1, Qt::AlignLeft | Qt::AlignBottom);
-        connect(buttonNew, SIGNAL(clicked()), p, SIGNAL(documentNew()));
+        connect(buttonNew, &QPushButton::clicked, p, &MDIWidget::documentNew);
 
-        KPushButton *buttonOpen = new KPushButton(KIcon("document-open"), i18n("Open..."), welcomeWidget);
+        QPushButton *buttonOpen = new QPushButton(QIcon::fromTheme(QStringLiteral("document-open")), i18n("Open..."), welcomeWidget);
         layout->addWidget(buttonOpen, 2, 4, 1, 1, Qt::AlignRight | Qt::AlignBottom);
-        connect(buttonOpen, SIGNAL(clicked()), p, SIGNAL(documentOpen()));
+        connect(buttonOpen, &QPushButton::clicked, p, &MDIWidget::documentOpen);
 
         label = new QLabel(i18n("List of recently used files:"), welcomeWidget);
         layout->addWidget(label, 3, 1, 1, 5, Qt::AlignLeft | Qt::AlignBottom);
         listLRU = new QTreeView(p);
         listLRU->setRootIsDecorated(false);
         listLRU->setSortingEnabled(true);
-        listLRU->header()->setResizeMode(QHeaderView::ResizeToContents);
+        listLRU->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
         layout->addWidget(listLRU, 4, 1, 1, 5);
-        connect(listLRU, SIGNAL(activated(QModelIndex)), p, SLOT(slotOpenLRU(QModelIndex)));
+        connect(listLRU, &QTreeView::activated, p, &MDIWidget::slotOpenLRU);
         label->setBuddy(listLRU);
 
         p->addWidget(welcomeWidget);
@@ -180,7 +184,6 @@ private:
 
 public:
     MDIWidget *p;
-    OpenFileInfoManager *ofim;
     OpenFileInfo *currentFile;
     QWidget *welcomeWidget;
     QSignalMapper signalMapperCompleted;
@@ -188,30 +191,29 @@ public:
     KSharedConfigPtr config;
 
     MDIWidgetPrivate(MDIWidget *parent)
-            : p(parent), ofim(new OpenFileInfoManager(parent)), currentFile(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))) {
+            : p(parent), currentFile(nullptr), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))) {
         createWelcomeWidget();
 
-        modelLRU = new LRUItemModel(ofim, listLRU);
+        modelLRU = new LRUItemModel(listLRU);
         QSortFilterProxyModel *sfpm = new QSortFilterProxyModel(listLRU);
         sfpm->setSourceModel(modelLRU);
         sfpm->setSortRole(LRUItemModel::SortRole);
         listLRU->setModel(sfpm);
 
-        connect(&signalMapperCompleted, SIGNAL(mapped(QObject*)), p, SLOT(slotCompleted(QObject*)));
+        connect(&signalMapperCompleted, static_cast<void(QSignalMapper::*)(QObject *)>(&QSignalMapper::mapped), p, &MDIWidget::slotCompleted);
 
         restoreColumnsState();
     }
 
     ~MDIWidgetPrivate() {
         saveColumnsState();
-        delete ofim;
         delete welcomeWidget;
     }
 
     void addToMapper(OpenFileInfo *openFileInfo) {
         KParts::ReadOnlyPart *part = openFileInfo->part(p);
         signalMapperCompleted.setMapping(part, openFileInfo);
-        connect(part, SIGNAL(completed()), &signalMapperCompleted, SLOT(map()));
+        connect(part, static_cast<void(KParts::ReadOnlyPart::*)()>(&KParts::ReadOnlyPart::completed), &signalMapperCompleted, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
     }
 
     void updateLRU() {
@@ -219,13 +221,13 @@ public:
     }
 };
 
-const QString MDIWidget::MDIWidgetPrivate::configGroupName = QLatin1String("WelcomeWidget");
-const QString MDIWidget::MDIWidgetPrivate::configHeaderState = QLatin1String("LRUlistHeaderState");
+const QString MDIWidget::MDIWidgetPrivate::configGroupName = QStringLiteral("WelcomeWidget");
+const QString MDIWidget::MDIWidgetPrivate::configHeaderState = QStringLiteral("LRUlistHeaderState");
 
 MDIWidget::MDIWidget(QWidget *parent)
         : QStackedWidget(parent), d(new MDIWidgetPrivate(this))
 {
-    connect(d->ofim, SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)), this, SLOT(slotStatusFlagsChanged(OpenFileInfo::StatusFlags)));
+    connect(&OpenFileInfoManager::instance(), &OpenFileInfoManager::flagsChanged, this, &MDIWidget::slotStatusFlagsChanged);
 }
 
 MDIWidget::~MDIWidget()
@@ -233,26 +235,41 @@ MDIWidget::~MDIWidget()
     delete d;
 }
 
+/// Clazy warns: "Missing reference on non-trivial type" for argument 'servicePtr',
+/// but type 'KService::Ptr' is actually a pointer (QExplicitlySharedDataPointer).
 void MDIWidget::setFile(OpenFileInfo *openFileInfo, KService::Ptr servicePtr)
 {
-    FileView *oldEditor = NULL;
-    bool hasChanged = true;
-
-    KParts::Part *part = openFileInfo == NULL ? NULL : openFileInfo->part(this, servicePtr);
-    QWidget *widget = d->welcomeWidget;
-    if (part != NULL) {
+    KParts::Part *part = openFileInfo == nullptr ? nullptr : openFileInfo->part(this, servicePtr);
+    /// 'part' will only be NULL if no OpenFileInfo object was given,
+    /// or if the OpenFileInfo could not locate a KPart
+    QWidget *widget = d->welcomeWidget; ///< by default use Welcome widget
+    if (part != nullptr) {
+        /// KPart object was located, use its object (instead of Welcome widget)
         widget = part->widget();
-    } else if (openFileInfo != NULL) {
-        KMessageBox::error(this, i18n("No part available for file '%1'.", openFileInfo->url().fileName()), i18n("No part available"));
-        d->ofim->close(openFileInfo);
+    } else if (openFileInfo != nullptr) {
+        /// A valid OpenFileInfo was given, but no KPart could be located
+
+        OpenFileInfoManager::instance().close(openFileInfo); // FIXME does not close correctly if file is new
+        const QString filename = openFileInfo->url().fileName();
+        if (filename.isEmpty())
+            KMessageBox::error(this, i18n("No part available for file of mime type '%1'.", openFileInfo->mimeType()), i18n("No Part Available"));
+        else
+            KMessageBox::error(this, i18n("No part available for file '%1'.", filename), i18n("No Part Available"));
         return;
     }
 
+    FileView *oldEditor = nullptr;
+    bool hasChanged = true;
     if (indexOf(widget) >= 0) {
+        /// Chosen widget is already known (Welcome widget or a previously used KPart)
+
+        /// In case previous (still current) widget was a KBibTeX Part, remember its editor
         PartWidget *currentPartWidget = qobject_cast<PartWidget *>(currentWidget());
-        oldEditor = currentPartWidget == NULL ? NULL : currentPartWidget->fileView();
+        oldEditor = currentPartWidget == nullptr ? nullptr : currentPartWidget->fileView();
+        /// Record if set widget is different from previous (still current) widget
         hasChanged = widget != currentWidget();
-    } else if (openFileInfo != NULL) {
+    } else if (openFileInfo != nullptr) {
+        /// Widget was not known previously, but a valid (new?) OpenFileInfo was given
         addWidget(widget);
         d->addToMapper(openFileInfo);
     }
@@ -260,25 +277,30 @@ void MDIWidget::setFile(OpenFileInfo *openFileInfo, KService::Ptr servicePtr)
     d->currentFile = openFileInfo;
 
     if (hasChanged) {
-        PartWidget *newPartWidget = qobject_cast<PartWidget *>(widget);
-        FileView *newEditor = newPartWidget == NULL ? NULL : newPartWidget->fileView();
+        /// This signal gets forwarded to KParts::MainWindow::createGUI(KParts::Part*)
         emit activePartChanged(part);
-        emit documentSwitch(oldEditor, newEditor);
+        /// If new widget comes from a KBibTeX Part, retrieve its editor
+        PartWidget *newPartWidget = qobject_cast<PartWidget *>(widget);
+        FileView *newEditor = newPartWidget == nullptr ? nullptr : newPartWidget->fileView();
+        emit documentSwitched(oldEditor, newEditor);
     }
 
-    if (openFileInfo != NULL) {
-        KUrl url = openFileInfo->url();
+    /// Notify main window about a change of current file,
+    /// so that the title may contain the current file's URL.
+    /// This signal will be connected to KMainWindow::setCaption.
+    if (openFileInfo != nullptr) {
+        QUrl url = openFileInfo->url();
         if (url.isValid())
-            emit setCaption(QString(QLatin1String("%1 [%2]")).arg(openFileInfo->shortCaption(), squeeze_text(openFileInfo->fullCaption(), 64)));
+            emit setCaption(QString(QStringLiteral("%1 [%2]")).arg(openFileInfo->shortCaption(), squeeze_text(openFileInfo->fullCaption(), 64)));
         else
             emit setCaption(openFileInfo->shortCaption());
     } else
-        emit setCaption("");
+        emit setCaption(QString());
 }
 
 FileView *MDIWidget::fileView()
 {
-    OpenFileInfo *ofi = d->ofim->currentFile();
+    OpenFileInfo *ofi = OpenFileInfoManager::instance().currentFile();
     return qobject_cast<PartWidget *>(ofi->part(this)->widget())->fileView();
 }
 
@@ -287,25 +309,20 @@ OpenFileInfo *MDIWidget::currentFile()
     return d->currentFile;
 }
 
-OpenFileInfoManager *MDIWidget::getOpenFileInfoManager()
-{
-    return d->ofim;
-}
-
 void MDIWidget::slotCompleted(QObject *obj)
 {
     OpenFileInfo *ofi = static_cast<OpenFileInfo *>(obj);
-    KUrl oldUrl = ofi->url();
-    KUrl newUrl = ofi->part(this)->url();
+    QUrl oldUrl = ofi->url();
+    QUrl newUrl = ofi->part(this)->url();
 
-    if (!oldUrl.equals(newUrl)) {
-        kDebug() << "Url changed from " << oldUrl.pathOrUrl() << " to " << newUrl.pathOrUrl() << endl;
-        d->ofim->changeUrl(ofi, newUrl);
+    if (oldUrl != newUrl) {
+        qCDebug(LOG_KBIBTEX_PROGRAM) << "Url changed from " << oldUrl.url(QUrl::PreferLocalFile) << " to " << newUrl.url(QUrl::PreferLocalFile) << endl;
+        OpenFileInfoManager::instance().changeUrl(ofi, newUrl);
 
         /// completely opened or saved files should be marked as "recently used"
         ofi->addFlags(OpenFileInfo::RecentlyUsed);
 
-        emit setCaption(QString(QLatin1String("%1 [%2]")).arg(ofi->shortCaption(), squeeze_text(ofi->fullCaption(), 64)));
+        emit setCaption(QString(QStringLiteral("%1 [%2]")).arg(ofi->shortCaption(), squeeze_text(ofi->fullCaption(), 64)));
     }
 }
 
@@ -321,3 +338,5 @@ void MDIWidget::slotOpenLRU(const QModelIndex &index)
     if (url.isValid())
         emit documentOpenURL(url);
 }
+
+#include "mdiwidget.moc"
